/* Excel specific API library */
/* Version: 16.0.6216.3006 */
/*
	Copyright (c) Microsoft Corporation.  All rights reserved.
*/

/*
	Your use of this file is governed by the Microsoft Services Agreement http://go.microsoft.com/fwlink/?LinkId=266419.
*/

/*
* @overview es6-promise - a tiny implementation of Promises/A+.
* @copyright Copyright (c) 2014 Yehuda Katz, Tom Dale, Stefan Penner and contributors (Conversion to ES6 API by Jake Archibald)
* @license   Licensed under MIT license
*            See https://raw.githubusercontent.com/jakearchibald/es6-promise/master/LICENSE
* @version   2.3.0
*/

var __extends=this.__extends||function(n,t){function r(){this.constructor=n}for(var i in t)t.hasOwnProperty(i)&&(n[i]=t[i]);r.prototype=t.prototype;n.prototype=new r},OsfMsAjaxFactory,OSF,msAjaxCDNPath,OSFRichclient,OfficeExt,OSFLog,Logger,OSFAppTelemetry,OfficeExtension,Excel;(function(n){var t=function(){function t(){}var i=null,n=!0;return t.prototype.isMsAjaxLoaded=function(){var t="function",i="undefined";return typeof Sys!==i&&typeof Type!==i&&Sys.StringBuilder&&typeof Sys.StringBuilder===t&&Type.registerNamespace&&typeof Type.registerNamespace===t&&Type.registerClass&&typeof Type.registerClass===t&&typeof Function._validateParams===t?n:!1},t.prototype.loadMsAjaxFull=function(n){var t=(window.location.protocol.toLowerCase()==="https:"?"https:":"http:")+"//ajax.aspnetcdn.com/ajax/3.5/MicrosoftAjax.js";OSF.OUtil.loadScript(t,n)},Object.defineProperty(t.prototype,"msAjaxError",{get:function(){var n=this;return n._msAjaxError==i&&n.isMsAjaxLoaded()&&(n._msAjaxError=Error),n._msAjaxError},set:function(n){this._msAjaxError=n},enumerable:n,configurable:n}),Object.defineProperty(t.prototype,"msAjaxSerializer",{get:function(){var n=this;return n._msAjaxSerializer==i&&n.isMsAjaxLoaded()&&(n._msAjaxSerializer=Sys.Serialization.JavaScriptSerializer),n._msAjaxSerializer},set:function(n){this._msAjaxSerializer=n},enumerable:n,configurable:n}),Object.defineProperty(t.prototype,"msAjaxString",{get:function(){var n=this;return n._msAjaxString==i&&n.isMsAjaxLoaded()&&(n._msAjaxSerializer=String),n._msAjaxString},set:function(n){this._msAjaxString=n},enumerable:n,configurable:n}),Object.defineProperty(t.prototype,"msAjaxDebug",{get:function(){var n=this;return n._msAjaxDebug==i&&n.isMsAjaxLoaded()&&(n._msAjaxDebug=Sys.Debug),n._msAjaxDebug},set:function(n){this._msAjaxDebug=n},enumerable:n,configurable:n}),t}();n.MicrosoftAjaxFactory=t})(OfficeExt||(OfficeExt={}));OsfMsAjaxFactory=new OfficeExt.MicrosoftAjaxFactory;OSF=OSF||{},function(n){var t=function(){function n(n){this._internalStorage=n}return n.prototype.getItem=function(n){try{return this._internalStorage&&this._internalStorage.getItem(n)}catch(t){return null}},n.prototype.setItem=function(n,t){try{this._internalStorage&&this._internalStorage.setItem(n,t)}catch(i){}},n.prototype.clear=function(){try{this._internalStorage&&this._internalStorage.clear()}catch(n){}},n.prototype.removeItem=function(n){try{this._internalStorage&&this._internalStorage.removeItem(n)}catch(t){}},n.prototype.getKeysWithPrefix=function(n){var r=[],u,t,i;try{for(u=this._internalStorage&&this._internalStorage.length||0,t=0;t<u;t++)i=this._internalStorage.key(t),i.indexOf(n)===0&&r.push(i)}catch(f){}return r},n}();n.SafeStorage=t}(OfficeExt||(OfficeExt={}));OSF.OUtil=function(){function k(){var n=o*Math.random();return n^=r^(new Date).getMilliseconds()<<Math.floor(Math.random()*21),n.toString(16)}function d(){if(!l){try{var n=window.sessionStorage}catch(i){n=t}l=new OfficeExt.SafeStorage(n)}return l}var u="on",v="configurable",y="writable",f="enumerable",e="undefined",i=!0,n=!1,o=2147483647,t=null,s=-1,p="&_xdm_Info=",w="&_serializer_version=",b="_xdm_",g="_serializer_version=",h="#",c={},nt=3e4,l=t,a=t,r=(new Date).getTime();return{set_entropy:function(n){var t,u,i;if(typeof n=="string")for(t=0;t<n.length;t+=4){for(u=0,i=0;i<4&&t+i<n.length;i++)u=(u<<8)+n.charCodeAt(t+i);r^=u}else r^=typeof n=="number"?n:o*Math.random();r&=o},extend:function(n,t){var i=function(){};i.prototype=t.prototype;n.prototype=new i;n.prototype.constructor=n;n.uber=t.prototype;t.prototype.constructor===Object.prototype.constructor&&(t.prototype.constructor=t)},setNamespace:function(n,t){t&&n&&!t[n]&&(t[n]={})},unsetNamespace:function(n,t){t&&n&&t[n]&&delete t[n]},loadScript:function(r,u,f){var s,e,o,h,l;r&&u&&(s=window.document,e=c[r],e?e.loaded?u():e.pendingCallbacks.push(u):(o=s.createElement("script"),o.type="text/javascript",e={loaded:n,pendingCallbacks:[u],timer:t},c[r]=e,h=function(){var r,n,u;for(e.timer!=t&&(clearTimeout(e.timer),delete e.timer),e.loaded=i,r=e.pendingCallbacks.length,n=0;n<r;n++)u=e.pendingCallbacks.shift(),u()},l=function(){var i,n,u;for(delete c[r],e.timer!=t&&(clearTimeout(e.timer),delete e.timer),i=e.pendingCallbacks.length,n=0;n<i;n++)u=e.pendingCallbacks.shift(),u()},o.readyState?o.onreadystatechange=function(){(o.readyState=="loaded"||o.readyState=="complete")&&(o.onreadystatechange=t,h())}:o.onload=h,o.onerror=l,f=f||nt,e.timer=setTimeout(l,f),o.src=r,s.getElementsByTagName("head")[0].appendChild(o)))},loadCSS:function(n){if(n){var i=window.document,t=i.createElement("link");t.type="text/css";t.rel="stylesheet";t.href=n;i.getElementsByTagName("head")[0].appendChild(t)}},parseEnum:function(n,t){var i=t[n.trim()];if(typeof i==e){OsfMsAjaxFactory.msAjaxDebug.trace("invalid enumeration string:"+n);throw OsfMsAjaxFactory.msAjaxError.argument("str");}return i},delayExecutionAndCache:function(){var n={calc:arguments[0]};return function(){return n.calc&&(n.val=n.calc.apply(this,arguments),delete n.calc),n.val}},getUniqueId:function(){return s=s+1,s.toString()},formatString:function(){var n=arguments,t=n[0];return t.replace(/{(\d+)}/gm,function(t,i){var r=parseInt(i,10)+1;return n[r]===undefined?"{"+i+"}":n[r]})},generateConversationId:function(){return[k(),k(),(new Date).getTime().toString()].join("_")},getFrameNameAndConversationId:function(n,t){var i=b+n+this.generateConversationId();return t.setAttribute("name",i),this.generateConversationId()},addXdmInfoAsHash:function(n,t){return OSF.OUtil.addInfoAsHash(n,p,t)},addSerializerVersionAsHash:function(n,t){return OSF.OUtil.addInfoAsHash(n,w,t)},addInfoAsHash:function(n,t,i){n=n.trim()||"";var r=n.split(h),u=r.shift(),f=r.join(h);return[u,h,f,t,i].join("")},parseXdmInfo:function(n){return OSF.OUtil.parseXdmInfoWithGivenFragment(n,window.location.hash)},parseXdmInfoWithGivenFragment:function(n,t){return OSF.OUtil.parseInfoWithGivenFragment(p,b,n,t)},parseSerializerVersion:function(n){return OSF.OUtil.parseSerializerVersionWithGivenFragment(n,window.location.hash)},parseSerializerVersionWithGivenFragment:function(n,t){return parseInt(OSF.OUtil.parseInfoWithGivenFragment(w,g,n,t))},parseInfoWithGivenFragment:function(n,i,r,u){var s=u.split(n),f=s.length>1?s[s.length-1]:t,h=d(),e,o,c;return!r&&h&&(e=window.name.indexOf(i),e>-1&&(o=window.name.indexOf(";",e),o==-1&&(o=window.name.length),c=window.name.substring(e,o),f?h.setItem(c,f):f=h.getItem(c))),f},getConversationId:function(){var i=window.location.search,n=t,r;return i&&(r=i.indexOf("&"),n=r>0?i.substring(1,r):i.substr(1),n&&n.charAt(n.length-1)==="="&&(n=n.substring(0,n.length-1),n&&(n=decodeURIComponent(n)))),n},getInfoItems:function(n){var t=n.split("$");return typeof t[1]==e&&(t=n.split("|")),t},getConversationUrl:function(){var t="",r=OSF.OUtil.parseXdmInfo(i),n;return r&&(n=OSF.OUtil.getInfoItems(r),n!=undefined&&n.length>=3&&(t=n[2])),t},validateParamObject:function(t,r){var u=Function._validateParams(arguments,[{name:"params",type:Object,mayBeNull:n},{name:"expectedProperties",type:Object,mayBeNull:n},{name:"callback",type:Function,mayBeNull:i}]),f;if(u)throw u;for(f in r)if(u=Function._validateParameter(t[f],r[f],f),u)throw u;},writeProfilerMark:function(n){window.msWriteProfilerMark&&(window.msWriteProfilerMark(n),OsfMsAjaxFactory.msAjaxDebug.trace(n))},outputDebug:function(n){typeof Sys!==e&&Sys&&Sys.Debug&&OsfMsAjaxFactory.msAjaxDebug.trace(n)},defineNondefaultProperty:function(n,t,r,u){var e,f;r=r||{};for(e in u)f=u[e],r[f]==undefined&&(r[f]=i);return Object.defineProperty(n,t,r),n},defineNondefaultProperties:function(n,t,i){t=t||{};for(var r in t)OSF.OUtil.defineNondefaultProperty(n,r,t[r],i);return n},defineEnumerableProperty:function(n,t,i){return OSF.OUtil.defineNondefaultProperty(n,t,i,[f])},defineEnumerableProperties:function(n,t){return OSF.OUtil.defineNondefaultProperties(n,t,[f])},defineMutableProperty:function(n,t,i){return OSF.OUtil.defineNondefaultProperty(n,t,i,[y,f,v])},defineMutableProperties:function(n,t){return OSF.OUtil.defineNondefaultProperties(n,t,[y,f,v])},finalizeProperties:function(t,r){var e,u;r=r||{};for(var o=Object.getOwnPropertyNames(t),s=o.length,f=0;f<s;f++)e=o[f],u=Object.getOwnPropertyDescriptor(t,e),u.get||u.set||(u.writable=r.writable||n),u.configurable=r.configurable||n,u.enumerable=r.enumerable||i,Object.defineProperty(t,e,u);return t},mapList:function(n,t){var i=[],r;if(n)for(r in n)i.push(t(n[r]));return i},listContainsKey:function(t,r){for(var u in t)if(r==u)return i;return n},listContainsValue:function(t,r){for(var u in t)if(r==t[u])return i;return n},augmentList:function(n,t){var r=n.push?function(t,i){n.push(i)}:function(t,i){n[t]=i};for(var i in t)r(i,t[i])},redefineList:function(n,t){var r,i;for(r in n)delete n[r];for(i in t)n[i]=t[i]},isArray:function(n){return Object.prototype.toString.apply(n)==="[object Array]"},isFunction:function(n){return Object.prototype.toString.apply(n)==="[object Function]"},isDate:function(n){return Object.prototype.toString.apply(n)==="[object Date]"},addEventListener:function(t,i,r){t.addEventListener?t.addEventListener(i,r,n):Sys.Browser.agent===Sys.Browser.InternetExplorer&&t.attachEvent?t.attachEvent(u+i,r):t[u+i]=r},removeEventListener:function(i,r,f){i.removeEventListener?i.removeEventListener(r,f,n):Sys.Browser.agent===Sys.Browser.InternetExplorer&&i.detachEvent?i.detachEvent(u+r,f):i[u+r]=t},xhrGet:function(n,t,r){var u;try{u=new XMLHttpRequest;u.onreadystatechange=function(){u.readyState==4&&(u.status==200?t(u.responseText):r(u.status))};u.open("GET",n,i);u.send()}catch(f){r(f)}},encodeBase64:function(n){var h;if(!n)return n;var a="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",l=[],i=[],o=0,c,e,s,r,u,f,t,v=n.length;do for(c=n.charCodeAt(o++),e=n.charCodeAt(o++),s=n.charCodeAt(o++),t=0,r=c&255,u=c>>8,f=e&255,i[t++]=r>>2,i[t++]=(r&3)<<4|u>>4,i[t++]=(u&15)<<2|f>>6,i[t++]=f&63,isNaN(e)||(r=e>>8,u=s&255,f=s>>8,i[t++]=r>>2,i[t++]=(r&3)<<4|u>>4,i[t++]=(u&15)<<2|f>>6,i[t++]=f&63),isNaN(e)?i[t-1]=64:isNaN(s)&&(i[t-2]=64,i[t-1]=64),h=0;h<t;h++)l.push(a.charAt(i[h]));while(o<v);return l.join("")},getSessionStorage:function(){return d()},getLocalStorage:function(){if(!a){try{var n=window.localStorage}catch(i){n=t}a=new OfficeExt.SafeStorage(n)}return a},convertIntToCssHexColor:function(n){return"#"+(Number(n)+16777216).toString(16).slice(-6)},attachClickHandler:function(n,t){n.onclick=function(){t()};n.ontouchend=function(n){t();n.preventDefault()}},getQueryStringParamValue:function(t,i){var u=Function._validateParams(arguments,[{name:"queryString",type:String,mayBeNull:n},{name:"paramName",type:String,mayBeNull:n}]),r;return u?(OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: Parameters cannot be null."),""):(r=new RegExp("[\\?&]"+i+"=([^&#]*)","i"),!r.test(t))?(OsfMsAjaxFactory.msAjaxDebug.trace("OSF_Outil_getQueryStringParamValue: The parameter is not found."),""):r.exec(t)[1]},isiOS:function(){return window.navigator.userAgent.match(/(iPad|iPhone|iPod)/g)?i:n},shallowCopy:function(n){var i=n.constructor();for(var t in n)n.hasOwnProperty(t)&&(i[t]=n[t]);return i}}}();OSF.OUtil.Guid=function(){var n=["0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f"];return{generateNewGuid:function(){for(var i="",r=(new Date).getTime(),t=0;t<32&&r>0;t++)(t==8||t==12||t==16||t==20)&&(i+="-"),i+=n[r%16],r=Math.floor(r/16);for(;t<32;t++)(t==8||t==12||t==16||t==20)&&(i+="-"),i+=n[Math.floor(Math.random()*16)];return i}}}();window.OSF=OSF;OSF.OUtil.setNamespace("OSF",window);OSF.AppName={Unsupported:0,Excel:1,Word:2,PowerPoint:4,Outlook:8,ExcelWebApp:16,WordWebApp:32,OutlookWebApp:64,Project:128,AccessWebApp:256,PowerpointWebApp:512,ExcelIOS:1024,Sway:2048,WordIOS:4096,PowerPointIOS:8192,Access:16384,Lync:32768,OutlookIOS:65536,OneNoteWebApp:131072};OSF.InternalPerfMarker={DataCoercionBegin:"Agave.HostCall.CoerceDataStart",DataCoercionEnd:"Agave.HostCall.CoerceDataEnd"};OSF.HostCallPerfMarker={IssueCall:"Agave.HostCall.IssueCall",ReceiveResponse:"Agave.HostCall.ReceiveResponse",RuntimeExceptionRaised:"Agave.HostCall.RuntimeExecptionRaised"};OSF.AgaveHostAction={Select:0,UnSelect:1,CancelDialog:2,InsertAgave:3,CtrlF6In:4,CtrlF6Exit:5,CtrlF6ExitShift:6,SelectWithError:7,NotifyHostError:8};OSF.SharedConstants={NotificationConversationIdSuffix:"_ntf"};OSF.OfficeAppContext=function(n,t,i,r,u,f,e,o,s,h,c,l,a,v,y,p,w){var b=this;b._id=n;b._appName=t;b._appVersion=i;b._appUILocale=r;b._dataLocale=u;b._docUrl=f;b._clientMode=e;b._settings=o;b._reason=s;b._osfControlType=h;b._eToken=c;b._correlationId=l;b._appInstanceId=a;b._touchEnabled=v;b._commerceAllowed=y;b._appMinorVersion=p;b._requirementMatrix=w;b.get_id=function(){return this._id};b.get_appName=function(){return this._appName};b.get_appVersion=function(){return this._appVersion};b.get_appUILocale=function(){return this._appUILocale};b.get_dataLocale=function(){return this._dataLocale};b.get_docUrl=function(){return this._docUrl};b.get_clientMode=function(){return this._clientMode};b.get_bindings=function(){return this._bindings};b.get_settings=function(){return this._settings};b.get_reason=function(){return this._reason};b.get_osfControlType=function(){return this._osfControlType};b.get_eToken=function(){return this._eToken};b.get_correlationId=function(){return this._correlationId};b.get_appInstanceId=function(){return this._appInstanceId};b.get_touchEnabled=function(){return this._touchEnabled};b.get_commerceAllowed=function(){return this._commerceAllowed};b.get_appMinorVersion=function(){return this._appMinorVersion};b.get_requirementMatrix=function(){return this._requirementMatrix}};OSF.OsfControlType={DocumentLevel:0,ContainerLevel:1};OSF.ClientMode={ReadOnly:0,ReadWrite:1};OSF.OUtil.setNamespace("Microsoft",window);OSF.OUtil.setNamespace("Office",Microsoft);OSF.OUtil.setNamespace("Client",Microsoft.Office);OSF.OUtil.setNamespace("WebExtension",Microsoft.Office);Microsoft.Office.WebExtension.InitializationReason={Inserted:"inserted",DocumentOpened:"documentOpened"};Microsoft.Office.WebExtension.ValueFormat={Unformatted:"unformatted",Formatted:"formatted"};Microsoft.Office.WebExtension.FilterType={All:"all"};Microsoft.Office.WebExtension.Parameters={BindingType:"bindingType",CoercionType:"coercionType",ValueFormat:"valueFormat",FilterType:"filterType",Columns:"columns",SampleData:"sampleData",GoToType:"goToType",SelectionMode:"selectionMode",Id:"id",PromptText:"promptText",ItemName:"itemName",FailOnCollision:"failOnCollision",StartRow:"startRow",StartColumn:"startColumn",RowCount:"rowCount",ColumnCount:"columnCount",Callback:"callback",AsyncContext:"asyncContext",Data:"data",Rows:"rows",OverwriteIfStale:"overwriteIfStale",FileType:"fileType",EventType:"eventType",Handler:"handler",SliceSize:"sliceSize",SliceIndex:"sliceIndex",ActiveView:"activeView",Status:"status",Xml:"xml",Namespace:"namespace",Prefix:"prefix",XPath:"xPath",ImageLeft:"imageLeft",ImageTop:"imageTop",ImageWidth:"imageWidth",ImageHeight:"imageHeight",TaskId:"taskId",FieldId:"fieldId",FieldValue:"fieldValue",ServerUrl:"serverUrl",ListName:"listName",ResourceId:"resourceId",ViewType:"viewType",ViewName:"viewName",GetRawValue:"getRawValue",CellFormat:"cellFormat",TableOptions:"tableOptions",TaskIndex:"taskIndex",ResourceIndex:"resourceIndex"};OSF.OUtil.setNamespace("DDA",OSF);OSF.DDA.DocumentMode={ReadOnly:1,ReadWrite:0};OSF.DDA.PropertyDescriptors={AsyncResultStatus:"AsyncResultStatus"};OSF.DDA.EventDescriptors={};OSF.DDA.ListDescriptors={};OSF.DDA.getXdmEventName=function(n,t){return t==Microsoft.Office.WebExtension.EventType.BindingSelectionChanged||t==Microsoft.Office.WebExtension.EventType.BindingDataChanged?n+"_"+t:t};OSF.DDA.MethodDispId={dispidMethodMin:64,dispidGetSelectedDataMethod:64,dispidSetSelectedDataMethod:65,dispidAddBindingFromSelectionMethod:66,dispidAddBindingFromPromptMethod:67,dispidGetBindingMethod:68,dispidReleaseBindingMethod:69,dispidGetBindingDataMethod:70,dispidSetBindingDataMethod:71,dispidAddRowsMethod:72,dispidClearAllRowsMethod:73,dispidGetAllBindingsMethod:74,dispidLoadSettingsMethod:75,dispidSaveSettingsMethod:76,dispidGetDocumentCopyMethod:77,dispidAddBindingFromNamedItemMethod:78,dispidAddColumnsMethod:79,dispidGetDocumentCopyChunkMethod:80,dispidReleaseDocumentCopyMethod:81,dispidNavigateToMethod:82,dispidGetActiveViewMethod:83,dispidGetDocumentThemeMethod:84,dispidGetOfficeThemeMethod:85,dispidGetFilePropertiesMethod:86,dispidClearFormatsMethod:87,dispidSetTableOptionsMethod:88,dispidSetFormatsMethod:89,dispidExecuteRichApiRequestMethod:93,dispidAppCommandInvocationCompletedMethod:94,dispidAddDataPartMethod:128,dispidGetDataPartByIdMethod:129,dispidGetDataPartsByNamespaceMethod:130,dispidGetDataPartXmlMethod:131,dispidGetDataPartNodesMethod:132,dispidDeleteDataPartMethod:133,dispidGetDataNodeValueMethod:134,dispidGetDataNodeXmlMethod:135,dispidGetDataNodesMethod:136,dispidSetDataNodeValueMethod:137,dispidSetDataNodeXmlMethod:138,dispidAddDataNamespaceMethod:139,dispidGetDataUriByPrefixMethod:140,dispidGetDataPrefixByUriMethod:141,dispidMethodMax:141,dispidGetSelectedTaskMethod:110,dispidGetSelectedResourceMethod:111,dispidGetTaskMethod:112,dispidGetResourceFieldMethod:113,dispidGetWSSUrlMethod:114,dispidGetTaskFieldMethod:115,dispidGetProjectFieldMethod:116,dispidGetSelectedViewMethod:117,dispidGetTaskByIndexMethod:118,dispidGetResourceByIndexMethod:119,dispidSetTaskFieldMethod:120,dispidSetResourceFieldMethod:121,dispidGetMaxTaskIndexMethod:122,dispidGetMaxResourceIndexMethod:123};OSF.DDA.EventDispId={dispidEventMin:0,dispidInitializeEvent:0,dispidSettingsChangedEvent:1,dispidDocumentSelectionChangedEvent:2,dispidBindingSelectionChangedEvent:3,dispidBindingDataChangedEvent:4,dispidDocumentOpenEvent:5,dispidDocumentCloseEvent:6,dispidActiveViewChangedEvent:7,dispidDocumentThemeChangedEvent:8,dispidOfficeThemeChangedEvent:9,dispidActivationStatusChangedEvent:32,dispidAppCommandInvokedEvent:39,dispidTaskSelectionChangedEvent:56,dispidResourceSelectionChangedEvent:57,dispidViewSelectionChangedEvent:58,dispidDataNodeAddedEvent:60,dispidDataNodeReplacedEvent:61,dispidDataNodeDeletedEvent:62,dispidEventMax:63};OSF.DDA.ErrorCodeManager=function(){var n={};return{getErrorArgs:function(t){return n[t]||n[this.errorCodes.ooeInternalError]},addErrorMessage:function(t,i){n[t]=i},errorCodes:{ooeSuccess:0,ooeChunkResult:1,ooeCoercionTypeNotSupported:1e3,ooeGetSelectionNotMatchDataType:1001,ooeCoercionTypeNotMatchBinding:1002,ooeInvalidGetRowColumnCounts:1003,ooeSelectionNotSupportCoercionType:1004,ooeInvalidGetStartRowColumn:1005,ooeNonUniformPartialGetNotSupported:1006,ooeGetDataIsTooLarge:1008,ooeFileTypeNotSupported:1009,ooeGetDataParametersConflict:1010,ooeInvalidGetColumns:1011,ooeInvalidGetRows:1012,ooeInvalidReadForBlankRow:1013,ooeUnsupportedDataObject:2e3,ooeCannotWriteToSelection:2001,ooeDataNotMatchSelection:2002,ooeOverwriteWorksheetData:2003,ooeDataNotMatchBindingSize:2004,ooeInvalidSetStartRowColumn:2005,ooeInvalidDataFormat:2006,ooeDataNotMatchCoercionType:2007,ooeDataNotMatchBindingType:2008,ooeSetDataIsTooLarge:2009,ooeNonUniformPartialSetNotSupported:2010,ooeInvalidSetColumns:2011,ooeInvalidSetRows:2012,ooeSetDataParametersConflict:2013,ooeCellDataAmountBeyondLimits:2014,ooeSelectionCannotBound:3e3,ooeBindingNotExist:3002,ooeBindingToMultipleSelection:3003,ooeInvalidSelectionForBindingType:3004,ooeOperationNotSupportedOnThisBindingType:3005,ooeNamedItemNotFound:3006,ooeMultipleNamedItemFound:3007,ooeInvalidNamedItemForBindingType:3008,ooeUnknownBindingType:3009,ooeOperationNotSupportedOnMatrixData:3010,ooeInvalidColumnsForBinding:3011,ooeSettingNameNotExist:4e3,ooeSettingsCannotSave:4001,ooeSettingsAreStale:4002,ooeOperationNotSupported:5e3,ooeInternalError:5001,ooeDocumentReadOnly:5002,ooeEventHandlerNotExist:5003,ooeInvalidApiCallInContext:5004,ooeShuttingDown:5005,ooeUnsupportedEnumeration:5007,ooeIndexOutOfRange:5008,ooeBrowserAPINotSupported:5009,ooeInvalidParam:5010,ooeRequestTimeout:5011,ooeTooManyIncompleteRequests:5100,ooeRequestTokenUnavailable:5101,ooeActivityLimitReached:5102,ooeCustomXmlNodeNotFound:6e3,ooeCustomXmlError:6100,ooeCustomXmlExceedQuota:6101,ooeCustomXmlOutOfDate:6102,ooeNoCapability:7e3,ooeCannotNavTo:7001,ooeSpecifiedIdNotExist:7002,ooeNavOutOfBound:7004,ooeElementMissing:8e3,ooeProtectedError:8001,ooeInvalidCellsValue:8010,ooeInvalidTableOptionValue:8011,ooeInvalidFormatValue:8012,ooeRowIndexOutOfRange:8020,ooeColIndexOutOfRange:8021,ooeFormatValueOutOfRange:8022,ooeCellFormatAmountBeyondLimits:8023,ooeMemoryFileLimit:11e3,ooeNetworkProblemRetrieveFile:11001,ooeInvalidSliceSize:11002,ooeInvalidCallback:11101},initializeErrorMessages:function(t){n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotSupported]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetSelectionNotMatchDataType]={name:t.L_DataReadError,message:t.L_GetSelectionNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding]={name:t.L_InvalidCoercion,message:t.L_CoercionTypeNotMatchBinding};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRowColumnCounts]={name:t.L_DataReadError,message:t.L_InvalidGetRowColumnCounts};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionNotSupportCoercionType]={name:t.L_DataReadError,message:t.L_SelectionNotSupportCoercionType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetStartRowColumn]={name:t.L_DataReadError,message:t.L_InvalidGetStartRowColumn};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialGetNotSupported]={name:t.L_DataReadError,message:t.L_NonUniformPartialGetNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataIsTooLarge]={name:t.L_DataReadError,message:t.L_GetDataIsTooLarge};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeFileTypeNotSupported]={name:t.L_DataReadError,message:t.L_FileTypeNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeGetDataParametersConflict]={name:t.L_DataReadError,message:t.L_GetDataParametersConflict};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetColumns]={name:t.L_DataReadError,message:t.L_InvalidGetColumns};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidGetRows]={name:t.L_DataReadError,message:t.L_InvalidGetRows};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidReadForBlankRow]={name:t.L_DataReadError,message:t.L_InvalidReadForBlankRow};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedDataObject]={name:t.L_DataWriteError,message:t.L_UnsupportedDataObject};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotWriteToSelection]={name:t.L_DataWriteError,message:t.L_CannotWriteToSelection};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchSelection]={name:t.L_DataWriteError,message:t.L_DataNotMatchSelection};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeOverwriteWorksheetData]={name:t.L_DataWriteError,message:t.L_OverwriteWorksheetData};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingSize]={name:t.L_DataWriteError,message:t.L_DataNotMatchBindingSize};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetStartRowColumn]={name:t.L_DataWriteError,message:t.L_InvalidSetStartRowColumn};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidDataFormat]={name:t.L_InvalidFormat,message:t.L_InvalidDataFormat};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchCoercionType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchCoercionType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeDataNotMatchBindingType]={name:t.L_InvalidDataObject,message:t.L_DataNotMatchBindingType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataIsTooLarge]={name:t.L_DataWriteError,message:t.L_SetDataIsTooLarge};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNonUniformPartialSetNotSupported]={name:t.L_DataWriteError,message:t.L_NonUniformPartialSetNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetColumns]={name:t.L_DataWriteError,message:t.L_InvalidSetColumns};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSetRows]={name:t.L_DataWriteError,message:t.L_InvalidSetRows};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSetDataParametersConflict]={name:t.L_DataWriteError,message:t.L_SetDataParametersConflict};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSelectionCannotBound]={name:t.L_BindingCreationError,message:t.L_SelectionCannotBound};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingNotExist]={name:t.L_InvalidBindingError,message:t.L_BindingNotExist};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeBindingToMultipleSelection]={name:t.L_BindingCreationError,message:t.L_BindingToMultipleSelection};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSelectionForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidSelectionForBindingType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnThisBindingType]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnThisBindingType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNamedItemNotFound]={name:t.L_BindingCreationError,message:t.L_NamedItemNotFound};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeMultipleNamedItemFound]={name:t.L_BindingCreationError,message:t.L_MultipleNamedItemFound};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidNamedItemForBindingType]={name:t.L_BindingCreationError,message:t.L_InvalidNamedItemForBindingType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnknownBindingType]={name:t.L_InvalidBinding,message:t.L_UnknownBindingType};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupportedOnMatrixData]={name:t.L_InvalidBindingOperation,message:t.L_OperationNotSupportedOnMatrixData};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidColumnsForBinding]={name:t.L_InvalidBinding,message:t.L_InvalidColumnsForBinding};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingNameNotExist]={name:t.L_ReadSettingsError,message:t.L_SettingNameNotExist};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsCannotSave]={name:t.L_SaveSettingsError,message:t.L_SettingsCannotSave};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSettingsAreStale]={name:t.L_SettingsStaleError,message:t.L_SettingsAreStale};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeOperationNotSupported]={name:t.L_HostError,message:t.L_OperationNotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError]={name:t.L_InternalError,message:t.L_InternalErrorDescription};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeDocumentReadOnly]={name:t.L_PermissionDenied,message:t.L_DocumentReadOnly};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist]={name:t.L_EventRegistrationError,message:t.L_EventHandlerNotExist};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext]={name:t.L_InvalidAPICall,message:t.L_InvalidApiCallInContext};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeShuttingDown]={name:t.L_ShuttingDown,message:t.L_ShuttingDown};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration]={name:t.L_UnsupportedEnumeration,message:t.L_UnsupportedEnumerationMessage};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeIndexOutOfRange]={name:t.L_IndexOutOfRange,message:t.L_IndexOutOfRange};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeBrowserAPINotSupported]={name:t.L_APINotSupported,message:t.L_BrowserAPINotSupported};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTimeout]={name:t.L_APICallFailed,message:t.L_RequestTimeout};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeTooManyIncompleteRequests]={name:t.L_APICallFailed,message:t.L_TooManyIncompleteRequests};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeRequestTokenUnavailable]={name:t.L_APICallFailed,message:t.L_RequestTokenUnavailable};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeActivityLimitReached]={name:t.L_APICallFailed,message:t.L_ActivityLimitReached};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlNodeNotFound]={name:t.L_InvalidNode,message:t.L_CustomXmlNodeNotFound};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlError]={name:t.L_CustomXmlError,message:t.L_CustomXmlError};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlExceedQuota]={name:t.L_CustomXmlError,message:t.L_CustomXmlError};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCustomXmlOutOfDate]={name:t.L_CustomXmlError,message:t.L_CustomXmlError};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability]={name:t.L_PermissionDenied,message:t.L_NoCapability};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCannotNavTo]={name:t.L_CannotNavigateTo,message:t.L_CannotNavigateTo};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeSpecifiedIdNotExist]={name:t.L_SpecifiedIdNotExist,message:t.L_SpecifiedIdNotExist};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNavOutOfBound]={name:t.L_NavOutOfBound,message:t.L_NavOutOfBound};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellDataAmountBeyondLimits]={name:t.L_DataWriteReminder,message:t.L_CellDataAmountBeyondLimits};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeElementMissing]={name:t.L_MissingParameter,message:t.L_ElementMissing};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeProtectedError]={name:t.L_PermissionDenied,message:t.L_NoCapability};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidCellsValue]={name:t.L_InvalidValue,message:t.L_InvalidCellsValue};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidTableOptionValue]={name:t.L_InvalidValue,message:t.L_InvalidTableOptionValue};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidFormatValue]={name:t.L_InvalidValue,message:t.L_InvalidFormatValue};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeRowIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_RowIndexOutOfRange};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeColIndexOutOfRange]={name:t.L_OutOfRange,message:t.L_ColIndexOutOfRange};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeFormatValueOutOfRange]={name:t.L_OutOfRange,message:t.L_FormatValueOutOfRange};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeCellFormatAmountBeyondLimits]={name:t.L_FormattingReminder,message:t.L_CellFormatAmountBeyondLimits};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeMemoryFileLimit]={name:t.L_MemoryLimit,message:t.L_CloseFileBeforeRetrieve};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeNetworkProblemRetrieveFile]={name:t.L_NetworkProblem,message:t.L_NetworkProblemRetrieveFile};n[OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidSliceSize]={name:t.L_InvalidValue,message:t.L_SliceSizeNotSupported}}}}(),function(n){(function(n){var t=1.1,r=function(){function n(n){this.isSetSupported=function(n,t){var u,i,r;return n==undefined?!1:(t==undefined&&(t=0),u=this._setMap,i=u._sets,i.hasOwnProperty(n.toLowerCase())?(r=i[n.toLowerCase()],r>0&&r>=t):!1)};this._setMap=n}return n}(),i,f,e,o,s,h,c,l,a,u,v,y,p,w,b,k,d;n.RequirementMatrix=r;i=function(){function n(n){this._sets=n}return n}();n.DefaultSetRequirement=i;f=function(n){function i(){n.call(this,{bindingevents:t,documentevents:t,excelapi:t,matrixbindings:t,matrixcoercion:t,selection:t,settings:t,tablebindings:t,tablecoercion:t,textbindings:t,textcoercion:t})}return __extends(i,n),i}(i);n.ExcelClientDefaultSetRequirement=f;e=function(n){function t(){n.call(this,{mailbox:1.3})}return __extends(t,n),t}(i);n.OutlookClientDefaultSetRequirement=e;o=function(n){function i(){n.call(this,{bindingevents:t,compressedfile:t,customxmlparts:t,documentevents:t,file:t,htmlcoercion:t,matrixbindings:t,matrixcoercion:t,ooxmlcoercion:t,pdffile:t,selection:t,settings:t,tablebindings:t,tablecoercion:t,textbindings:t,textcoercion:t,textfile:t,wordapi:t})}return __extends(i,n),i}(i);n.WordClientDefaultSetRequirement=o;s=function(n){function i(){n.call(this,{activeview:t,compressedfile:t,documentevents:t,file:t,pdffile:t,selection:t,settings:t,textcoercion:t})}return __extends(i,n),i}(i);n.PowerpointClientDefaultSetRequirement=s;h=function(n){function i(){n.call(this,{selection:t,textcoercion:t})}return __extends(i,n),i}(i);n.ProjectClientDefaultSetRequirement=h;c=function(n){function i(){n.call(this,{bindingevents:t,documentevents:t,matrixbindings:t,matrixcoercion:t,selection:t,settings:t,tablebindings:t,tablecoercion:t,textbindings:t,textcoercion:t,file:t})}return __extends(i,n),i}(i);n.ExcelWebDefaultSetRequirement=c;l=function(n){function i(){n.call(this,{customxmlparts:t,documentevents:t,file:t,ooxmlcoercion:t,selection:t,settings:t,textcoercion:t})}return __extends(i,n),i}(i);n.WordWebDefaultSetRequirement=l;a=function(n){function i(){n.call(this,{activeview:t,settings:t})}return __extends(i,n),i}(i);n.PowerpointWebDefaultSetRequirement=a;u=function(n){function t(){n.call(this,{mailbox:1.3})}return __extends(t,n),t}(i);n.OutlookWebDefaultSetRequirement=u;v=function(n){function i(){n.call(this,{activeview:t,documentevents:t,selection:t,settings:t,textcoercion:t})}return __extends(i,n),i}(i);n.SwayWebDefaultSetRequirement=v;y=function(n){function i(){n.call(this,{bindingevents:t,partialtablebindings:t,settings:t,tablebindings:t,tablecoercion:t})}return __extends(i,n),i}(i);n.AccessWebDefaultSetRequirement=y;p=function(n){function i(){n.call(this,{bindingevents:t,documentevents:t,matrixbindings:t,matrixcoercion:t,selection:t,settings:t,tablebindings:t,tablecoercion:t,textbindings:t,textcoercion:t})}return __extends(i,n),i}(i);n.ExcelIOSDefaultSetRequirement=p;w=function(n){function i(){n.call(this,{bindingevents:t,compressedfile:t,customxmlparts:t,documentevents:t,file:t,htmlcoercion:t,matrixbindings:t,matrixcoercion:t,ooxmlcoercion:t,pdffile:t,selection:t,settings:t,tablebindings:t,tablecoercion:t,textbindings:t,textcoercion:t,textfile:t})}return __extends(i,n),i}(i);n.WordIOSDefaultSetRequirement=w;b=function(n){function i(){n.call(this,{activeview:t,compressedfile:t,documentevents:t,file:t,pdffile:t,selection:t,settings:t,textcoercion:t})}return __extends(i,n),i}(i);n.PowerpointIOSDefaultSetRequirement=b;k=function(n){function i(){n.call(this,{mailbox:t})}return __extends(i,n),i}(i);n.OutlookIOSDefaultSetRequirement=k;d=function(){function n(){}return n.initializeOsfDda=function(){OSF.OUtil.setNamespace("Requirement",OSF.DDA)},n.getDefaultRequirementMatrix=function(t){var u,s,f,e,h,o;return this.initializeDefaultSetMatrix(),u=undefined,t.get_requirementMatrix()!=undefined&&typeof JSON!="undefined"?(s=JSON.parse(t.get_requirementMatrix()),u=new r(new i(s))):(f=t.get_appMinorVersion(),e="",e=f<10?"0"+f:""+f,h=t.get_appVersion()+"."+e,o=t.get_appName()+"-"+h,u=n.DefaultSetArrayMatrix!=undefined&&n.DefaultSetArrayMatrix[o]!=undefined?new r(n.DefaultSetArrayMatrix[o]):new r(new i({}))),u},n.initializeDefaultSetMatrix=function(){n.DefaultSetArrayMatrix[n.Excel_RCLIENT_1600]=new f;n.DefaultSetArrayMatrix[n.Word_RCLIENT_1600]=new o;n.DefaultSetArrayMatrix[n.PowerPoint_RCLIENT_1600]=new s;n.DefaultSetArrayMatrix[n.Outlook_RCLIENT_1600]=new e;n.DefaultSetArrayMatrix[n.Excel_WAC_1600]=new c;n.DefaultSetArrayMatrix[n.Word_WAC_1600]=new l;n.DefaultSetArrayMatrix[n.Outlook_WAC_1600]=new u;n.DefaultSetArrayMatrix[n.Outlook_WAC_1601]=new u;n.DefaultSetArrayMatrix[n.Project_RCLIENT_1600]=new h;n.DefaultSetArrayMatrix[n.Access_WAC_1600]=new y;n.DefaultSetArrayMatrix[n.PowerPoint_WAC_1600]=new a;n.DefaultSetArrayMatrix[n.Excel_IOS_1600]=new p;n.DefaultSetArrayMatrix[n.SWAY_WAC_1600]=new v;n.DefaultSetArrayMatrix[n.Word_IOS_1600]=new w;n.DefaultSetArrayMatrix[n.PowerPoint_IOS_1600]=new b;n.DefaultSetArrayMatrix[n.Outlook_IOS_1600]=new k},n.Excel_RCLIENT_1600="1-16.00",n.Word_RCLIENT_1600="2-16.00",n.PowerPoint_RCLIENT_1600="4-16.00",n.Outlook_RCLIENT_1600="8-16.00",n.Excel_WAC_1600="16-16.00",n.Word_WAC_1600="32-16.00",n.Outlook_WAC_1600="64-16.00",n.Outlook_WAC_1601="64-16.01",n.Project_RCLIENT_1600="128-16.00",n.Access_WAC_1600="256-16.00",n.PowerPoint_WAC_1600="512-16.00",n.Excel_IOS_1600="1024-16.01",n.SWAY_WAC_1600="2048-16.00",n.Word_IOS_1600="4096-16.00",n.PowerPoint_IOS_1600="8192-16.00",n.Outlook_IOS_1600="65536-16.00",n.DefaultSetArrayMatrix={},n}();n.RequirementsMatrixFactory=d})(n.Requirement||(n.Requirement={}));var t=n.Requirement}(OfficeExt||(OfficeExt={}));OfficeExt.Requirement.RequirementsMatrixFactory.initializeOsfDda();Microsoft.Office.WebExtension.ApplicationMode={WebEditor:"webEditor",WebViewer:"webViewer",Client:"client"};Microsoft.Office.WebExtension.DocumentMode={ReadOnly:"readOnly",ReadWrite:"readWrite"};OSF.NamespaceManager=function(){var t,n=!1;return{enableShortcut:function(){n||(window.Office?t=window.Office:OSF.OUtil.setNamespace("Office",window),window.Office=Microsoft.Office.WebExtension,n=!0)},disableShortcut:function(){n&&(t?window.Office=t:OSF.OUtil.unsetNamespace("Office",window),n=!1)}}}();OSF.NamespaceManager.enableShortcut();Microsoft.Office.WebExtension.useShortNamespace=function(n){n?OSF.NamespaceManager.enableShortcut():OSF.NamespaceManager.disableShortcut()};Microsoft.Office.WebExtension.select=function(n,t){var i,r,o,u,f,e;if(n&&typeof n=="string"&&(r=n.indexOf("#"),r!=-1)){o=n.substring(0,r);u=n.substring(r+1);switch(o){case"binding":case"bindings":u&&(i=new OSF.DDA.BindingPromise(u))}}if(i)return i.onFail=t,i;else if(t)if(f=typeof t,f=="function")e={},e[Microsoft.Office.WebExtension.Parameters.Callback]=t,OSF.DDA.issueAsyncResult(e,OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext,OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInvalidApiCallInContext));else throw OSF.OUtil.formatString(Strings.OfficeOM.L_CallbackNotAFunction,f);};OSF.DDA.Context=function(n,t,i,r,u){var f=this,e,o;OSF.OUtil.defineEnumerableProperties(f,{contentLanguage:{value:n.get_dataLocale()},displayLanguage:{value:n.get_appUILocale()},touchEnabled:{value:n.get_touchEnabled()},commerceAllowed:{value:n.get_commerceAllowed()}});t&&OSF.OUtil.defineEnumerableProperty(f,"document",{value:t});i&&OSF.OUtil.defineEnumerableProperty(f,"license",{value:i});r&&(e=r.displayName||"appOM",delete r.displayName,OSF.OUtil.defineEnumerableProperty(f,e,{value:r}));u&&OSF.OUtil.defineEnumerableProperty(f,"officeTheme",{get:function(){return u()}});o=OfficeExt.Requirement.RequirementsMatrixFactory.getDefaultRequirementMatrix(n);OSF.OUtil.defineEnumerableProperty(f,"requirements",{value:o})};OSF.DDA.OutlookContext=function(n,t,i,r,u){OSF.DDA.OutlookContext.uber.constructor.call(this,n,null,i,r,u);t&&OSF.OUtil.defineEnumerableProperty(this,"roamingSettings",{value:t})};OSF.OUtil.extend(OSF.DDA.OutlookContext,OSF.DDA.Context);OSF.DDA.OutlookAppOm=function(){};OSF.DDA.Document=function(n,t){var i;switch(n.get_clientMode()){case OSF.ClientMode.ReadOnly:i=Microsoft.Office.WebExtension.DocumentMode.ReadOnly;break;case OSF.ClientMode.ReadWrite:i=Microsoft.Office.WebExtension.DocumentMode.ReadWrite}t&&OSF.OUtil.defineEnumerableProperty(this,"settings",{value:t});OSF.OUtil.defineMutableProperties(this,{mode:{value:i},url:{value:n.get_docUrl()}})};OSF.DDA.JsomDocument=function(n,t,i){var r=this,u;OSF.DDA.JsomDocument.uber.constructor.call(r,n,i);t&&OSF.OUtil.defineEnumerableProperty(r,"bindings",{get:function(){return t}});u=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(r,[u.GetSelectedDataAsync,u.SetSelectedDataAsync]);OSF.DDA.DispIdHost.addEventSupport(r,new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged]))};OSF.OUtil.extend(OSF.DDA.JsomDocument,OSF.DDA.Document);OSF.OUtil.defineEnumerableProperty(Microsoft.Office.WebExtension,"context",{get:function(){var n;return OSF&&OSF._OfficeAppFactory&&(n=OSF._OfficeAppFactory.getContext()),n}});OSF.DDA.License=function(n){OSF.OUtil.defineEnumerableProperty(this,"value",{value:n})};OSF.OUtil.setNamespace("AsyncResultEnum",OSF.DDA);OSF.DDA.AsyncResultEnum.Properties={Context:"Context",Value:"Value",Status:"Status",Error:"Error"};Microsoft.Office.WebExtension.AsyncResultStatus={Succeeded:"succeeded",Failed:"failed"};OSF.DDA.AsyncResultEnum.ErrorCode={Success:0,Failed:1};OSF.DDA.AsyncResultEnum.ErrorProperties={Name:"Name",Message:"Message",Code:"Code"};OSF.DDA.AsyncMethodNames={};OSF.DDA.AsyncMethodNames.addNames=function(n){var t,i;for(t in n)i={},OSF.OUtil.defineEnumerableProperties(i,{id:{value:t},displayName:{value:n[t]}}),OSF.DDA.AsyncMethodNames[t]=i};OSF.DDA.AsyncMethodCall=function(n,t,i,r,u,f,e){function c(n,t){var u,i,r;for(u in n){if(i=n[u],r=t[u],i["enum"])switch(typeof r){case"string":if(OSF.OUtil.listContainsValue(i["enum"],r))break;case"undefined":throw OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedEnumeration;break;default:throw s();}if(i.types&&!OSF.OUtil.listContainsValue(i.types,typeof r))throw s();}}function l(t,i,r){var f,u,h,e,l,a;if(t.length<o)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_MissingRequiredArguments);for(f=[],u=0;u<o;u++)f.push(t[u]);for(c(n,f),h={},u=0;u<o;u++){if(e=n[u],l=f[u],e.verify&&(a=e.verify(l,i,r),!a))throw s();h[e.name]=l}return h}function a(n,i,r,u){var f,l,a,v,y,e,s;if(n.length>o+2)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyArguments);for(a=n.length-1;a>=o;a--){v=n[a];switch(typeof v){case"object":if(f)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalObjects);else f=v;break;case h:if(l)throw OsfMsAjaxFactory.msAjaxError.parameterCount(Strings.OfficeOM.L_TooManyOptionalFunction);else l=v;break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Strings.OfficeOM.L_InValidOptionalArgument);}}f=f||{};for(y in t)OSF.OUtil.listContainsKey(f,y)||(e=undefined,s=t[y],s.calculate&&i&&(e=s.calculate(i,r,u)),e||s.defaultValue===undefined||(e=s.defaultValue),f[y]=e);if(l)if(f[Microsoft.Office.WebExtension.Parameters.Callback])throw Strings.OfficeOM.L_RedundantCallbackSpecification;else f[Microsoft.Office.WebExtension.Parameters.Callback]=l;return c(t,f),f}var h="function",o=n.length,s=OSF.OUtil.delayExecutionAndCache(function(){return OSF.OUtil.formatString(Strings.OfficeOM.L_InvalidParameters,e)});this.verifyAndExtractCall=function(n,t,r){var e=l(n,t,r),c=a(n,e,t,r),u={},o,s,h;for(o in e)u[o]=e[o];for(s in c)u[s]=c[s];for(h in i)u[h]=i[h](t,r);return f&&(u=f(u,t,r)),u};this.processResponse=function(n,t,i,f){return n==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?r?r(t,i,f):t:u?u(n,t):OSF.DDA.ErrorCodeManager.getErrorArgs(n)};this.getCallArgs=function(n){for(var t,u,r,i=n.length-1;i>=o;i--){r=n[i];switch(typeof r){case"object":t=r;break;case h:u=r}}return t=t||{},u&&(t[Microsoft.Office.WebExtension.Parameters.Callback]=u),t}};OSF.DDA.AsyncMethodCallFactory=function(){function n(n){var i=null,r,t;if(n)for(i={},r=n.length,t=0;t<r;t++)i[n[t].name]=n[t].value;return i}return{manufacture:function(t){var i=t.supportedOptions?n(t.supportedOptions):[],r=t.privateStateCallbacks?n(t.privateStateCallbacks):[];return new OSF.DDA.AsyncMethodCall(t.requiredArguments||[],i,r,t.onSucceeded,t.onFailed,t.checkCallArgs,t.method.displayName)}}}();OSF.DDA.AsyncMethodCalls={};OSF.DDA.AsyncMethodCalls.define=function(n){OSF.DDA.AsyncMethodCalls[n.method.id]=OSF.DDA.AsyncMethodCallFactory.manufacture(n)};OSF.DDA.Error=function(n,t,i){OSF.OUtil.defineEnumerableProperties(this,{name:{value:n},message:{value:t},code:{value:i}})};OSF.DDA.AsyncResult=function(n,t){OSF.OUtil.defineEnumerableProperties(this,{value:{value:n[OSF.DDA.AsyncResultEnum.Properties.Value]},status:{value:t?Microsoft.Office.WebExtension.AsyncResultStatus.Failed:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded}});n[OSF.DDA.AsyncResultEnum.Properties.Context]&&OSF.OUtil.defineEnumerableProperty(this,"asyncContext",{value:n[OSF.DDA.AsyncResultEnum.Properties.Context]});t&&OSF.OUtil.defineEnumerableProperty(this,"error",{value:new OSF.DDA.Error(t[OSF.DDA.AsyncResultEnum.ErrorProperties.Name],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Message],t[OSF.DDA.AsyncResultEnum.ErrorProperties.Code])})};OSF.DDA.issueAsyncResult=function(n,t,i){var f=n[Microsoft.Office.WebExtension.Parameters.Callback],u,r;f&&(u={},u[OSF.DDA.AsyncResultEnum.Properties.Context]=n[Microsoft.Office.WebExtension.Parameters.AsyncContext],t==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?u[OSF.DDA.AsyncResultEnum.Properties.Value]=i:(r={},i=i||OSF.DDA.ErrorCodeManager.getErrorArgs(OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError),r[OSF.DDA.AsyncResultEnum.ErrorProperties.Code]=t||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError,r[OSF.DDA.AsyncResultEnum.ErrorProperties.Name]=i.name||i,r[OSF.DDA.AsyncResultEnum.ErrorProperties.Message]=i.message||i),f(new OSF.DDA.AsyncResult(u,r)))};OSF.DDA.ListType=function(){var n={};return{setListType:function(t,i){n[t]=i},isListType:function(t){return OSF.OUtil.listContainsKey(n,t)},getDescriptor:function(t){return n[t]}}}();OSF.DDA.HostParameterMap=function(n,t){function e(i,f){var a=i?{}:undefined,s,h,o,v,c,l;for(s in i){if(h=i[s],OSF.DDA.ListType.isListType(s)){o=[];for(v in h)o.push(e(h[v],f))}else OSF.OUtil.listContainsKey(r,s)?o=r[s][f](h):f==u&&n.preserveNesting(s)?o=e(h,f):(c=t[s],c?(l=c[f],l&&(o=l[h],o===undefined&&(o=h))):o=h);a[s]=o}return a}function c(i,r){var e,u,h,s;for(u in r)h=n.isComplexType(u)?c(i,t[u][o]):i[u],h!=undefined&&(e||(e={}),s=r[u],s==f&&(s=u),e[s]=n.pack(u,h));return e}function s(i,r,e){var o,l,h,c,v,a;e||(e={});for(o in r)if(l=r[o],h=l==f?i:i[l],h===null||h===undefined)e[o]=undefined;else if(h=n.unpack(o,h),n.isComplexType(o))c=t[o][u],n.preserveNesting(o)?e[o]=s(h,c):s(h,c,e);else{if(OSF.DDA.ListType.isListType(o)){c={};v=OSF.DDA.ListType.getDescriptor(o);c[v]=f;for(a in h)h[a]=s(h[a],c)}e[o]=h}return e}function l(n,i,r){var f=t[n][r],u,o,l;return r=="toHost"?(o=e(i,r),u=c(o,f)):r==h&&(l=s(i,f),u=e(l,r)),u}var h="fromHost",i=this,o="toHost",u=h,f="self",r={};r[Microsoft.Office.WebExtension.Parameters.Data]={toHost:function(n){if(n!=null&&n.rows!==undefined){var t={};t[OSF.DDA.TableDataProperties.TableRows]=n.rows;t[OSF.DDA.TableDataProperties.TableHeaders]=n.headers;n=t}return n},fromHost:function(n){return n}};r[Microsoft.Office.WebExtension.Parameters.SampleData]=r[Microsoft.Office.WebExtension.Parameters.Data];t||(t={});i.addMapping=function(n,i){var e,h,c,l,r,s,a,v;if(i.map){e=i.map;h={};for(c in e)l=e[c],l==f&&(l=c),h[l]=c}else e=i.toHost,h=i.fromHost;if(r=t[n],r){s=r[o];for(a in s)e[a]=s[a];s=r[u];for(v in s)h[v]=s[v]}else r=t[n]={};r[o]=e;r[u]=h};i.toHost=function(n,t){return l(n,t,o)};i.fromHost=function(n,t){return l(n,t,u)};i.self=f;i.addComplexType=function(t){n.addComplexType(t)};i.getDynamicType=function(t){return n.getDynamicType(t)};i.setDynamicType=function(t,i){n.setDynamicType(t,i)};i.dynamicTypes=r;i.doMapValues=function(n,t){return e(n,t)}};OSF.DDA.SpecialProcessor=function(n,t){var i=this;i.addComplexType=function(t){n.push(t)};i.getDynamicType=function(n){return t[n]};i.setDynamicType=function(n,i){t[n]=i};i.isComplexType=function(t){return OSF.OUtil.listContainsValue(n,t)};i.isDynamicType=function(n){return OSF.OUtil.listContainsKey(t,n)};i.preserveNesting=function(n){var t=[];return OSF.DDA.PropertyDescriptors&&t.push(OSF.DDA.PropertyDescriptors.Subset),OSF.DDA.DataNodeEventProperties&&(t=t.concat([OSF.DDA.DataNodeEventProperties.OldNode,OSF.DDA.DataNodeEventProperties.NewNode,OSF.DDA.DataNodeEventProperties.NextSiblingNode])),OSF.OUtil.listContainsValue(t,n)};i.pack=function(n,i){return this.isDynamicType(n)?t[n].toHost(i):i};i.unpack=function(n,i){return this.isDynamicType(n)?t[n].fromHost(i):i}};OSF.DDA.getDecoratedParameterMap=function(n,t){function r(n){var i=null,r,t;if(n)for(i={},r=n.length,t=0;t<r;t++)i[n[t].name]=n[t].value;return i}var i=new OSF.DDA.HostParameterMap(n),f=i.self,u;i.define=function(n){var t={},u=r(n.toHost);n.invertible?t.map=u:n.canonical?t.toHost=t.fromHost=u:(t.toHost=u,t.fromHost=r(n.fromHost));i.addMapping(n.type,t);n.isComplexType&&i.addComplexType(n.type)};for(u in t)i.define(t[u]);return i};OSF.OUtil.setNamespace("DispIdHost",OSF.DDA);OSF.DDA.DispIdHost.Methods={InvokeMethod:"invokeMethod",AddEventHandler:"addEventHandler",RemoveEventHandler:"removeEventHandler"};OSF.DDA.DispIdHost.Delegates={ExecuteAsync:"executeAsync",RegisterEventAsync:"registerEventAsync",UnregisterEventAsync:"unregisterEventAsync",ParameterMap:"parameterMap"};OSF.DDA.DispIdHost.Facade=function(n,t){function s(n,t,i,r){if(typeof n=="number")r||(r=t.getCallArgs(i)),OSF.DDA.issueAsyncResult(r,n,OSF.DDA.ErrorCodeManager.getErrorArgs(n));else throw n;}var r={},u=OSF.DDA.AsyncMethodNames,i=OSF.DDA.MethodDispId,h={GoToByIdAsync:i.dispidNavigateToMethod,GetSelectedDataAsync:i.dispidGetSelectedDataMethod,SetSelectedDataAsync:i.dispidSetSelectedDataMethod,GetDocumentCopyChunkAsync:i.dispidGetDocumentCopyChunkMethod,ReleaseDocumentCopyAsync:i.dispidReleaseDocumentCopyMethod,GetDocumentCopyAsync:i.dispidGetDocumentCopyMethod,AddFromSelectionAsync:i.dispidAddBindingFromSelectionMethod,AddFromPromptAsync:i.dispidAddBindingFromPromptMethod,AddFromNamedItemAsync:i.dispidAddBindingFromNamedItemMethod,GetAllAsync:i.dispidGetAllBindingsMethod,GetByIdAsync:i.dispidGetBindingMethod,ReleaseByIdAsync:i.dispidReleaseBindingMethod,GetDataAsync:i.dispidGetBindingDataMethod,SetDataAsync:i.dispidSetBindingDataMethod,AddRowsAsync:i.dispidAddRowsMethod,AddColumnsAsync:i.dispidAddColumnsMethod,DeleteAllDataValuesAsync:i.dispidClearAllRowsMethod,RefreshAsync:i.dispidLoadSettingsMethod,SaveAsync:i.dispidSaveSettingsMethod,GetActiveViewAsync:i.dispidGetActiveViewMethod,GetFilePropertiesAsync:i.dispidGetFilePropertiesMethod,GetOfficeThemeAsync:i.dispidGetOfficeThemeMethod,GetDocumentThemeAsync:i.dispidGetDocumentThemeMethod,ClearFormatsAsync:i.dispidClearFormatsMethod,SetTableOptionsAsync:i.dispidSetTableOptionsMethod,SetFormatsAsync:i.dispidSetFormatsMethod,ExecuteRichApiRequestAsync:i.dispidExecuteRichApiRequestMethod,AppCommandInvocationCompletedAsync:i.dispidAppCommandInvocationCompletedMethod,AddDataPartAsync:i.dispidAddDataPartMethod,GetDataPartByIdAsync:i.dispidGetDataPartByIdMethod,GetDataPartsByNameSpaceAsync:i.dispidGetDataPartsByNamespaceMethod,GetPartXmlAsync:i.dispidGetDataPartXmlMethod,GetPartNodesAsync:i.dispidGetDataPartNodesMethod,DeleteDataPartAsync:i.dispidDeleteDataPartMethod,GetNodeValueAsync:i.dispidGetDataNodeValueMethod,GetNodeXmlAsync:i.dispidGetDataNodeXmlMethod,GetRelativeNodesAsync:i.dispidGetDataNodesMethod,SetNodeValueAsync:i.dispidSetDataNodeValueMethod,SetNodeXmlAsync:i.dispidSetDataNodeXmlMethod,AddDataPartNamespaceAsync:i.dispidAddDataNamespaceMethod,GetDataPartNamespaceAsync:i.dispidGetDataUriByPrefixMethod,GetDataPartPrefixAsync:i.dispidGetDataPrefixByUriMethod,GetSelectedTask:i.dispidGetSelectedTaskMethod,GetTask:i.dispidGetTaskMethod,GetWSSUrl:i.dispidGetWSSUrlMethod,GetTaskField:i.dispidGetTaskFieldMethod,GetSelectedResource:i.dispidGetSelectedResourceMethod,GetResourceField:i.dispidGetResourceFieldMethod,GetProjectField:i.dispidGetProjectFieldMethod,GetSelectedView:i.dispidGetSelectedViewMethod,GetTaskByIndex:i.dispidGetTaskByIndexMethod,GetResourceByIndex:i.dispidGetResourceByIndexMethod,SetTaskField:i.dispidSetTaskFieldMethod,SetResourceField:i.dispidSetResourceFieldMethod,GetMaxTaskIndex:i.dispidGetMaxTaskIndexMethod,GetMaxResourceIndex:i.dispidGetMaxResourceIndexMethod},f,o,e;for(f in h)u[f]&&(r[u[f].id]=h[f]);u=Microsoft.Office.WebExtension.EventType;i=OSF.DDA.EventDispId;o={SettingsChanged:i.dispidSettingsChangedEvent,DocumentSelectionChanged:i.dispidDocumentSelectionChangedEvent,BindingSelectionChanged:i.dispidBindingSelectionChangedEvent,BindingDataChanged:i.dispidBindingDataChangedEvent,ActiveViewChanged:i.dispidActiveViewChangedEvent,OfficeThemeChanged:i.dispidOfficeThemeChangedEvent,DocumentThemeChanged:i.dispidDocumentThemeChangedEvent,AppCommandInvoked:i.dispidAppCommandInvokedEvent,TaskSelectionChanged:i.dispidTaskSelectionChangedEvent,ResourceSelectionChanged:i.dispidResourceSelectionChangedEvent,ViewSelectionChanged:i.dispidViewSelectionChangedEvent,DataNodeInserted:i.dispidDataNodeAddedEvent,DataNodeReplaced:i.dispidDataNodeReplacedEvent,DataNodeDeleted:i.dispidDataNodeDeletedEvent};for(e in o)u[e]&&(r[u[e]]=o[e]);this[OSF.DDA.DispIdHost.Methods.InvokeMethod]=function(i,u,f,e){var o,h,c,l,a,v;try{h=i.id;c=OSF.DDA.AsyncMethodCalls[h];o=c.verifyAndExtractCall(u,f,e);l=r[h];a=n(h);v=t.toHost?t.toHost(l,o):o;a[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]({dispId:l,hostCallArgs:v,onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:function(n,i){var r,u;r=n==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess?t.fromHost?t.fromHost(l,i):i:i;u=c.processResponse(n,r,f,o);OSF.DDA.issueAsyncResult(o,n,u)}})}catch(y){s(y,c,u,o)}};this[OSF.DDA.DispIdHost.Methods.AddEventHandler]=function(i,u,f){function a(n){var t,i;n==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=u.addEventHandler(e,l),t||(n=OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerAdditionFailed));n!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(i=OSF.DDA.ErrorCodeManager.getErrorArgs(n));OSF.DDA.issueAsyncResult(o,n,i)}var o,e,l,h,c,v;try{h=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.AddHandlerAsync.id];o=h.verifyAndExtractCall(i,f,u);e=o[Microsoft.Office.WebExtension.Parameters.EventType];l=o[Microsoft.Office.WebExtension.Parameters.Handler];u.getEventHandlerCount(e)==0?(c=r[e],v=n(e)[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync],v({eventType:e,dispId:c,targetId:f.id||"",onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:a,onEvent:function(n){var i=t.fromHost(c,n);u.fireEvent(OSF.DDA.OMFactory.manufactureEventArgs(e,f,i))}})):a(OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess)}catch(y){s(y,h,i,o)}};this[OSF.DDA.DispIdHost.Methods.RemoveEventHandler]=function(t,i,u){function a(n){var t;n!=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess&&(t=OSF.DDA.ErrorCodeManager.getErrorArgs(n));OSF.DDA.issueAsyncResult(e,n,t)}var e,f,h,c,l,o,v,y;try{c=OSF.DDA.AsyncMethodCalls[OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.id];e=c.verifyAndExtractCall(t,u,i);f=e[Microsoft.Office.WebExtension.Parameters.EventType];h=e[Microsoft.Office.WebExtension.Parameters.Handler];h===null?(o=i.clearEventHandlers(f),l=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess):(o=i.removeEventHandler(f,h),l=o?OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess:OSF.DDA.ErrorCodeManager.errorCodes.ooeEventHandlerNotExist);o&&i.getEventHandlerCount(f)==0?(v=r[f],y=n(f)[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync],y({eventType:f,dispId:v,targetId:u.id||"",onCalling:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.IssueCall)},onReceiving:function(){OSF.OUtil.writeProfilerMark(OSF.HostCallPerfMarker.ReceiveResponse)},onComplete:a})):a(l)}catch(p){s(p,c,t,e)}}};OSF.DDA.DispIdHost.addAsyncMethods=function(n,t,i){var f,r,u;for(f in t)r=t[f],u=r.displayName,n[u]||OSF.OUtil.defineEnumerableProperty(n,u,{value:function(t){return function(){var r=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.InvokeMethod];r(t,arguments,n,i)}}(r)})};OSF.DDA.DispIdHost.addEventSupport=function(n,t){var i=OSF.DDA.AsyncMethodNames.AddHandlerAsync.displayName,r=OSF.DDA.AsyncMethodNames.RemoveHandlerAsync.displayName;n[i]||OSF.OUtil.defineEnumerableProperty(n,i,{value:function(){var i=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.AddEventHandler];i(arguments,t,n)}});n[r]||OSF.OUtil.defineEnumerableProperty(n,r,{value:function(){var i=OSF._OfficeAppFactory.getHostFacade()[OSF.DDA.DispIdHost.Methods.RemoveEventHandler];i(arguments,t,n)}})};OsfMsAjaxFactory.isMsAjaxLoaded()||OSF._OfficeAppFactory&&OSF._OfficeAppFactory&&OSF._OfficeAppFactory.getLoadScriptHelper&&OSF._OfficeAppFactory.getLoadScriptHelper().isScriptLoading(OSF.ConstantNames.MicrosoftAjaxId)||(msAjaxCDNPath=(window.location.protocol.toLowerCase()==="https:"?"https:":"http:")+"//ajax.aspnetcdn.com/ajax/3.5/MicrosoftAjax.js",OsfMsAjaxFactory.loadMsAjaxFull(function(){if(!OsfMsAjaxFactory.isMsAjaxLoaded())throw"Not able to load MicrosoftAjax.js.";}));OSF.OUtil.setNamespace("SafeArray",OSF.DDA);OSF.DDA.SafeArray.Response={Status:0,Payload:1};OSF.DDA.SafeArray.UniqueArguments={Offset:"offset",Run:"run",BindingSpecificData:"bindingSpecificData",MergedCellGuid:"{66e7831f-81b2-42e2-823c-89e872d541b3}"};OSF.OUtil.setNamespace("Delegate",OSF.DDA.SafeArray);OSF.DDA.SafeArray.Delegate._onException=function(n,t){var i,r=n.number;if(r)switch(r){case-2146828218:i=OSF.DDA.ErrorCodeManager.errorCodes.ooeNoCapability;break;case-2146827850:default:i=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError}t.onComplete&&t.onComplete(i||OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError)};OSF.DDA.SafeArray.Delegate.SpecialProcessor=function(){function n(n){var t,e,i,r,u,f,o;try{if(e=n.ubound(1),i=n.ubound(2),n=n.toArray(),e==1&&i==1)t=[n];else for(t=[],r=0;r<e;r++){for(u=[],f=0;f<i;f++)o=n[r*i+f],o!=OSF.DDA.SafeArray.UniqueArguments.MergedCellGuid&&u.push(o);u.length>0&&t.push(u)}}catch(s){}return t}var i=[],t={};t[Microsoft.Office.WebExtension.Parameters.Data]=function(){var t=0,i=1;return{toHost:function(n){if(typeof n!="string"&&n[OSF.DDA.TableDataProperties.TableRows]!==undefined){var r=[];r[t]=n[OSF.DDA.TableDataProperties.TableRows];r[i]=n[OSF.DDA.TableDataProperties.TableHeaders];n=r}return n},fromHost:function(r){var f,e,u;return r.toArray?(e=r.dimensions(),e===2?f=n(r):(u=r.toArray(),u.length===2&&(u[0]!=null&&u[0].toArray||u[1]!=null&&u[1].toArray)?(f={},f[OSF.DDA.TableDataProperties.TableRows]=n(u[t]),f[OSF.DDA.TableDataProperties.TableHeaders]=n(u[i])):f=u)):f=r,f}}}();OSF.DDA.SafeArray.Delegate.SpecialProcessor.uber.constructor.call(this,i,t);this.unpack=function(n,i){var r,u;return this.isComplexType(n)||OSF.DDA.ListType.isListType(n)?(u=(i||typeof i=="unknown")&&i.toArray,r=u?i.toArray():i||{}):r=this.isDynamicType(n)?t[n].fromHost(i):i,r}};OSF.OUtil.extend(OSF.DDA.SafeArray.Delegate.SpecialProcessor,OSF.DDA.SpecialProcessor);OSF.DDA.SafeArray.Delegate.ParameterMap=OSF.DDA.getDecoratedParameterMap(new OSF.DDA.SafeArray.Delegate.SpecialProcessor,[{type:Microsoft.Office.WebExtension.Parameters.ValueFormat,toHost:[{name:Microsoft.Office.WebExtension.ValueFormat.Unformatted,value:0},{name:Microsoft.Office.WebExtension.ValueFormat.Formatted,value:1}]},{type:Microsoft.Office.WebExtension.Parameters.FilterType,toHost:[{name:Microsoft.Office.WebExtension.FilterType.All,value:0}]}]);OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.AsyncResultStatus,fromHost:[{name:Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded,value:0},{name:Microsoft.Office.WebExtension.AsyncResultStatus.Failed,value:1}]});OSF.DDA.SafeArray.Delegate.executeAsync=function(n){function t(n){var i=n,f,r,u;if(OSF.OUtil.isArray(n))for(f=i.length,r=0;r<f;r++)i[r]=t(i[r]);else if(OSF.OUtil.isDate(n))i=n.getVarDate();else if(typeof n=="object"&&!OSF.OUtil.isArray(n)){i=[];for(u in n)OSF.OUtil.isFunction(n[u])||(i[u]=t(n[u]))}return i}function i(n){var u=n,t,r;if(n!=null&&n.toArray){for(t=n.toArray(),r=0;r<t.length;r++)t[r]=i(t[r]);u=t}return u}try{n.onCalling&&n.onCalling();var r=(new Date).getTime();OSF.ClientHostController.execute(n.dispId,t(n.hostCallArgs),function(t){var f=t.toArray(),e=f[OSF.DDA.SafeArray.Response.Status],u,o,s;if(e==OSF.DDA.ErrorCodeManager.errorCodes.ooeChunkResult)u=f[OSF.DDA.SafeArray.Response.Payload],u=i(u),u!=null&&(n._chunkResultData||(n._chunkResultData=[]),n._chunkResultData[u[0]]=u[1]);else{if(n.onReceiving&&n.onReceiving(),n.onComplete){if(e==OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess){if(f.length>2)for(u=[],o=1;o<f.length;o++)u[o-1]=f[o];else u=f[OSF.DDA.SafeArray.Response.Payload];n._chunkResultData&&(u=i(u),u!=null&&(s=u[u.length-1],n._chunkResultData.length==s?u[u.length-1]=n._chunkResultData:e=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError))}else u=f[OSF.DDA.SafeArray.Response.Payload];n.onComplete(e,u)}OSF.AppTelemetry&&OSF.AppTelemetry.onMethodDone(n.dispId,n.hostCallArgs,Math.abs((new Date).getTime()-r),e)}})}catch(u){OSF.DDA.SafeArray.Delegate._onException(u,n)}};OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent=function(n,t){var i=(new Date).getTime();return function(r){t.onReceiving&&t.onReceiving();var u=r.toArray?r.toArray()[OSF.DDA.SafeArray.Response.Status]:r;t.onComplete&&t.onComplete(u);OSF.AppTelemetry&&OSF.AppTelemetry.onRegisterDone(n,t.dispId,Math.abs((new Date).getTime()-i),u)}};OSF.DDA.SafeArray.Delegate.registerEventAsync=function(n){n.onCalling&&n.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!0,n);try{OSF.ClientHostController.registerEvent(n.dispId,n.targetId,function(t,i){n.onEvent&&n.onEvent(i);OSF.AppTelemetry&&OSF.AppTelemetry.onEventDone(n.dispId)},t)}catch(i){OSF.DDA.SafeArray.Delegate._onException(i,n)}};OSF.DDA.SafeArray.Delegate.unregisterEventAsync=function(n){n.onCalling&&n.onCalling();var t=OSF.DDA.SafeArray.Delegate._getOnAfterRegisterEvent(!1,n);try{OSF.ClientHostController.unregisterEvent(n.dispId,n.targetId,t)}catch(i){OSF.DDA.SafeArray.Delegate._onException(i,n)}};OSF.ClientMode={ReadWrite:0,ReadOnly:1};OSF.DDA.RichInitializationReason={1:Microsoft.Office.WebExtension.InitializationReason.Inserted,2:Microsoft.Office.WebExtension.InitializationReason.DocumentOpened};OSF.InitializationHelper=function(n,t,i,r,u){var f=this;f._hostInfo=n;f._webAppState=t;f._context=i;f._settings=r;f._hostFacade=u;f._initializeSettings=f.initializeSettings};OSF.InitializationHelper.prototype.deserializeSettings=function(n,t){var f="undefined",r=OSF.OUtil.getSessionStorage(),i,u;return r&&(i=r.getItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey()),i?n=typeof JSON!==f?JSON.parse(i):OsfMsAjaxFactory.msAjaxSerializer.deserialize(i,!0):(i=typeof JSON!==f?JSON.stringify(n):OsfMsAjaxFactory.msAjaxSerializer.serialize(n),r.setItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey(),i))),u=OSF.DDA.SettingsManager.deserializeSettings(n),t?new OSF.DDA.RefreshableSettings(u):new OSF.DDA.Settings(u)};OSF.InitializationHelper.prototype.setAgaveHostCommunication=function(){};OSF.InitializationHelper.prototype.prepareRightBeforeWebExtensionInitialize=function(n){this.prepareApiSurface(n);Microsoft.Office.WebExtension.initialize(this.getInitializationReason(n))};OSF.InitializationHelper.prototype.prepareApiSurface=function(n){var r=new OSF.DDA.License(n.get_eToken()),u=OSF.DDA.OfficeTheme&&OSF.DDA.OfficeTheme.getOfficeTheme?OSF.DDA.OfficeTheme.getOfficeTheme:null,t,i;OSF._OfficeAppFactory.setContext(new OSF.DDA.Context(n,n.doc,r,null,u));t=OSF.DDA.DispIdHost.getClientDelegateMethods;i=OSF.DDA.SafeArray.Delegate.ParameterMap;OSF._OfficeAppFactory.setHostFacade(new OSF.DDA.DispIdHost.Facade(t,i))};OSF.InitializationHelper.prototype.getInitializationReason=function(n){return OSF.DDA.RichInitializationReason[n.get_reason()]};OSF.DDA.DispIdHost.getClientDelegateMethods=function(n){var t={},i,r;return t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.SafeArray.Delegate.executeAsync,t[OSF.DDA.DispIdHost.Delegates.RegisterEventAsync]=OSF.DDA.SafeArray.Delegate.registerEventAsync,t[OSF.DDA.DispIdHost.Delegates.UnregisterEventAsync]=OSF.DDA.SafeArray.Delegate.unregisterEventAsync,OSF.DDA.AsyncMethodNames.RefreshAsync&&n==OSF.DDA.AsyncMethodNames.RefreshAsync.id&&(i=function(n,t,i){return OSF.DDA.ClientSettingsManager.read(t,i)},t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod(i)),OSF.DDA.AsyncMethodNames.SaveAsync&&n==OSF.DDA.AsyncMethodNames.SaveAsync.id&&(r=function(n,t,i){return OSF.DDA.ClientSettingsManager.write(n[OSF.DDA.SettingsManager.SerializedSettings],n[Microsoft.Office.WebExtension.Parameters.OverwriteIfStale],t,i)},t[OSF.DDA.DispIdHost.Delegates.ExecuteAsync]=OSF.DDA.ClientSettingsManager.getSettingsExecuteMethod(r)),t},function(n){var t=function(){function n(){}return n.prototype.execute=function(n,t,i){window.external.Execute(n,t,i)},n.prototype.registerEvent=function(n,t,i,r){window.external.RegisterEvent(n,t,i,r)},n.prototype.unregisterEvent=function(n,t,i){window.external.UnregisterEvent(n,t,i)},n}();n.RichClientHostController=t}(OSFRichclient||(OSFRichclient={}));OSF.ClientHostController=new OSFRichclient.RichClientHostController,function(n){(function(t){var i=function(){function n(){this._osfOfficeTheme=t;this._osfOfficeThemeTimeStamp=t}var t=null;return n.prototype.getOfficeTheme=function(){var r="GetOfficeThemeInfo",t=this,i;if(OSF.DDA._OsfControlContext){if(t._osfOfficeTheme&&t._osfOfficeThemeTimeStamp&&(new Date).getTime()-t._osfOfficeThemeTimeStamp<n._osfOfficeThemeCacheValidPeriod)OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone(r,0);else{var f=(new Date).getTime(),e=OSF.DDA._OsfControlContext.GetOfficeThemeInfo(),u=(new Date).getTime();OSF.AppTelemetry&&OSF.AppTelemetry.onPropertyDone(r,Math.abs(u-f));t._osfOfficeTheme=JSON.parse(e);for(i in t._osfOfficeTheme)t._osfOfficeTheme[i]=OSF.OUtil.convertIntToCssHexColor(t._osfOfficeTheme[i]);t._osfOfficeThemeTimeStamp=u}return t._osfOfficeTheme}},n.instance=function(){return n._instance==t&&(n._instance=new n),n._instance},n._osfOfficeThemeCacheValidPeriod=5e3,n._instance=t,n}();t.OfficeThemeManager=i;OSF.OUtil.setNamespace("OfficeTheme",OSF.DDA);OSF.DDA.OfficeTheme.getOfficeTheme=n.OfficeTheme.OfficeThemeManager.instance().getOfficeTheme})(n.OfficeTheme||(n.OfficeTheme={}));var t=n.OfficeTheme}(OfficeExt||(OfficeExt={}));OSF.DDA.ClientSettingsManager={getSettingsExecuteMethod:function(n){return function(t){var i,r;try{r=n(t.hostCallArgs,t.onCalling,t.onReceiving);i=OSF.DDA.ErrorCodeManager.errorCodes.ooeSuccess}catch(u){i=OSF.DDA.ErrorCodeManager.errorCodes.ooeInternalError;r={name:Strings.OfficeOM.L_InternalError,message:u}}t.onComplete&&t.onComplete(i,r)}},read:function(n,t){var r=[],f=[],u,i;for(n&&n(),OSF.DDA._OsfControlContext.GetSettings().Read(r,f),t&&t(),u={},i=0;i<r.length;i++)u[r[i]]=f[i];return u},write:function(n,t,i,r){var u=[],f=[];for(var e in n)u.push(e),f.push(n[e]);i&&i();OSF.DDA._OsfControlContext.GetSettings().Write(u,f);r&&r()}};OSF.InitializationHelper.prototype.initializeSettings=function(n){var t=OSF.DDA.ClientSettingsManager.read();return this.deserializeSettings(t,n)};OSF.InitializationHelper.prototype.getAppContext=function(n,t){var r="undefined",f,i=OSF.DDA._OsfControlContext=window.external.GetContext(),e=i.GetAppType(),o=!1,s,h,c,l,a,v;for(s in OSF.AppName)if(OSF.AppName[s]==e){o=!0;break}if(!o)throw"Unsupported client type "+e;var y=i.GetSolutionRef(),p=i.GetAppVersionMajor(),w=i.GetAppVersionMinor(),b=i.GetAppUILocale(),k=i.GetAppDataLocale(),d=i.GetDocUrl(),g=i.GetAppCapabilities(),nt=i.GetActivationMode(),tt=i.GetControlIntegrationLevel(),it=[],u;try{u=i.GetSolutionToken()}catch(rt){}typeof i.GetCorrelationId!==r&&(h=i.GetCorrelationId());typeof i.GetInstanceId!==r&&(c=i.GetInstanceId());typeof i.GetTouchEnabled!==r&&(l=i.GetTouchEnabled());typeof i.GetCommerceAllowed!==r&&(a=i.GetCommerceAllowed());typeof i.GetSupportedMatrix!==r&&(v=i.GetSupportedMatrix());u=u?u.toString():"";f=new OSF.OfficeAppContext(y,e,p,b,k,d,g,it,nt,tt,u,h,c,l,a,w,v);OSF.AppTelemetry&&OSF.AppTelemetry.initialize(f);t(f)},function(n){var u="ResponseTime",i="SessionId",r="CorrelationId",t=!0,f=function(){function n(n){this._table=n;this._fields={}}return Object.defineProperty(n.prototype,"Fields",{get:function(){return this._fields},enumerable:t,configurable:t}),Object.defineProperty(n.prototype,"Table",{get:function(){return this._table},enumerable:t,configurable:t}),n.prototype.SerializeFields=function(){},n.prototype.SetSerializedField=function(n,t){typeof t!="undefined"&&t!==null&&(this._serializedFields[n]=t.toString())},n.prototype.SerializeRow=function(){var n=this;return n._serializedFields={},n.SetSerializedField("Table",n._table),n.SerializeFields(),JSON.stringify(n._serializedFields)},n}(),e,o,s,h,c;n.BaseUsageData=f;e=function(n){function u(){n.call(this,"AppActivated")}var f="AppSizeHeight",e="AppSizeWidth",o="ClientId",s="HostVersion",h="Host",c="UserId",l="Browser",a="AssetId",v="AppURL",y="AppId";return __extends(u,n),Object.defineProperty(u.prototype,r,{get:function(){return this.Fields[r]},set:function(n){this.Fields[r]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,i,{get:function(){return this.Fields[i]},set:function(n){this.Fields[i]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,y,{get:function(){return this.Fields[y]},set:function(n){this.Fields[y]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,v,{get:function(){return this.Fields[v]},set:function(n){this.Fields[v]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,a,{get:function(){return this.Fields[a]},set:function(n){this.Fields[a]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,l,{get:function(){return this.Fields[l]},set:function(n){this.Fields[l]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,c,{get:function(){return this.Fields[c]},set:function(n){this.Fields[c]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,h,{get:function(){return this.Fields[h]},set:function(n){this.Fields[h]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,s,{get:function(){return this.Fields[s]},set:function(n){this.Fields[s]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,o,{get:function(){return this.Fields[o]},set:function(n){this.Fields[o]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,e,{get:function(){return this.Fields[e]},set:function(n){this.Fields[e]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,f,{get:function(){return this.Fields[f]},set:function(n){this.Fields[f]=n},enumerable:t,configurable:t}),u.prototype.SerializeFields=function(){var n=this;n.SetSerializedField(r,n.CorrelationId);n.SetSerializedField(i,n.SessionId);n.SetSerializedField(y,n.AppId);n.SetSerializedField(v,n.AppURL);n.SetSerializedField(a,n.AssetId);n.SetSerializedField(l,n.Browser);n.SetSerializedField(c,n.UserId);n.SetSerializedField(h,n.Host);n.SetSerializedField(s,n.HostVersion);n.SetSerializedField(o,n.ClientId);n.SetSerializedField(e,n.AppSizeWidth);n.SetSerializedField(f,n.AppSizeHeight)},u}(f);n.AppActivatedUsageData=e;o=function(n){function f(){n.call(this,"ScriptLoad")}var e="StartTime",o="ScriptId";return __extends(f,n),Object.defineProperty(f.prototype,r,{get:function(){return this.Fields[r]},set:function(n){this.Fields[r]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,i,{get:function(){return this.Fields[i]},set:function(n){this.Fields[i]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,o,{get:function(){return this.Fields[o]},set:function(n){this.Fields[o]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,e,{get:function(){return this.Fields[e]},set:function(n){this.Fields[e]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,u,{get:function(){return this.Fields[u]},set:function(n){this.Fields[u]=n},enumerable:t,configurable:t}),f.prototype.SerializeFields=function(){var n=this;n.SetSerializedField(r,n.CorrelationId);n.SetSerializedField(i,n.SessionId);n.SetSerializedField(o,n.ScriptId);n.SetSerializedField(e,n.StartTime);n.SetSerializedField(u,n.ResponseTime)},f}(f);n.ScriptLoadUsageData=o;s=function(n){function u(){n.call(this,"AppClosed")}var f="CloseMethod",e="OpenTime",o="AppSizeFinalHeight",s="AppSizeFinalWidth",h="FocusTime";return __extends(u,n),Object.defineProperty(u.prototype,r,{get:function(){return this.Fields[r]},set:function(n){this.Fields[r]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,i,{get:function(){return this.Fields[i]},set:function(n){this.Fields[i]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,h,{get:function(){return this.Fields[h]},set:function(n){this.Fields[h]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,s,{get:function(){return this.Fields[s]},set:function(n){this.Fields[s]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,o,{get:function(){return this.Fields[o]},set:function(n){this.Fields[o]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,e,{get:function(){return this.Fields[e]},set:function(n){this.Fields[e]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,f,{get:function(){return this.Fields[f]},set:function(n){this.Fields[f]=n},enumerable:t,configurable:t}),u.prototype.SerializeFields=function(){var n=this;n.SetSerializedField(r,n.CorrelationId);n.SetSerializedField(i,n.SessionId);n.SetSerializedField(h,n.FocusTime);n.SetSerializedField(s,n.AppSizeFinalWidth);n.SetSerializedField(o,n.AppSizeFinalHeight);n.SetSerializedField(e,n.OpenTime);n.SetSerializedField(f,n.CloseMethod)},u}(f);n.AppClosedUsageData=s;h=function(n){function f(){n.call(this,"APIUsage")}var e="ErrorType",o="Parameters",s="APIID",h="APIType";return __extends(f,n),Object.defineProperty(f.prototype,r,{get:function(){return this.Fields[r]},set:function(n){this.Fields[r]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,i,{get:function(){return this.Fields[i]},set:function(n){this.Fields[i]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,h,{get:function(){return this.Fields[h]},set:function(n){this.Fields[h]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,s,{get:function(){return this.Fields[s]},set:function(n){this.Fields[s]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,o,{get:function(){return this.Fields[o]},set:function(n){this.Fields[o]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,u,{get:function(){return this.Fields[u]},set:function(n){this.Fields[u]=n},enumerable:t,configurable:t}),Object.defineProperty(f.prototype,e,{get:function(){return this.Fields[e]},set:function(n){this.Fields[e]=n},enumerable:t,configurable:t}),f.prototype.SerializeFields=function(){var n=this;n.SetSerializedField(r,n.CorrelationId);n.SetSerializedField(i,n.SessionId);n.SetSerializedField(h,n.APIType);n.SetSerializedField(s,n.APIID);n.SetSerializedField(o,n.Parameters);n.SetSerializedField(u,n.ResponseTime);n.SetSerializedField(e,n.ErrorType)},f}(f);n.APIUsageUsageData=h;c=function(n){function u(){n.call(this,"AppInitialization")}var f="Message",e="SuccessCode";return __extends(u,n),Object.defineProperty(u.prototype,r,{get:function(){return this.Fields[r]},set:function(n){this.Fields[r]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,i,{get:function(){return this.Fields[i]},set:function(n){this.Fields[i]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,e,{get:function(){return this.Fields[e]},set:function(n){this.Fields[e]=n},enumerable:t,configurable:t}),Object.defineProperty(u.prototype,f,{get:function(){return this.Fields[f]},set:function(n){this.Fields[f]=n},enumerable:t,configurable:t}),u.prototype.SerializeFields=function(){var n=this;n.SetSerializedField(r,n.CorrelationId);n.SetSerializedField(i,n.SessionId);n.SetSerializedField(e,n.SuccessCode);n.SetSerializedField(f,n.Message)},u}(f);n.AppInitializationUsageData=c}(OSFLog||(OSFLog={})),function(n){"use strict";function u(){OSF.Logger&&OSF.Logger.ulsEndpoint&&OSF.Logger.ulsEndpoint.loadProxyFrame()}function f(n,t,i){if(OSF.Logger&&OSF.Logger.ulsEndpoint){var r={traceLevel:n,message:t,flag:i,internalLog:!0},u=JSON.stringify(r);OSF.Logger.ulsEndpoint.writeLog(u)}}function e(){try{return new t}catch(n){return null}}var i,r,t;(function(n){n[n.info=0]="info";n[n.warning=1]="warning";n[n.error=2]="error"})(n.TraceLevel||(n.TraceLevel={}));i=n.TraceLevel,function(n){n[n.none=0]="none";n[n.flush=1]="flush"}(n.SendFlag||(n.SendFlag={}));r=n.SendFlag;n.allowUploadingData=u;n.sendLog=f;t=function(){function n(){var n=this,t=n;n.proxyFrame=null;n.telemetryEndPoint="https://telemetryservice.firstpartyapps.oaspapps.com/telemetryservice/telemetryproxy.html";n.buffer=[];n.proxyFrameReady=!1;OSF.OUtil.addEventListener(window,"message",function(n){return t.tellProxyFrameReady(n)});setTimeout(function(){t.loadProxyFrame()},3e3)}return n.prototype.writeLog=function(n){var t=this;t.proxyFrameReady===!0?t.proxyFrame.contentWindow.postMessage(n,"*"):t.buffer.length<128&&t.buffer.push(n)},n.prototype.loadProxyFrame=function(){var n=this;n.proxyFrame==null&&(n.proxyFrame=document.createElement("iframe"),n.proxyFrame.setAttribute("style","display:none"),n.proxyFrame.setAttribute("src",n.telemetryEndPoint),document.head.appendChild(n.proxyFrame))},n.prototype.tellProxyFrameReady=function(n){var t=this,f=t,i,r,u;if(n.data==="ProxyFrameReadyToLog"){for(t.proxyFrameReady=!0,i=0;i<t.buffer.length;i++)t.writeLog(t.buffer[i]);t.buffer.length=0;OSF.OUtil.removeEventListener(window,"message",function(n){return f.tellProxyFrameReady(n)})}else n.data==="ProxyFrameReadyToInit"&&(r={appName:"Office APPs",sessionId:OSF.OUtil.Guid.generateNewGuid()},u=JSON.stringify(r),t.proxyFrame.contentWindow.postMessage(u,"*"))},n}();OSF.Logger||(OSF.Logger=n);n.ulsEndpoint=e()}(Logger||(Logger={})),function(n){function c(r){if(OSF.Logger&&!t){t=new h;t.hostVersion=r.get_appVersion();t.appId=r.get_id();t.host=r.get_appName();t.browser=window.navigator.userAgent;t.correlationId=r.get_correlationId();t.clientId=(new o).getClientId();var f=location.href.indexOf("?");t.appURL=f==-1?location.href:location.href.substring(0,f),function(n,t){var u,f,r;t.assetId="";t.userId="";try{u=decodeURIComponent(n);f=new DOMParser;r=f.parseFromString(u,"text/xml");t.userId=r.getElementsByTagName("t")[0].attributes.getNamedItem("cid").nodeValue;t.assetId=r.getElementsByTagName("t")[0].attributes.getNamedItem("aid").nodeValue}catch(e){}finally{u=i;r=i;f=i}}(r.get_eToken(),t),function(){var c=new Date,r=i,o=0,h=!1,f=function(){document.hasFocus()?r==i&&(r=new Date):r&&(o+=Math.abs((new Date).getTime()-r.getTime()),r=i)},t=[],s,e;for(t.push(new u("focus",f)),t.push(new u("blur",f)),t.push(new u("focusout",f)),t.push(new u("focusin",f)),s=function(){for(var u=0;u<t.length;u++)OSF.OUtil.removeEventListener(window,t[u].name,t[u].handler);if(t.length=0,!h){document.hasFocus()&&r&&(o+=Math.abs((new Date).getTime()-r.getTime()),r=i);n.onAppClosed(Math.abs((new Date).getTime()-c.getTime()),o);h=!0}},t.push(new u("beforeunload",s)),t.push(new u("unload",s)),e=0;e<t.length;e++)OSF.OUtil.addEventListener(window,t[e].name,t[e].handler);f()}();n.onAppActivated()}}function l(){if(t){(new o).enumerateLog(function(n,t){return(new f).LogRawData(t)},!0);var n=new OSFLog.AppActivatedUsageData;n.SessionId=r;n.AppId=t.appId;n.AssetId=t.assetId;n.AppURL=t.appURL;n.UserId=t.userId;n.ClientId=t.clientId;n.Browser=t.browser;n.Host=t.host;n.HostVersion=t.hostVersion;n.CorrelationId=t.correlationId;n.AppSizeWidth=window.innerWidth;n.AppSizeHeight=window.innerHeight;(new f).LogData(n);setTimeout(function(){OSF.Logger&&OSF.Logger.allowUploadingData()},100)}}function a(n,t,i,u){var e=new OSFLog.ScriptLoadUsageData;e.CorrelationId=u;e.SessionId=r;e.ScriptId=n;e.StartTime=t;e.ResponseTime=i;(new f).LogData(e)}function v(n,i,u,o,s){if(t){var h=new OSFLog.APIUsageUsageData;h.CorrelationId=e;h.SessionId=r;h.APIType=n;h.APIID=i;h.Parameters=u;h.ResponseTime=o;h.ErrorType=s;(new f).LogData(h)}}function y(n,t,r,u){var f=i,e;if(t)if(typeof t=="number")f=String(t);else if(typeof t=="object")for(e in t)f!==i?f+=",":f="",typeof t[e]=="number"&&(f+=String(t[e]));else f="";OSF.AppTelemetry.onCallDone("method",n,f,r,u)}function p(n,t){OSF.AppTelemetry.onCallDone("property",-1,n,t)}function w(n,t){OSF.AppTelemetry.onCallDone("event",n,i,0,t)}function b(n,t,r,u){OSF.AppTelemetry.onCallDone(n?"registerevent":"unregisterevent",t,i,r,u)}function k(n,i){if(t){var u=new OSFLog.AppClosedUsageData;u.CorrelationId=e;u.SessionId=r;u.FocusTime=i;u.OpenTime=n;u.AppSizeFinalWidth=window.innerWidth;u.AppSizeFinalHeight=window.innerHeight;(new o).saveLog(r,u.SerializeRow())}}function d(n){e=n}function s(n,t){var i=new OSFLog.AppInitializationUsageData;i.CorrelationId=e;i.SessionId=r;i.SuccessCode=n?1:0;i.Message=t;(new f).LogData(i)}function g(n){s(!1,n)}function nt(n){s(!0,n)}var i=null;"use strict";var t,r=OSF.OUtil.Guid.generateNewGuid(),e="",h=function(){function n(){}return n}(),u=function(){function n(n,t){this.name=n;this.handler=t}return n}(),o=function(){function n(){this.clientIDKey="Office API client";this.logIdSetKey="Office App Log Id Set"}return n.prototype.getClientId=function(){var t=this,n=t.getValue(t.clientIDKey);return(!n||n.length<=0||n.length>40)&&(n=OSF.OUtil.Guid.generateNewGuid(),t.setValue(t.clientIDKey,n)),n},n.prototype.saveLog=function(n,t){var i=this,r=i.getValue(i.logIdSetKey);r=(r&&r.length>0?r+";":"")+n;i.setValue(i.logIdSetKey,r);i.setValue(n,t)},n.prototype.enumerateLog=function(n,t){var i=this,e=i.getValue(i.logIdSetKey),u,o,r,f;if(e){u=e.split(";");for(o in u)r=u[o],f=i.getValue(r),f&&(n&&n(r,f),t&&i.remove(r));t&&i.remove(i.logIdSetKey)}},n.prototype.getValue=function(n){var t=OSF.OUtil.getLocalStorage(),i="";return t&&(i=t.getItem(n)),i},n.prototype.setValue=function(n,t){var i=OSF.OUtil.getLocalStorage();i&&i.setItem(n,t)},n.prototype.remove=function(n){var t=OSF.OUtil.getLocalStorage();if(t)try{t.removeItem(n)}catch(i){}},n}(),f=function(){function n(){}return n.prototype.LogData=function(n){OSF.Logger&&OSF.Logger.sendLog(OSF.Logger.TraceLevel.info,n.SerializeRow(),OSF.Logger.SendFlag.none)},n.prototype.LogRawData=function(n){OSF.Logger&&OSF.Logger.sendLog(OSF.Logger.TraceLevel.info,n,OSF.Logger.SendFlag.none)},n}();n.initialize=c;n.onAppActivated=l;n.onScriptDone=a;n.onCallDone=v;n.onMethodDone=y;n.onPropertyDone=p;n.onEventDone=w;n.onRegisterDone=b;n.onAppClosed=k;n.setOsfControlAppCorrelationId=d;n.doAppInitializationLogging=s;n.logAppCommonMessage=g;n.logAppException=nt;OSF.AppTelemetry=n}(OSFAppTelemetry||(OSFAppTelemetry={}));Microsoft.Office.WebExtension.TableData=function(n,t){function i(n){if(n==null||n==undefined)return null;try{for(var t=OSF.DDA.DataCoercion.findArrayDimensionality(n,2);t<2;t++)n=[n];return n}catch(i){}}OSF.OUtil.defineEnumerableProperties(this,{headers:{get:function(){return t},set:function(n){t=i(n)}},rows:{get:function(){return n},set:function(t){n=t==null||OSF.OUtil.isArray(t)&&t.length==0?[]:i(t)}}});this.headers=t;this.rows=n};OSF.DDA.OMFactory=OSF.DDA.OMFactory||{};OSF.DDA.OMFactory.manufactureTableData=function(n){return new Microsoft.Office.WebExtension.TableData(n[OSF.DDA.TableDataProperties.TableRows],n[OSF.DDA.TableDataProperties.TableHeaders])};Microsoft.Office.WebExtension.CoercionType={Text:"text",Matrix:"matrix",Table:"table"};OSF.DDA.DataCoercion=function(){var n=null;return{findArrayDimensionality:function(n){if(OSF.OUtil.isArray(n)){for(var t=0,i=0;i<n.length;i++)t=Math.max(t,OSF.DDA.DataCoercion.findArrayDimensionality(n[i]));return t+1}else return 0},getCoercionDefaultForBinding:function(n){switch(n){case Microsoft.Office.WebExtension.BindingType.Matrix:return Microsoft.Office.WebExtension.CoercionType.Matrix;case Microsoft.Office.WebExtension.BindingType.Table:return Microsoft.Office.WebExtension.CoercionType.Table;case Microsoft.Office.WebExtension.BindingType.Text:default:return Microsoft.Office.WebExtension.CoercionType.Text}},getBindingDefaultForCoercion:function(n){switch(n){case Microsoft.Office.WebExtension.CoercionType.Matrix:return Microsoft.Office.WebExtension.BindingType.Matrix;case Microsoft.Office.WebExtension.CoercionType.Table:return Microsoft.Office.WebExtension.BindingType.Table;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:return Microsoft.Office.WebExtension.BindingType.Text}},determineCoercionType:function(t){if(t==n||t==undefined)return n;var i=n,r=typeof t;if(t.rows!==undefined)i=Microsoft.Office.WebExtension.CoercionType.Table;else if(OSF.OUtil.isArray(t))i=Microsoft.Office.WebExtension.CoercionType.Matrix;else if(r=="string"||r=="number"||r=="boolean"||OSF.OUtil.isDate(t))i=Microsoft.Office.WebExtension.CoercionType.Text;else throw OSF.DDA.ErrorCodeManager.errorCodes.ooeUnsupportedDataObject;return i},coerceData:function(n,t,i){return i=i||OSF.DDA.DataCoercion.determineCoercionType(n),i&&i!=t&&(OSF.OUtil.writeProfilerMark(OSF.InternalPerfMarker.DataCoercionBegin),n=OSF.DDA.DataCoercion._coerceDataFromTable(t,OSF.DDA.DataCoercion._coerceDataToTable(n,i)),OSF.OUtil.writeProfilerMark(OSF.InternalPerfMarker.DataCoercionEnd)),n},_matrixToText:function(n){if(n.length==1&&n[0].length==1)return""+n[0][0];for(var t="",i=0;i<n.length;i++)t+=n[i].join("\t")+"\n";return t.substring(0,t.length-1)},_textToMatrix:function(n){for(var t=n.split("\n"),i=0;i<t.length;i++)t[i]=t[i].split("\t");return t},_tableToText:function(t){var i="",r;return t.headers!=n&&(i=OSF.DDA.DataCoercion._matrixToText([t.headers])+"\n"),r=OSF.DDA.DataCoercion._matrixToText(t.rows),r==""&&(i=i.substring(0,i.length-1)),i+r},_tableToMatrix:function(t){var i=t.rows;return t.headers!=n&&i.unshift(t.headers),i},_coerceDataFromTable:function(t,i){var r;switch(t){case Microsoft.Office.WebExtension.CoercionType.Table:r=i;break;case Microsoft.Office.WebExtension.CoercionType.Matrix:r=OSF.DDA.DataCoercion._tableToMatrix(i);break;case Microsoft.Office.WebExtension.CoercionType.SlideRange:r=n;OSF.DDA.OMFactory.manufactureSlideRange&&(r=OSF.DDA.OMFactory.manufactureSlideRange(OSF.DDA.DataCoercion._tableToText(i)));r==n&&(r=OSF.DDA.DataCoercion._tableToText(i));break;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:r=OSF.DDA.DataCoercion._tableToText(i)}return r},_coerceDataToTable:function(n,t){t==undefined&&(t=OSF.DDA.DataCoercion.determineCoercionType(n));var i;switch(t){case Microsoft.Office.WebExtension.CoercionType.Table:i=n;break;case Microsoft.Office.WebExtension.CoercionType.Matrix:i=new Microsoft.Office.WebExtension.TableData(n);break;case Microsoft.Office.WebExtension.CoercionType.Text:case Microsoft.Office.WebExtension.CoercionType.Html:case Microsoft.Office.WebExtension.CoercionType.Ooxml:default:i=new Microsoft.Office.WebExtension.TableData(OSF.DDA.DataCoercion._textToMatrix(n))}return i}}}();OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.CoercionType,toHost:[{name:Microsoft.Office.WebExtension.CoercionType.Text,value:0},{name:Microsoft.Office.WebExtension.CoercionType.Matrix,value:1},{name:Microsoft.Office.WebExtension.CoercionType.Table,value:2}]});OSF.DDA.AsyncMethodNames.addNames({GetSelectedDataAsync:"getSelectedDataAsync",SetSelectedDataAsync:"setSelectedDataAsync"}),function(){function r(n,t,i){var r=n[Microsoft.Office.WebExtension.Parameters.Data];return OSF.DDA.TableDataProperties&&r&&(r[OSF.DDA.TableDataProperties.TableRows]!=undefined||r[OSF.DDA.TableDataProperties.TableHeaders]!=undefined)&&(r=OSF.DDA.OMFactory.manufactureTableData(r)),r=OSF.DDA.DataCoercion.coerceData(r,i[Microsoft.Office.WebExtension.Parameters.CoercionType]),r==undefined?null:r}var i=!1,n="boolean",t="number";OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,"enum":Microsoft.Office.WebExtension.CoercionType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:{"enum":Microsoft.Office.WebExtension.ValueFormat,defaultValue:Microsoft.Office.WebExtension.ValueFormat.Unformatted}},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:{"enum":Microsoft.Office.WebExtension.FilterType,defaultValue:Microsoft.Office.WebExtension.FilterType.All}}],privateStateCallbacks:[],onSucceeded:r});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["string","object",t,n]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{"enum":Microsoft.Office.WebExtension.CoercionType,calculate:function(n){return OSF.DDA.DataCoercion.determineCoercionType(n[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.ImageLeft,value:{types:[t,n],defaultValue:i}},{name:Microsoft.Office.WebExtension.Parameters.ImageTop,value:{types:[t,n],defaultValue:i}},{name:Microsoft.Office.WebExtension.Parameters.ImageWidth,value:{types:[t,n],defaultValue:i}},{name:Microsoft.Office.WebExtension.Parameters.ImageHeight,value:{types:[t,n],defaultValue:i}}],privateStateCallbacks:[]})}();OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetSelectedDataMethod,fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:1},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:2}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetSelectedDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1},{name:Microsoft.Office.WebExtension.Parameters.ImageLeft,value:2},{name:Microsoft.Office.WebExtension.Parameters.ImageTop,value:3},{name:Microsoft.Office.WebExtension.Parameters.ImageWidth,value:4},{name:Microsoft.Office.WebExtension.Parameters.ImageHeight,value:5}]});OSF.DDA.SettingsManager={SerializedSettings:"serializedSettings",DateJSONPrefix:"Date(",DataJSONSuffix:")",serializeSettings:function(n){var r={},i,t;for(i in n){t=n[i];try{t=JSON?JSON.stringify(t,function(n,t){return OSF.OUtil.isDate(this[n])?OSF.DDA.SettingsManager.DateJSONPrefix+this[n].getTime()+OSF.DDA.SettingsManager.DataJSONSuffix:t}):Sys.Serialization.JavaScriptSerializer.serialize(t);r[i]=t}catch(u){}}return r},deserializeSettings:function(n){var r={},i,t;n=n||{};for(i in n){t=n[i];try{t=JSON?JSON.parse(t,function(n,t){var i;return typeof t=="string"&&t&&t.length>6&&t.slice(0,5)===OSF.DDA.SettingsManager.DateJSONPrefix&&t.slice(-1)===OSF.DDA.SettingsManager.DataJSONSuffix&&(i=new Date(parseInt(t.slice(5,-1))),i)?i:t}):Sys.Serialization.JavaScriptSerializer.deserialize(t,!0);r[i]=t}catch(u){}}return r}};OSF.DDA.Settings=function(n){var t="name",i;n=n||{};i=function(n){var i=OSF.OUtil.getSessionStorage(),t,r;i&&(t=OSF.DDA.SettingsManager.serializeSettings(n),r=JSON?JSON.stringify(t):Sys.Serialization.JavaScriptSerializer.serialize(t),i.setItem(OSF._OfficeAppFactory.getCachedSessionSettingsKey(),r))};OSF.OUtil.defineEnumerableProperties(this,{get:{value:function(i){var u=Function._validateParams(arguments,[{name:t,type:String,mayBeNull:!1}]),r;if(u)throw u;return r=n[i],typeof r=="undefined"?null:r}},set:{value:function(r,u){var f=Function._validateParams(arguments,[{name:t,type:String,mayBeNull:!1},{name:"value",mayBeNull:!0}]);if(f)throw f;n[r]=u;i(n)}},remove:{value:function(r){var u=Function._validateParams(arguments,[{name:t,type:String,mayBeNull:!1}]);if(u)throw u;delete n[r];i(n)}}});OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.SaveAsync],n)};OSF.DDA.RefreshableSettings=function(n){OSF.DDA.RefreshableSettings.uber.constructor.call(this,n);OSF.DDA.DispIdHost.addAsyncMethods(this,[OSF.DDA.AsyncMethodNames.RefreshAsync],n);OSF.DDA.DispIdHost.addEventSupport(this,new OSF.EventDispatch([Microsoft.Office.WebExtension.EventType.SettingsChanged]))};OSF.OUtil.extend(OSF.DDA.RefreshableSettings,OSF.DDA.Settings);Microsoft.Office.WebExtension.EventType={};OSF.EventDispatch=function(n){var t,i;this._eventHandlers={};for(t in n)i=n[t],this._eventHandlers[i]=[]};OSF.EventDispatch.prototype={getSupportedEvents:function(){var n=[];for(var t in this._eventHandlers)n.push(t);return n},supportsEvent:function(n){var t=!1;for(var i in this._eventHandlers)if(n==i){t=!0;break}return t},hasEventHandler:function(n,t){var i=this._eventHandlers[n],r;if(i&&i.length>0)for(r in i)if(i[r]===t)return!0;return!1},addEventHandler:function(n,t){if(typeof t!="function")return!1;var i=this._eventHandlers[n];return i&&!this.hasEventHandler(n,t)?(i.push(t),!0):!1},removeEventHandler:function(n,t){var i=this._eventHandlers[n],r;if(i&&i.length>0)for(r=0;r<i.length;r++)if(i[r]===t)return i.splice(r,1),!0;return!1},clearEventHandlers:function(n){return typeof this._eventHandlers[n]!="undefined"&&this._eventHandlers[n].length>0?(this._eventHandlers[n]=[],!0):!1},getEventHandlerCount:function(n){return this._eventHandlers[n]!=undefined?this._eventHandlers[n].length:-1},fireEvent:function(n){var t,i,r;if(n.type==undefined)return!1;if(t=n.type,t&&this._eventHandlers[t]){i=this._eventHandlers[t];for(r in i)i[r](n);return!0}else return!1}};OSF.DDA.OMFactory=OSF.DDA.OMFactory||{};OSF.DDA.OMFactory.manufactureEventArgs=function(n,t,i){var u=this,r;switch(n){case Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged:r=new OSF.DDA.DocumentSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.BindingSelectionChanged:r=new OSF.DDA.BindingSelectionChangedEventArgs(u.manufactureBinding(i,t.document),i[OSF.DDA.PropertyDescriptors.Subset]);break;case Microsoft.Office.WebExtension.EventType.BindingDataChanged:r=new OSF.DDA.BindingDataChangedEventArgs(u.manufactureBinding(i,t.document));break;case Microsoft.Office.WebExtension.EventType.SettingsChanged:r=new OSF.DDA.SettingsChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ActiveViewChanged:r=new OSF.DDA.ActiveViewChangedEventArgs(i);break;case Microsoft.Office.WebExtension.EventType.OfficeThemeChanged:r=new OSF.DDA.Theming.OfficeThemeChangedEventArgs(i);break;case Microsoft.Office.WebExtension.EventType.DocumentThemeChanged:r=new OSF.DDA.Theming.DocumentThemeChangedEventArgs(i);break;case Microsoft.Office.WebExtension.EventType.AppCommandInvoked:r=OSF.DDA.AppCommand.AppCommandInvokedEventArgs.create(i);break;case Microsoft.Office.WebExtension.EventType.DataNodeInserted:r=new OSF.DDA.NodeInsertedEventArgs(u.manufactureDataNode(i[OSF.DDA.DataNodeEventProperties.NewNode]),i[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeReplaced:r=new OSF.DDA.NodeReplacedEventArgs(u.manufactureDataNode(i[OSF.DDA.DataNodeEventProperties.OldNode]),u.manufactureDataNode(i[OSF.DDA.DataNodeEventProperties.NewNode]),i[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.DataNodeDeleted:r=new OSF.DDA.NodeDeletedEventArgs(u.manufactureDataNode(i[OSF.DDA.DataNodeEventProperties.OldNode]),u.manufactureDataNode(i[OSF.DDA.DataNodeEventProperties.NextSiblingNode]),i[OSF.DDA.DataNodeEventProperties.InUndoRedo]);break;case Microsoft.Office.WebExtension.EventType.TaskSelectionChanged:r=new OSF.DDA.TaskSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ResourceSelectionChanged:r=new OSF.DDA.ResourceSelectionChangedEventArgs(t);break;case Microsoft.Office.WebExtension.EventType.ViewSelectionChanged:r=new OSF.DDA.ViewSelectionChangedEventArgs(t);break;default:throw OsfMsAjaxFactory.msAjaxError.argument(Microsoft.Office.WebExtension.Parameters.EventType,OSF.OUtil.formatString(Strings.OfficeOM.L_NotSupportedEventType,n));}return r};OSF.DDA.AsyncMethodNames.addNames({AddHandlerAsync:"addHandlerAsync",RemoveHandlerAsync:"removeHandlerAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,"enum":Microsoft.Office.WebExtension.EventType,verify:function(n,t,i){return i.supportsEvent(n)}},{name:Microsoft.Office.WebExtension.Parameters.Handler,types:["function"]}],supportedOptions:[],privateStateCallbacks:[]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.RemoveHandlerAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.EventType,"enum":Microsoft.Office.WebExtension.EventType,verify:function(n,t,i){return i.supportsEvent(n)}}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Handler,value:{types:["function","object"],defaultValue:null}}],privateStateCallbacks:[]});OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{SettingsChanged:"settingsChanged"});OSF.DDA.SettingsChangedEventArgs=function(n){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.SettingsChanged},settings:{value:n}})};OSF.DDA.AsyncMethodNames.addNames({RefreshAsync:"refreshAsync",SaveAsync:"saveAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.RefreshAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[],onSucceeded:function(n){var t=n[OSF.DDA.SettingsManager.SerializedSettings];return OSF.DDA.SettingsManager.deserializeSettings(t)}});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SaveAsync,requiredArguments:[],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale,value:{types:["boolean"],defaultValue:!0}}],privateStateCallbacks:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:function(n,t){return OSF.DDA.SettingsManager.serializeSettings(t)}}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidLoadSettingsMethod,fromHost:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSaveSettingsMethod,toHost:[{name:OSF.DDA.SettingsManager.SerializedSettings,value:OSF.DDA.SettingsManager.SerializedSettings},{name:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale,value:Microsoft.Office.WebExtension.Parameters.OverwriteIfStale}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidSettingsChangedEvent});Microsoft.Office.WebExtension.BindingType={Table:"table",Text:"text",Matrix:"matrix"};OSF.DDA.BindingProperties={Id:"BindingId",Type:Microsoft.Office.WebExtension.Parameters.BindingType};OSF.OUtil.augmentList(OSF.DDA.ListDescriptors,{BindingList:"BindingList"});OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{Subset:"subset",BindingProperties:"BindingProperties"});OSF.DDA.ListType.setListType(OSF.DDA.ListDescriptors.BindingList,OSF.DDA.PropertyDescriptors.BindingProperties);OSF.DDA.BindingPromise=function(n,t){this._id=n;OSF.OUtil.defineEnumerableProperty(this,"onFail",{get:function(){return t},set:function(n){var i=typeof n;if(i!="undefined"&&i!="function")throw OSF.OUtil.formatString(Strings.OfficeOM.L_CallbackNotAFunction,i);t=n}})};OSF.DDA.BindingPromise.prototype={_fetch:function(n){var t=this,i;return t.binding?n&&n(t.binding):t._binding||(i=t,Microsoft.Office.WebExtension.context.document.bindings.getByIdAsync(t._id,function(t){t.status==Microsoft.Office.WebExtension.AsyncResultStatus.Succeeded?(OSF.OUtil.defineEnumerableProperty(i,"binding",{value:t.value}),n&&n(i.binding)):i.onFail&&i.onFail(t)})),t},getDataAsync:function(){var n=arguments;return this._fetch(function(t){t.getDataAsync.apply(t,n)}),this},setDataAsync:function(){var n=arguments;return this._fetch(function(t){t.setDataAsync.apply(t,n)}),this},addHandlerAsync:function(){var n=arguments;return this._fetch(function(t){t.addHandlerAsync.apply(t,n)}),this},removeHandlerAsync:function(){var n=arguments;return this._fetch(function(t){t.removeHandlerAsync.apply(t,n)}),this}};OSF.DDA.BindingFacade=function(n){this._eventDispatches=[];OSF.OUtil.defineEnumerableProperty(this,"document",{value:n});var t=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[t.AddFromSelectionAsync,t.AddFromNamedItemAsync,t.GetAllAsync,t.GetByIdAsync,t.ReleaseByIdAsync])};OSF.DDA.UnknownBinding=function(n,t){OSF.OUtil.defineEnumerableProperties(this,{document:{value:t},id:{value:n}})};OSF.DDA.Binding=function(n,t){var r,u,i,f;OSF.OUtil.defineEnumerableProperties(this,{document:{value:t},id:{value:n}});r=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[r.GetDataAsync,r.SetDataAsync]);u=Microsoft.Office.WebExtension.EventType;i=t.bindings._eventDispatches;i[n]||(i[n]=new OSF.EventDispatch([u.BindingSelectionChanged,u.BindingDataChanged]));f=i[n];OSF.DDA.DispIdHost.addEventSupport(this,f)};OSF.DDA.generateBindingId=function(){return"UnnamedBinding_"+OSF.OUtil.getUniqueId()+"_"+(new Date).getTime()};OSF.DDA.OMFactory=OSF.DDA.OMFactory||{};OSF.DDA.OMFactory.manufactureBinding=function(n,t){var r=n[OSF.DDA.BindingProperties.Id],u=n[OSF.DDA.BindingProperties.RowCount],f=n[OSF.DDA.BindingProperties.ColumnCount],s=n[OSF.DDA.BindingProperties.HasHeaders],i,e,o;switch(n[OSF.DDA.BindingProperties.Type]){case Microsoft.Office.WebExtension.BindingType.Text:i=new OSF.DDA.TextBinding(r,t);break;case Microsoft.Office.WebExtension.BindingType.Matrix:i=new OSF.DDA.MatrixBinding(r,t,u,f);break;case Microsoft.Office.WebExtension.BindingType.Table:e=function(){return OSF.DDA.ExcelDocument&&Microsoft.Office.WebExtension.context.document&&Microsoft.Office.WebExtension.context.document instanceof OSF.DDA.ExcelDocument};o=e()&&OSF.DDA.ExcelTableBinding?OSF.DDA.ExcelTableBinding:OSF.DDA.TableBinding;i=new o(r,t,u,f,s);break;default:i=new OSF.DDA.UnknownBinding(r,t)}return i};OSF.DDA.AsyncMethodNames.addNames({AddFromSelectionAsync:"addFromSelectionAsync",AddFromNamedItemAsync:"addFromNamedItemAsync",GetAllAsync:"getAllAsync",GetByIdAsync:"getByIdAsync",ReleaseByIdAsync:"releaseByIdAsync",GetDataAsync:"getDataAsync",SetDataAsync:"setDataAsync"}),function(){function u(n){return OSF.DDA.OMFactory.manufactureBinding(n,Microsoft.Office.WebExtension.context.document)}function f(n){return n.id}function e(n,t,i){var u=n[Microsoft.Office.WebExtension.Parameters.Data];return OSF.DDA.TableDataProperties&&u&&(u[OSF.DDA.TableDataProperties.TableRows]!=undefined||u[OSF.DDA.TableDataProperties.TableHeaders]!=undefined)&&(u=OSF.DDA.OMFactory.manufactureTableData(u)),u=OSF.DDA.DataCoercion.coerceData(u,i[Microsoft.Office.WebExtension.Parameters.CoercionType]),u==undefined?r:u}var t="number",i="object",n="string",r=null;OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromSelectionAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.BindingType,"enum":Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:[n],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:[i],defaultValue:r}}],privateStateCallbacks:[],onSucceeded:u});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromNamedItemAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.ItemName,types:[n]},{name:Microsoft.Office.WebExtension.Parameters.BindingType,"enum":Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:[n],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:[i],defaultValue:r}}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.FailOnCollision,value:function(){return!0}}],onSucceeded:u});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetAllAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[],onSucceeded:function(n){return OSF.OUtil.mapList(n[OSF.DDA.ListDescriptors.BindingList],u)}});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:[n]}],supportedOptions:[],privateStateCallbacks:[],onSucceeded:u});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ReleaseByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:[n]}],supportedOptions:[],privateStateCallbacks:[],onSucceeded:function(n,t,i){var r=i[Microsoft.Office.WebExtension.Parameters.Id];delete t._eventDispatches[r]}});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetDataAsync,requiredArguments:[],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{"enum":Microsoft.Office.WebExtension.CoercionType,calculate:function(n,t){return OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)}}},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:{"enum":Microsoft.Office.WebExtension.ValueFormat,defaultValue:Microsoft.Office.WebExtension.ValueFormat.Unformatted}},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:{"enum":Microsoft.Office.WebExtension.FilterType,defaultValue:Microsoft.Office.WebExtension.FilterType.All}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:[i,n],defaultValue:r}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:[i],defaultValue:r}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.RowCount,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.ColumnCount,value:{types:[t],defaultValue:0}}],checkCallArgs:function(n,t){if(n[Microsoft.Office.WebExtension.Parameters.StartRow]==0&&n[Microsoft.Office.WebExtension.Parameters.StartColumn]==0&&n[Microsoft.Office.WebExtension.Parameters.RowCount]==0&&n[Microsoft.Office.WebExtension.Parameters.ColumnCount]==0&&(delete n[Microsoft.Office.WebExtension.Parameters.StartRow],delete n[Microsoft.Office.WebExtension.Parameters.StartColumn],delete n[Microsoft.Office.WebExtension.Parameters.RowCount],delete n[Microsoft.Office.WebExtension.Parameters.ColumnCount]),n[Microsoft.Office.WebExtension.Parameters.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(n[Microsoft.Office.WebExtension.Parameters.StartRow]||n[Microsoft.Office.WebExtension.Parameters.StartColumn]||n[Microsoft.Office.WebExtension.Parameters.RowCount]||n[Microsoft.Office.WebExtension.Parameters.ColumnCount]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return n},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:f}],onSucceeded:e});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:[n,i,t,"boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{"enum":Microsoft.Office.WebExtension.CoercionType,calculate:function(n){return OSF.DDA.DataCoercion.determineCoercionType(n[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:[i,n],defaultValue:r}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:[i],defaultValue:r}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:[t],defaultValue:0}}],checkCallArgs:function(n,t){if(n[Microsoft.Office.WebExtension.Parameters.StartRow]==0&&n[Microsoft.Office.WebExtension.Parameters.StartColumn]==0&&(delete n[Microsoft.Office.WebExtension.Parameters.StartRow],delete n[Microsoft.Office.WebExtension.Parameters.StartColumn]),n[Microsoft.Office.WebExtension.Parameters.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(n[Microsoft.Office.WebExtension.Parameters.StartRow]||n[Microsoft.Office.WebExtension.Parameters.StartColumn]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return n},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:f}]})}();OSF.OUtil.augmentList(OSF.DDA.BindingProperties,{RowCount:"BindingRowCount",ColumnCount:"BindingColumnCount",HasHeaders:"HasHeaders"});OSF.DDA.MatrixBinding=function(n,t,i,r){OSF.DDA.MatrixBinding.uber.constructor.call(this,n,t);OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.BindingType.Matrix},rowCount:{value:i?i:0},columnCount:{value:r?r:0}})};OSF.OUtil.extend(OSF.DDA.MatrixBinding,OSF.DDA.Binding);OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.BindingProperties,fromHost:[{name:OSF.DDA.BindingProperties.Id,value:0},{name:OSF.DDA.BindingProperties.Type,value:1},{name:OSF.DDA.SafeArray.UniqueArguments.BindingSpecificData,value:2}],isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.BindingType,toHost:[{name:Microsoft.Office.WebExtension.BindingType.Text,value:0},{name:Microsoft.Office.WebExtension.BindingType.Matrix,value:1},{name:Microsoft.Office.WebExtension.BindingType.Table,value:2}],invertible:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromSelectionMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:1}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromNamedItemMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.ItemName,value:0},{name:Microsoft.Office.WebExtension.Parameters.Id,value:1},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:2},{name:Microsoft.Office.WebExtension.Parameters.FailOnCollision,value:3}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidReleaseBindingMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetBindingMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetAllBindingsMethod,fromHost:[{name:OSF.DDA.ListDescriptors.BindingList,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetBindingDataMethod,fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.ValueFormat,value:2},{name:Microsoft.Office.WebExtension.Parameters.FilterType,value:3},{name:OSF.DDA.PropertyDescriptors.Subset,value:4}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetBindingDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.Data,value:2},{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:3}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.BindingSpecificData,fromHost:[{name:OSF.DDA.BindingProperties.RowCount,value:0},{name:OSF.DDA.BindingProperties.ColumnCount,value:1},{name:OSF.DDA.BindingProperties.HasHeaders,value:2}],isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.Subset,toHost:[{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:0},{name:OSF.DDA.SafeArray.UniqueArguments.Run,value:1}],canonical:!0,isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.Offset,toHost:[{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:0},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:1}],canonical:!0,isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.SafeArray.UniqueArguments.Run,toHost:[{name:Microsoft.Office.WebExtension.Parameters.RowCount,value:0},{name:Microsoft.Office.WebExtension.Parameters.ColumnCount,value:1}],canonical:!0,isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddRowsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddColumnsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidClearAllRowsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]});OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{TableDataProperties:"TableDataProperties"});OSF.OUtil.augmentList(OSF.DDA.BindingProperties,{RowCount:"BindingRowCount",ColumnCount:"BindingColumnCount",HasHeaders:"HasHeaders"});OSF.DDA.TableDataProperties={TableRows:"TableRows",TableHeaders:"TableHeaders"};OSF.DDA.TableBinding=function(n,t,i,r,u){OSF.DDA.TableBinding.uber.constructor.call(this,n,t);OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.BindingType.Table},rowCount:{value:i?i:0},columnCount:{value:r?r:0},hasHeaders:{value:u?u:!1}});var f=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[f.AddRowsAsync,f.AddColumnsAsync,f.DeleteAllDataValuesAsync])};OSF.OUtil.extend(OSF.DDA.TableBinding,OSF.DDA.Binding);OSF.DDA.AsyncMethodNames.addNames({AddRowsAsync:"addRowsAsync",AddColumnsAsync:"addColumnsAsync",DeleteAllDataValuesAsync:"deleteAllDataValuesAsync"}),function(){function n(n){return n.id}OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddRowsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddColumnsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.DeleteAllDataValuesAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]})}();OSF.DDA.TextBinding=function(n,t){OSF.DDA.TextBinding.uber.constructor.call(this,n,t);OSF.OUtil.defineEnumerableProperty(this,"type",{value:Microsoft.Office.WebExtension.BindingType.Text})};OSF.OUtil.extend(OSF.DDA.TextBinding,OSF.DDA.Binding);OSF.DDA.AsyncMethodNames.addNames({AddFromPromptAsync:"addFromPromptAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.AddFromPromptAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.BindingType,"enum":Microsoft.Office.WebExtension.BindingType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:{types:["string"],calculate:OSF.DDA.generateBindingId}},{name:Microsoft.Office.WebExtension.Parameters.PromptText,value:{types:["string"],calculate:function(){return Strings.OfficeOM.L_AddBindingFromPromptDefaultText}}},{name:Microsoft.Office.WebExtension.Parameters.SampleData,value:{types:["object"],defaultValue:null}}],privateStateCallbacks:[],onSucceeded:function(n){return OSF.DDA.OMFactory.manufactureBinding(n,Microsoft.Office.WebExtension.context.document)}});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidAddBindingFromPromptMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.BindingType,value:1},{name:Microsoft.Office.WebExtension.Parameters.PromptText,value:2}]});OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{DocumentSelectionChanged:"documentSelectionChanged"});OSF.DDA.DocumentSelectionChangedEventArgs=function(n){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.DocumentSelectionChanged},document:{value:n}})};OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidDocumentSelectionChangedEvent});OSF.OUtil.augmentList(Microsoft.Office.WebExtension.EventType,{BindingSelectionChanged:"bindingSelectionChanged",BindingDataChanged:"bindingDataChanged"});OSF.OUtil.augmentList(OSF.DDA.EventDescriptors,{BindingSelectionChangedEvent:"BindingSelectionChangedEvent"});OSF.DDA.BindingSelectionChangedEventArgs=function(n,t){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.BindingSelectionChanged},binding:{value:n}});for(var i in t)OSF.OUtil.defineEnumerableProperty(this,i,{value:t[i]})};OSF.DDA.BindingDataChangedEventArgs=function(n){OSF.OUtil.defineEnumerableProperties(this,{type:{value:Microsoft.Office.WebExtension.EventType.BindingDataChanged},binding:{value:n}})};OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDescriptors.BindingSelectionChangedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:0},{name:OSF.DDA.PropertyDescriptors.Subset,value:1}],isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidBindingSelectionChangedEvent,fromHost:[{name:OSF.DDA.EventDescriptors.BindingSelectionChangedEvent,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}],isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.EventDispId.dispidBindingDataChangedEvent,fromHost:[{name:OSF.DDA.PropertyDescriptors.BindingProperties,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]});OSF.OUtil.augmentList(Microsoft.Office.WebExtension.FilterType,{OnlyVisible:"onlyVisible"});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.FilterType,toHost:[{name:Microsoft.Office.WebExtension.FilterType.OnlyVisible,value:1}]});Microsoft.Office.WebExtension.GoToType={Binding:"binding",NamedItem:"namedItem",Slide:"slide",Index:"index"};Microsoft.Office.WebExtension.SelectionMode={Default:"default",Selected:"selected",None:"none"};Microsoft.Office.WebExtension.Index={First:"first",Last:"last",Next:"next",Previous:"previous"};OSF.DDA.AsyncMethodNames.addNames({GoToByIdAsync:"goToByIdAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GoToByIdAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Id,types:["string","number"]},{name:Microsoft.Office.WebExtension.Parameters.GoToType,"enum":Microsoft.Office.WebExtension.GoToType}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.SelectionMode,value:{"enum":Microsoft.Office.WebExtension.SelectionMode,defaultValue:Microsoft.Office.WebExtension.SelectionMode.Default}}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.GoToType,toHost:[{name:Microsoft.Office.WebExtension.GoToType.Binding,value:0},{name:Microsoft.Office.WebExtension.GoToType.NamedItem,value:1},{name:Microsoft.Office.WebExtension.GoToType.Slide,value:2},{name:Microsoft.Office.WebExtension.GoToType.Index,value:3}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:Microsoft.Office.WebExtension.Parameters.SelectionMode,toHost:[{name:Microsoft.Office.WebExtension.SelectionMode.Default,value:0},{name:Microsoft.Office.WebExtension.SelectionMode.Selected,value:1},{name:Microsoft.Office.WebExtension.SelectionMode.None,value:2}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidNavigateToMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.GoToType,value:1},{name:Microsoft.Office.WebExtension.Parameters.SelectionMode,value:2}]});OSF.DDA.AsyncMethodNames.addNames({ExecuteRichApiRequestAsync:"executeRichApiRequestAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:["object"]}],supportedOptions:[]});OSF.OUtil.setNamespace("RichApi",OSF.DDA);OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidExecuteRichApiRequestMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:0}],fromHost:[{name:Microsoft.Office.WebExtension.Parameters.Data,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]});OSF.DDA.FilePropertiesDescriptor={Url:"Url"};OSF.OUtil.augmentList(OSF.DDA.PropertyDescriptors,{FilePropertiesDescriptor:"FilePropertiesDescriptor"});Microsoft.Office.WebExtension.FileProperties=function(n){OSF.OUtil.defineEnumerableProperties(this,{url:{value:n[OSF.DDA.FilePropertiesDescriptor.Url]}})};OSF.DDA.AsyncMethodNames.addNames({GetFilePropertiesAsync:"getFilePropertiesAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.GetFilePropertiesAsync,fromHost:[{name:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,value:0}],requiredArguments:[],supportedOptions:[],onSucceeded:function(n){return new Microsoft.Office.WebExtension.FileProperties(n)}});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,fromHost:[{name:OSF.DDA.FilePropertiesDescriptor.Url,value:0}],isComplexType:!0});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidGetFilePropertiesMethod,fromHost:[{name:OSF.DDA.PropertyDescriptors.FilePropertiesDescriptor,value:OSF.DDA.SafeArray.Delegate.ParameterMap.self}]});OSF.DDA.ExcelTableBinding=function(n,t,i,r,u){var f=OSF.DDA.AsyncMethodNames;OSF.DDA.DispIdHost.addAsyncMethods(this,[f.ClearFormatsAsync,f.SetTableOptionsAsync,f.SetFormatsAsync]);OSF.DDA.ExcelTableBinding.uber.constructor.call(this,n,t,i,r,u);OSF.OUtil.finalizeProperties(this)};OSF.OUtil.extend(OSF.DDA.ExcelTableBinding,OSF.DDA.TableBinding),function(){var t="number",n="object",i="string";OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetSelectedDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:[i,n,t,"boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{"enum":Microsoft.Office.WebExtension.CoercionType,calculate:function(n){return OSF.DDA.DataCoercion.determineCoercionType(n[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:{types:[n],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:{types:[n],defaultValue:[]}}],privateStateCallbacks:[]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetDataAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.Data,types:[i,n,t,"boolean"]}],supportedOptions:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:{"enum":Microsoft.Office.WebExtension.CoercionType,calculate:function(n){return OSF.DDA.DataCoercion.determineCoercionType(n[Microsoft.Office.WebExtension.Parameters.Data])}}},{name:Microsoft.Office.WebExtension.Parameters.Rows,value:{types:[n,i],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.Columns,value:{types:[n],defaultValue:null}},{name:Microsoft.Office.WebExtension.Parameters.StartRow,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.StartColumn,value:{types:[t],defaultValue:0}},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:{types:[n],defaultValue:[]}},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:{types:[n],defaultValue:[]}}],checkCallArgs:function(n,t){var i=Microsoft.Office.WebExtension.Parameters;if(n[i.StartRow]==0&&n[i.StartColumn]==0&&OSF.OUtil.isArray(n[i.CellFormat])&&n[i.CellFormat].length===0&&OSF.OUtil.isArray(n[i.TableOptions])&&n[i.TableOptions].length===0&&(delete n[i.StartRow],delete n[i.StartColumn],delete n[i.CellFormat],delete n[i.TableOptions]),n[i.CoercionType]!=OSF.DDA.DataCoercion.getCoercionDefaultForBinding(t.type)&&(n[i.StartRow]&&n[i.StartRow]!=0||n[i.StartColumn]&&n[i.StartColumn]!=0||n[i.CellFormat]||n[i.TableOptions]))throw OSF.DDA.ErrorCodeManager.errorCodes.ooeCoercionTypeNotMatchBinding;return n},privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:function(n){return n.id}}]});OSF.DDA.BindingPromise.prototype.setTableOptionsAsync=function(){var n=arguments;return this._fetch(function(t){t.setTableOptionsAsync.apply(t,n)}),this};OSF.DDA.BindingPromise.prototype.setFormatsAsync=function(){var n=arguments;return this._fetch(function(t){t.setFormatsAsync.apply(t,n)}),this};OSF.DDA.BindingPromise.prototype.clearFormatsAsync=function(){var n=arguments;return this._fetch(function(t){t.clearFormatsAsync.apply(t,n)}),this}}(),function(){function n(n){return n.id}OSF.DDA.AsyncMethodNames.addNames({ClearFormatsAsync:"clearFormatsAsync",SetTableOptionsAsync:"setTableOptionsAsync",SetFormatsAsync:"setFormatsAsync"});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.ClearFormatsAsync,requiredArguments:[],supportedOptions:[],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetTableOptionsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.TableOptions,defaultValue:[]}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]});OSF.DDA.AsyncMethodCalls.define({method:OSF.DDA.AsyncMethodNames.SetFormatsAsync,requiredArguments:[{name:Microsoft.Office.WebExtension.Parameters.CellFormat,defaultValue:[]}],privateStateCallbacks:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:n}]})}();Microsoft.Office.WebExtension.Table={All:0,Data:1,Headers:2},function(){var u="undefined",e="auto fit",o="distributed",f="none";OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidClearFormatsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetTableOptionsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:1}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetFormatsMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:1}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetSelectedDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:0},{name:Microsoft.Office.WebExtension.Parameters.Data,value:1},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:2},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:3}]});OSF.DDA.SafeArray.Delegate.ParameterMap.define({type:OSF.DDA.MethodDispId.dispidSetBindingDataMethod,toHost:[{name:Microsoft.Office.WebExtension.Parameters.Id,value:0},{name:Microsoft.Office.WebExtension.Parameters.CoercionType,value:1},{name:Microsoft.Office.WebExtension.Parameters.Data,value:2},{name:OSF.DDA.SafeArray.UniqueArguments.Offset,value:3},{name:Microsoft.Office.WebExtension.Parameters.CellFormat,value:4},{name:Microsoft.Office.WebExtension.Parameters.TableOptions,value:5}]});var s={headerRow:0,bandedRows:1,firstColumn:2,lastColumn:3,bandedColumns:4,filterButton:5,style:6,totalRow:7},h={row:0,column:1},t={alignHorizontal:{text:"alignHorizontal",type:1},alignVertical:{text:"alignVertical",type:2},backgroundColor:{text:"backgroundColor",type:101},borderStyle:{text:"borderStyle",type:201},borderColor:{text:"borderColor",type:202},borderTopStyle:{text:"borderTopStyle",type:203},borderTopColor:{text:"borderTopColor",type:204},borderBottomStyle:{text:"borderBottomStyle",type:205},borderBottomColor:{text:"borderBottomColor",type:206},borderLeftStyle:{text:"borderLeftStyle",type:207},borderLeftColor:{text:"borderLeftColor",type:208},borderRightStyle:{text:"borderRightStyle",type:209},borderRightColor:{text:"borderRightColor",type:210},borderOutlineStyle:{text:"borderOutlineStyle",type:211},borderOutlineColor:{text:"borderOutlineColor",type:212},borderInlineStyle:{text:"borderInlineStyle",type:213},borderInlineColor:{text:"borderInlineColor",type:214},fontFamily:{text:"fontFamily",type:301},fontStyle:{text:"fontStyle",type:302},fontSize:{text:"fontSize",type:303},fontUnderlineStyle:{text:"fontUnderlineStyle",type:304},fontColor:{text:"fontColor",type:305},fontDirection:{text:"fontDirection",type:306},fontStrikethrough:{text:"fontStrikethrough",type:307},fontSuperscript:{text:"fontSuperscript",type:308},fontSubscript:{text:"fontSubscript",type:309},fontNormal:{text:"fontNormal",type:310},indentLeft:{text:"indentLeft",type:401},indentRight:{text:"indentRight",type:402},numberFormat:{text:"numberFormat",type:501},width:{text:"width",type:701},height:{text:"height",type:702},wrapping:{text:"wrapping",type:703}},r=[{name:f,value:0},{name:"thin",value:1},{name:"medium",value:2},{name:"dashed",value:3},{name:"dotted",value:4},{name:"thick",value:5},{name:"double",value:6},{name:"hair",value:7},{name:"medium dashed",value:8},{name:"dash dot",value:9},{name:"medium dash dot",value:10},{name:"dash dot dot",value:11},{name:"medium dash dot dot",value:12},{name:"slant dash dot",value:13}],i=[{name:f,value:0},{name:"black",value:1},{name:"blue",value:2},{name:"gray",value:3},{name:"green",value:4},{name:"orange",value:5},{name:"pink",value:6},{name:"purple",value:7},{name:"red",value:8},{name:"teal",value:9},{name:"turquoise",value:10},{name:"violet",value:11},{name:"white",value:12},{name:"yellow",value:13},{name:"automatic",value:14}],n=OSF.DDA.SafeArray.Delegate.ParameterMap;n.define({type:t.alignHorizontal.text,toHost:[{name:"general",value:0},{name:"left",value:1},{name:"center",value:2},{name:"right",value:3},{name:"fill",value:4},{name:"justify",value:5},{name:"center across selection",value:6},{name:o,value:7}]});n.define({type:t.alignVertical.text,toHost:[{name:"top",value:0},{name:"center",value:1},{name:"bottom",value:2},{name:"justify",value:3},{name:o,value:4}]});n.define({type:t.backgroundColor.text,toHost:i});n.define({type:t.borderStyle.text,toHost:r});n.define({type:t.borderColor.text,toHost:i});n.define({type:t.borderTopStyle.text,toHost:r});n.define({type:t.borderTopColor.text,toHost:i});n.define({type:t.borderBottomStyle.text,toHost:r});n.define({type:t.borderBottomColor.text,toHost:i});n.define({type:t.borderLeftStyle.text,toHost:r});n.define({type:t.borderLeftColor.text,toHost:i});n.define({type:t.borderRightStyle.text,toHost:r});n.define({type:t.borderRightColor.text,toHost:i});n.define({type:t.borderOutlineStyle.text,toHost:r});n.define({type:t.borderOutlineColor.text,toHost:i});n.define({type:t.borderInlineStyle.text,toHost:r});n.define({type:t.borderInlineColor.text,toHost:i});n.define({type:t.fontStyle.text,toHost:[{name:"regular",value:0},{name:"italic",value:1},{name:"bold",value:2},{name:"bold italic",value:3}]});n.define({type:t.fontUnderlineStyle.text,toHost:[{name:f,value:0},{name:"single",value:1},{name:"double",value:2},{name:"single accounting",value:3},{name:"double accounting",value:4}]});n.define({type:t.fontColor.text,toHost:i});n.define({type:t.fontDirection.text,toHost:[{name:"context",value:0},{name:"left-to-right",value:1},{name:"right-to-left",value:2}]});n.define({type:t.width.text,toHost:[{name:e,value:-1}]});n.define({type:t.height.text,toHost:[{name:e,value:-1}]});n.define({type:Microsoft.Office.WebExtension.Parameters.TableOptions,toHost:[{name:"headerRow",value:0},{name:"bandedRows",value:1},{name:"firstColumn",value:2},{name:"lastColumn",value:3},{name:"bandedColumns",value:4},{name:"filterButton",value:5},{name:"style",value:6},{name:"totalRow",value:7}]});n.dynamicTypes[Microsoft.Office.WebExtension.Parameters.CellFormat]={toHost:function(t){for(var i in t)t[i].format&&(t[i].format=n.doMapValues(t[i].format,"toHost"));return t},fromHost:function(n){return n}};n.setDynamicType(Microsoft.Office.WebExtension.Parameters.CellFormat,{toHost:function(n){var c="cells",y="format",w=0,b=1,p=[],l,i,r,f,e,o,a,v,s;for(l in n){if(i=n[l],r=[],typeof i[c]!==u){if(f=i[c],typeof i[c]=="object"){e=[];for(o in f)typeof h[o]!==u&&(e[h[o]]=f[o])}else e=f;r[w]=e}if(i[y]){a=i[y];v=[];for(s in a)typeof t[s]!==u&&v.push([t[s].type,a[s]]);r[b]=v}p[l]=r}return p},fromHost:function(n){return n}});n.setDynamicType(Microsoft.Office.WebExtension.Parameters.TableOptions,{toHost:function(n){var i=[];for(var t in n)typeof s[t]!==u&&(i[s[t]]=n[t]);return i},fromHost:function(n){return n}})}();OSF.DDA.ExcelDocument=function(n,t){var i=this,r=new OSF.DDA.BindingFacade(i);OSF.DDA.DispIdHost.addAsyncMethods(r,[OSF.DDA.AsyncMethodNames.AddFromPromptAsync]);OSF.DDA.DispIdHost.addAsyncMethods(i,[OSF.DDA.AsyncMethodNames.GoToByIdAsync]);OSF.DDA.DispIdHost.addAsyncMethods(i,[OSF.DDA.AsyncMethodNames.GetFilePropertiesAsync]);OSF.DDA.ExcelDocument.uber.constructor.call(i,n,r,t);OSF.OUtil.finalizeProperties(i)};OSF.OUtil.extend(OSF.DDA.ExcelDocument,OSF.DDA.JsomDocument);OSF.InitializationHelper.prototype.loadAppSpecificScriptAndCreateOM=function(n,t){OSF.DDA.ErrorCodeManager.initializeErrorMessages(Strings.OfficeOM);n.doc=new OSF.DDA.ExcelDocument(n,this._initializeSettings(!1));OSF.DDA.DispIdHost.addAsyncMethods(OSF.DDA.RichApi,[OSF.DDA.AsyncMethodNames.ExecuteRichApiRequestAsync]);t()},function(n){var t=function(){function n(n,t){this.m_actionInfo=n;this.m_isWriteOperation=t}return Object.defineProperty(n.prototype,"actionInfo",{get:function(){return this.m_actionInfo},enumerable:!0,configurable:!0}),Object.defineProperty(n.prototype,"isWriteOperation",{get:function(){return this.m_isWriteOperation},enumerable:!0,configurable:!0}),n}();n.Action=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(){}return t.createSetPropertyAction=function(t,i,r,u){var f;n.Utility.validateObjectPath(i);var e={Id:t._nextId(),ActionType:4,Name:r,ObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}},s=[u],o=n.Utility.setMethodArguments(t,e.ArgumentInfo,s);return n.Utility.validateReferencedObjectPaths(o),f=new n.Action(e,!0),t._pendingRequest.addAction(f),t._pendingRequest.addReferencedObjectPath(i._objectPath),t._pendingRequest.addReferencedObjectPaths(o),f},t.createMethodAction=function(t,i,r,u,f){var e,o,h,s;return n.Utility.validateObjectPath(i),e={Id:t._nextId(),ActionType:3,Name:r,ObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}},o=n.Utility.setMethodArguments(t,e.ArgumentInfo,f),n.Utility.validateReferencedObjectPaths(o),h=u!=1,s=new n.Action(e,h),t._pendingRequest.addAction(s),t._pendingRequest.addReferencedObjectPath(i._objectPath),t._pendingRequest.addReferencedObjectPaths(o),s},t.createQueryAction=function(t,i,r){var u,f;return n.Utility.validateObjectPath(i),u={Id:t._nextId(),ActionType:2,Name:"",ObjectPathId:i._objectPath.objectPathInfo.Id},u.QueryInfo=r,f=new n.Action(u,!1),t._pendingRequest.addAction(f),t._pendingRequest.addReferencedObjectPath(i._objectPath),f},t.createInstantiateAction=function(t,i){n.Utility.validateObjectPath(i);var u={Id:t._nextId(),ActionType:1,Name:"",ObjectPathId:i._objectPath.objectPathInfo.Id},r=new n.Action(u,!1);return t._pendingRequest.addAction(r),t._pendingRequest.addReferencedObjectPath(i._objectPath),t._pendingRequest.addActionResultHandler(r,new n.InstantiateActionResultHandler(i)),r},t.createTraceAction=function(t,i){var r={Id:t._nextId(),ActionType:5,Name:"Trace",ObjectPathId:0},u=new n.Action(r,!1);return t._pendingRequest.addAction(u),t._pendingRequest.addTrace(r.Id,i),u},t}();n.ActionFactory=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(t,i){n.Utility.checkArgumentNull(t,"context");this.m_context=t;this.m_objectPath=i;this.m_objectPath&&(t._processingResult||(n.ActionFactory.createInstantiateAction(t,this),t._autoCleanup&&this._KeepReference&&t.trackedObjects._autoAdd(this)))}return Object.defineProperty(t.prototype,"context",{get:function(){return this.m_context},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"_objectPath",{get:function(){return this.m_objectPath},set:function(n){this.m_objectPath=n},enumerable:!0,configurable:!0}),t.prototype._handleResult=function(n){},t}();n.ClientObject=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(n){this.m_context=n;this.m_actions=[];this.m_actionResultHandler={};this.m_referencedObjectPaths={};this.m_flags=0;this.m_traceInfos={}}return Object.defineProperty(t.prototype,"flags",{get:function(){return this.m_flags},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"traceInfos",{get:function(){return this.m_traceInfos},enumerable:!0,configurable:!0}),t.prototype.addAction=function(n){n.isWriteOperation&&(this.m_flags=this.m_flags|1);this.m_actions.push(n)},Object.defineProperty(t.prototype,"hasActions",{get:function(){return this.m_actions.length>0},enumerable:!0,configurable:!0}),t.prototype.addTrace=function(n,t){this.m_traceInfos[n]=t},t.prototype.addReferencedObjectPath=function(t){if(!this.m_referencedObjectPaths[t.objectPathInfo.Id])for(t.isValid||n.Utility.throwError(n.ResourceStrings.invalidObjectPath,n.Utility.getObjectPathExpression(t));t;)t.isWriteOperation&&(this.m_flags=this.m_flags|1),this.m_referencedObjectPaths[t.objectPathInfo.Id]=t,t.objectPathInfo.ObjectPathType==3&&this.addReferencedObjectPaths(t.argumentObjectPaths),t=t.parentObjectPath},t.prototype.addReferencedObjectPaths=function(n){if(n)for(var t=0;t<n.length;t++)this.addReferencedObjectPath(n[t])},t.prototype.addActionResultHandler=function(n,t){this.m_actionResultHandler[n.actionInfo.Id]=t},t.prototype.buildRequestMessageBody=function(){var r={},t,i,n;for(t in this.m_referencedObjectPaths)r[t]=this.m_referencedObjectPaths[t].objectPathInfo;for(i=[],n=0;n<this.m_actions.length;n++)i.push(this.m_actions[n].actionInfo);return{Actions:i,ObjectPaths:r}},t.prototype.processResponse=function(n){var t,i,r;if(n&&n.Results)for(t=0;t<n.Results.length;t++)i=n.Results[t],r=this.m_actionResultHandler[i.ActionId],r&&r._handleResult(i.Value)},t.prototype.invalidatePendingInvalidObjectPaths=function(){for(var n in this.m_referencedObjectPaths)this.m_referencedObjectPaths[n].isInvalidAfterRequest&&(this.m_referencedObjectPaths[n].isValid=!1)},t}();n.ClientRequest=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(t){this.m_nextId=0;this.m_url=t;n.Utility.isNullOrEmptyString(this.m_url)&&(this.m_url=n.Constants.localDocument);this._processingResult=!1;this._customData=n.Constants.iterativeExecutor;this._requestExecutor=new n.OfficeJsRequestExecutor;this.sync=this.sync.bind(this)}return Object.defineProperty(t.prototype,"_pendingRequest",{get:function(){return this.m_pendingRequest==null&&(this.m_pendingRequest=new n.ClientRequest(this)),this.m_pendingRequest},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"trackedObjects",{get:function(){return this.m_trackedObjects||(this.m_trackedObjects=new n.TrackedObjects(this)),this.m_trackedObjects},enumerable:!0,configurable:!0}),t.prototype.load=function(t,i){var u,f,r,e;n.Utility.validateContext(this,t);u={};typeof i=="string"?(f=i,u.Select=this.parseSelectExpand(f)):Array.isArray(i)?u.Select=i:typeof i=="object"?(r=i,typeof r.select=="string"?u.Select=this.parseSelectExpand(r.select):Array.isArray(r.select)?u.Select=r.select:n.Utility.isNullOrUndefined(r.select)||n.Utility.throwError(n.ResourceStrings.invalidArgument,"option.select"),typeof r.expand=="string"?u.Expand=this.parseSelectExpand(r.expand):Array.isArray(r.expand)?u.Expand=r.expand:n.Utility.isNullOrUndefined(r.expand)||n.Utility.throwError(n.ResourceStrings.invalidArgument,"option.expand"),typeof r.top=="number"?u.Top=r.top:n.Utility.isNullOrUndefined(r.top)||n.Utility.throwError(n.ResourceStrings.invalidArgument,"option.top"),typeof r.skip=="number"?u.Skip=r.skip:n.Utility.isNullOrUndefined(r.skip)||n.Utility.throwError(n.ResourceStrings.invalidArgument,"option.skip")):n.Utility.isNullOrUndefined(i)||n.Utility.throwError(n.ResourceStrings.invalidArgument,"option");e=n.ActionFactory.createQueryAction(this,t,u);this._pendingRequest.addActionResultHandler(e,t)},t.prototype.trace=function(t){n.ActionFactory.createTraceAction(this,t)},t.prototype.parseSelectExpand=function(t){var f=[],u,i,r;if(!n.Utility.isNullOrEmptyString(t))for(u=t.split(","),i=0;i<u.length;i++)r=u[i],r=r.trim(),f.push(r);return f},t.prototype.syncPrivate=function(t,i){var r=this._pendingRequest,e,f;if(!r.hasActions){t();return}this.m_pendingRequest=null;var o=r.buildRequestMessageBody(),s=r.flags,u=this._requestExecutor;u||(u=new n.OfficeJsRequestExecutor);e={Url:this.m_url,Headers:null,Body:o};r.invalidatePendingInvalidObjectPaths();f=this;u.executeAsync(this._customData,s,e,function(u){var e,s=[],h,o,c,l;if(n.Utility.isNullOrEmptyString(u.ErrorCode)?u.Body&&u.Body.Error&&(e=new n._Internal.RuntimeError(u.Body.Error.Code,u.Body.Error.Message,s,{errorLocation:u.Body.Error.Location})):e=new n._Internal.RuntimeError(u.ErrorCode,u.ErrorMessage,s,{}),u.Body&&u.Body.TraceIds)for(h=r.traceInfos,o=0;o<u.Body.TraceIds.length;o++)c=u.Body.TraceIds[o],l=h[c],s.push(l);if(e){i(e);return}else{f._processingResult=!0;try{r.processResponse(u.Body)}finally{f._processingResult=!1}t();return}})},t.prototype.sync=function(t){var i=this;return n._EnsurePromise(),new n.Promise(function(n,r){i.syncPrivate(function(){n(t)},function(n){r(n)})})},t._run=function(t,i,r,u,f,e){r===void 0&&(r=3);u===void 0&&(u=5e3);n._EnsurePromise();var c=new n.Promise(function(n,t){n()}),o,h=!1,s;return c.then(function(){o=t();o._autoCleanup=!0;var r=i(o);return(n.Utility.isNullOrUndefined(r)||typeof r.then!="function")&&n.Utility.throwError(n.ResourceStrings.runMustReturnPromise),r}).then(function(n){return o.sync(n)}).then(function(n){h=!0;s=n}).catch(function(n){s=n}).then(function(){function s(){n++;for(var i in t)o.trackedObjects.remove(t[i]);o.sync().then(function(){f&&f(n)}).catch(function(){e&&e(n);n<r&&setTimeout(function(){s()},u)})}var t=o.trackedObjects._retrieveAndClearAutoCleanupList(),i,n;o._autoCleanup=!1;for(i in t)t[i]._objectPath.isValid=!1;n=0;s()}).then(function(){if(h)return s;else throw s;})},t.prototype._nextId=function(){return++this.m_nextId},t}();n.ClientRequestContext=t}(OfficeExtension||(OfficeExtension={})),function(n){(function(n){n[n.None=0]="None";n[n.WriteOperation=1]="WriteOperation"})(n.ClientRequestFlags||(n.ClientRequestFlags={}));var t=n.ClientRequestFlags}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function n(){}return Object.defineProperty(n.prototype,"value",{get:function(){return this.m_value},enumerable:!0,configurable:!0}),n.prototype._handleResult=function(n){this.m_value=n},n}();n.ClientResult=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function n(){}return n.getItemAt="GetItemAt",n.id="Id",n.idPrivate="_Id",n.index="_Index",n.items="_Items",n.iterativeExecutor="IterativeExecutor",n.localDocument="http://document.localhost/",n.localDocumentApiPrefix="http://document.localhost/_api/",n.referenceId="_ReferenceId",n}();n.Constants=t}(OfficeExtension||(OfficeExtension={}));__extends=this.__extends||function(n,t){function r(){this.constructor=n}for(var i in t)t.hasOwnProperty(i)&&(n[i]=t[i]);r.prototype=t.prototype;n.prototype=new r},function(n){var t;(function(n){var t=function(n){function t(t,i,r,u){n.call(this,i);this.name="OfficeExtension.Error";this.code=t;this.message=i;this.traceMessages=r;this.debugInfo=u}return __extends(t,n),t.prototype.toString=function(){return this.code+": "+this.message},t}(Error);n.RuntimeError=t})(t=n._Internal||(n._Internal={}));n.Error=n._Internal.RuntimeError}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function n(){}return n.accessDenied="AccessDenied",n.generalException="GeneralException",n.activityLimitReached="ActivityLimitReached",n}();n.ErrorCodes=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(n){this.m_clientObject=n}return t.prototype._handleResult=function(t){n.Utility.fixObjectPathIfNecessary(this.m_clientObject,t);t&&!n.Utility.isNullOrUndefined(t[n.Constants.referenceId])&&this.m_clientObject._initReferenceId&&this.m_clientObject._initReferenceId(t[n.Constants.referenceId])},t}();n.InstantiateActionResultHandler=t}(OfficeExtension||(OfficeExtension={})),function(n){var t,i,r,u;(function(n){n[n.CustomData=0]="CustomData";n[n.Method=1]="Method";n[n.PathAndQuery=2]="PathAndQuery";n[n.Headers=3]="Headers";n[n.Body=4]="Body";n[n.AppPermission=5]="AppPermission";n[n.RequestFlags=6]="RequestFlags"})(n.RichApiRequestMessageIndex||(n.RichApiRequestMessageIndex={}));t=n.RichApiRequestMessageIndex,function(n){n[n.StatusCode=0]="StatusCode";n[n.Headers=1]="Headers";n[n.Body=2]="Body"}(n.RichApiResponseMessageIndex||(n.RichApiResponseMessageIndex={}));i=n.RichApiResponseMessageIndex,function(n){n[n.Instantiate=1]="Instantiate";n[n.Query=2]="Query";n[n.Method=3]="Method";n[n.SetProperty=4]="SetProperty";n[n.Trace=5]="Trace"}(n.ActionType||(n.ActionType={}));r=n.ActionType,function(n){n[n.GlobalObject=1]="GlobalObject";n[n.NewObject=2]="NewObject";n[n.Method=3]="Method";n[n.Property=4]="Property";n[n.Indexer=5]="Indexer";n[n.ReferenceId=6]="ReferenceId"}(n.ObjectPathType||(n.ObjectPathType={}));u=n.ObjectPathType}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(n,t,i,r){this.m_objectPathInfo=n;this.m_parentObjectPath=t;this.m_isWriteOperation=!1;this.m_isCollection=i;this.m_isInvalidAfterRequest=r;this.m_isValid=!0}return Object.defineProperty(t.prototype,"objectPathInfo",{get:function(){return this.m_objectPathInfo},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isWriteOperation",{get:function(){return this.m_isWriteOperation},set:function(n){this.m_isWriteOperation=n},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isCollection",{get:function(){return this.m_isCollection},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isInvalidAfterRequest",{get:function(){return this.m_isInvalidAfterRequest},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"parentObjectPath",{get:function(){return this.m_parentObjectPath},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"argumentObjectPaths",{get:function(){return this.m_argumentObjectPaths},set:function(n){this.m_argumentObjectPaths=n},enumerable:!0,configurable:!0}),Object.defineProperty(t.prototype,"isValid",{get:function(){return this.m_isValid},set:function(n){this.m_isValid=n},enumerable:!0,configurable:!0}),t.prototype.updateUsingObjectData=function(t){var r=t[n.Constants.referenceId],i;if(!n.Utility.isNullOrEmptyString(r)){this.m_isInvalidAfterRequest=!1;this.m_isValid=!0;this.m_objectPathInfo.ObjectPathType=6;this.m_objectPathInfo.Name=r;this.m_objectPathInfo.ArgumentInfo={};this.m_parentObjectPath=null;this.m_argumentObjectPaths=null;return}if(this.parentObjectPath&&this.parentObjectPath.isCollection&&(i=t[n.Constants.id],n.Utility.isNullOrUndefined(i)&&(i=t[n.Constants.idPrivate]),!n.Utility.isNullOrUndefined(i))){this.m_isInvalidAfterRequest=!1;this.m_isValid=!0;this.m_objectPathInfo.ObjectPathType=5;this.m_objectPathInfo.Name="";this.m_objectPathInfo.ArgumentInfo={};this.m_objectPathInfo.ArgumentInfo.Arguments=[i];this.m_argumentObjectPaths=null;return}},t}();n.ObjectPath=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(){}return t.createGlobalObjectObjectPath=function(t){var i={Id:t._nextId(),ObjectPathType:1,Name:""};return new n.ObjectPath(i,null,!1,!1)},t.createNewObjectObjectPath=function(t,i,r){var u={Id:t._nextId(),ObjectPathType:2,Name:i};return new n.ObjectPath(u,null,r,!1)},t.createPropertyObjectPath=function(t,i,r,u,f){var e={Id:t._nextId(),ObjectPathType:4,Name:r,ParentObjectPathId:i._objectPath.objectPathInfo.Id};return new n.ObjectPath(e,i._objectPath,u,f)},t.createIndexerObjectPath=function(t,i,r){var u={Id:t._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}};return u.ArgumentInfo.Arguments=r,new n.ObjectPath(u,i._objectPath,!1,!1)},t.createIndexerObjectPathUsingParentPath=function(t,i,r){var u={Id:t._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:i.objectPathInfo.Id,ArgumentInfo:{}};return u.ArgumentInfo.Arguments=r,new n.ObjectPath(u,i,!1,!1)},t.createMethodObjectPath=function(t,i,r,u,f,e,o){var h={Id:t._nextId(),ObjectPathType:3,Name:r,ParentObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}},c=n.Utility.setMethodArguments(t,h.ArgumentInfo,f),s=new n.ObjectPath(h,i._objectPath,e,o);return s.argumentObjectPaths=c,s.isWriteOperation=u!=1,s},t.createChildItemObjectPathUsingIndexer=function(t,i,r){var f=r[n.Constants.id],u;return n.Utility.isNullOrUndefined(f)&&(f=r[n.Constants.idPrivate]),u=u={Id:t._nextId(),ObjectPathType:5,Name:"",ParentObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}},u.ArgumentInfo.Arguments=[f],new n.ObjectPath(u,i._objectPath,!1,!1)},t.createChildItemObjectPathUsingGetItemAt=function(t,i,r,u){var e=r[n.Constants.index],f;return e&&(u=e),f={Id:t._nextId(),ObjectPathType:3,Name:n.Constants.getItemAt,ParentObjectPathId:i._objectPath.objectPathInfo.Id,ArgumentInfo:{}},f.ArgumentInfo.Arguments=[u],new n.ObjectPath(f,i._objectPath,!1,!1)},t.createReferenceIdObjectPath=function(t,i){var r={Id:t._nextId(),ObjectPathType:6,Name:i};return new n.ObjectPath(r,null,!1,!1)},t}();n.ObjectPathFactory=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(){}return t.prototype.executeAsync=function(i,r,u,f){var e=JSON.stringify(u.Body),o;n.Utility.log("Request:");n.Utility.log(e);o=n.RichApiMessageUtility.buildRequestMessageSafeArray(i,r,"POST","ProcessQuery",null,e);OSF.DDA.RichApi.executeRichApiRequestAsync(o,function(i){var r,u;n.Utility.log("Response:");n.Utility.log(JSON.stringify(i));r={ErrorCode:"",ErrorMessage:"",Headers:null,Body:null};i.status=="succeeded"?(u=n.RichApiMessageUtility.getResponseBody(i),r.Body=JSON.parse(u),r.Headers=n.RichApiMessageUtility.getResponseHeaders(i),f(r)):(r.ErrorCode=n.ErrorCodes.generalException,i.error.code==t.OfficeJsErrorCode_ooeNoCapability?r.ErrorCode=n.ErrorCodes.accessDenied:i.error.code==t.OfficeJsErrorCode_ooeActivityLimitReached&&(r.ErrorCode=n.ErrorCodes.activityLimitReached),r.ErrorMessage=i.error.message,f(r))})},t.OfficeJsErrorCode_ooeNoCapability=7e3,t.OfficeJsErrorCode_ooeActivityLimitReached=5102,t}();n.OfficeJsRequestExecutor=t}(OfficeExtension||(OfficeExtension={})),function(n){function u(n){return t.settings.oldxhr=n,r}function r(){return new t}var i=function(){function n(){}return n}(),t;n.OfficeXHRSettings=i;n.resetXHRFactory=u;n.officeXHRFactory=r;t=function(){function t(){}return t.prototype.open=function(i,r){if(this.m_method=i,this.m_url=r,this.m_url.toLowerCase().indexOf(n.Constants.localDocumentApiPrefix)==0)this.m_url=this.m_url.substr(n.Constants.localDocumentApiPrefix.length);else{this.m_innerXhr=t.settings.oldxhr();var u=this;this.m_innerXhr.onreadystatechange=function(){u.innerXhrOnreadystatechage()};this.m_innerXhr.open(i,this.m_url)}},t.prototype.abort=function(){this.m_innerXhr&&this.m_innerXhr.abort()},t.prototype.send=function(i){var f,u,r;this.m_innerXhr?this.m_innerXhr.send(i):(f=this,u=0,n.Utility.isReadonlyRestRequest(this.m_method)||(u=1),r=t.settings.executeRichApiRequestAsync,r||(r=OSF.DDA.RichApi.executeRichApiRequestAsync),r(n.RichApiMessageUtility.buildRequestMessageSafeArray("",u,this.m_method,this.m_url,this.m_requestHeaders,i),function(n){f.officeContextRequestCallback(n)}))},t.prototype.setRequestHeader=function(n,t){this.m_innerXhr?this.m_innerXhr.setRequestHeader(n,t):(this.m_requestHeaders||(this.m_requestHeaders={}),this.m_requestHeaders[n]=t)},t.prototype.getResponseHeader=function(n){return this.m_responseHeaders?this.m_responseHeaders[n.toUpperCase()]:null},t.prototype.getAllResponseHeaders=function(){return this.m_allResponseHeaders},t.prototype.overrideMimeType=function(n){this.m_innerXhr&&this.m_innerXhr.overrideMimeType(n)},t.prototype.innerXhrOnreadystatechage=function(){this.readyState=this.m_innerXhr.readyState;this.readyState==t.DONE&&(this.status=this.m_innerXhr.status,this.statusText=this.m_innerXhr.statusText,this.responseText=this.m_innerXhr.responseText,this.response=this.m_innerXhr.response,this.responseType=this.m_innerXhr.responseType,this.setAllResponseHeaders(this.m_innerXhr.getAllResponseHeaders()));this.onreadystatechange&&this.onreadystatechange()},t.prototype.officeContextRequestCallback=function(i){this.readyState=t.DONE;i.status=="succeeded"?(this.status=n.RichApiMessageUtility.getResponseStatusCode(i),this.m_responseHeaders=n.RichApiMessageUtility.getResponseHeaders(i),console.debug("ResponseHeaders="+JSON.stringify(this.m_responseHeaders)),this.responseText=n.RichApiMessageUtility.getResponseBody(i),console.debug("ResponseText="+this.responseText),this.response=this.responseText):(this.status=500,this.statusText="Internal Error");this.onreadystatechange&&this.onreadystatechange()},t.prototype.setAllResponseHeaders=function(t){var s,o,r,i,u,f,e;if(this.m_allResponseHeaders=t,this.m_responseHeaders={},this.m_allResponseHeaders!=null)for(s=new RegExp("\r?\n"),o=this.m_allResponseHeaders.split(s),r=0;r<o.length;r++)i=o[r],i!=null&&(u=i.indexOf(":"),u>0&&(f=i.substr(0,u),e=i.substr(u+1),f=n.Utility.trim(f),e=n.Utility.trim(e),this.m_responseHeaders[f.toUpperCase()]=e))},t.UNSENT=0,t.OPENED=1,t.DONE=4,t.settings=new i,t}();n.OfficeXHR=t}(OfficeExtension||(OfficeExtension={})),function(n){function i(){n.Promise||t.Init()}n._EnsurePromise=i;var t;(function(t){function i(){(function(){"use strict";function wt(n){return typeof n=="function"||typeof n=="object"&&n!==null}function b(n){return typeof n=="function"}function bt(n){return typeof n=="object"&&n!==null}function kt(n){k=n}function dt(n){f=n}function ti(){var t=process.nextTick,n=process.versions.node.match(/^(?:(\d+)\.)?(?:(\d+)\.)?(\*|\d+)$/);return Array.isArray(n)&&n[1]==="0"&&n[2]==="10"&&(t=setImmediate),function(){t(c)}}function ii(){return function(){tt(c)}}function ri(){var n=0,i=new ut(c),t=document.createTextNode("");return i.observe(t,{characterData:!0}),function(){t.data=n=++n%2}}function ui(){var n=new MessageChannel;return n.port1.onmessage=c,function(){n.port2.postMessage(0)}}function ft(){return function(){setTimeout(c,1)}}function c(){for(var t,i,n=0;n<h;n+=2)t=e[n],i=e[n+1],t(i),e[n]=undefined,e[n+1]=undefined;h=0}function fi(){try{var t=require,n=t("vertx");return tt=n.runOnLoop||n.runOnContext,ii()}catch(i){return ft()}}function l(){}function ei(){return new TypeError("You cannot resolve a promise with itself")}function oi(){return new TypeError("A promises callback cannot return that same promise.")}function si(n){try{return n.then}catch(t){return y.error=t,y}}function hi(n,t,i,r){try{n.call(t,i,r)}catch(u){return u}}function ci(n,i,u){f(function(n){var f=!1,e=hi(u,i,function(t){f||(f=!0,i!==t?v(n,t):r(n,t))},function(i){f||(f=!0,t(n,i))},"Settle: "+(n._label||" unknown promise"));!f&&e&&(f=!0,t(n,e))},n)}function li(n,i){i._state===a?r(n,i._result):i._state===s?t(n,i._result):p(i,undefined,function(t){v(n,t)},function(i){t(n,i)})}function ai(n,i){if(i.constructor===n.constructor)li(n,i);else{var u=si(i);u===y?t(n,y.error):u===undefined?r(n,i):b(u)?ci(n,i,u):r(n,i)}}function v(n,i){n===i?t(n,ei()):wt(i)?ai(n,i):r(n,i)}function vi(n){n._onerror&&n._onerror(n._result);d(n)}function r(n,t){n._state===o&&(n._result=t,n._state=a,n._subscribers.length!==0&&f(d,n))}function t(n,t){n._state===o&&(n._state=s,n._result=t,f(vi,n))}function p(n,t,i,r){var u=n._subscribers,e=u.length;n._onerror=null;u[e]=t;u[e+a]=i;u[e+s]=r;e===0&&n._state&&f(d,n)}function d(n){var i=n._subscribers,e=n._state,r,u,f,t;if(i.length!==0){for(f=n._result,t=0;t<i.length;t+=3)r=i[t],u=i[t+e],r?st(e,r,u,f):u(f);n._subscribers.length=0}}function ot(){this.error=null}function yi(n,t){try{return n(t)}catch(i){return w.error=i,w}}function st(n,i,u,f){var c=b(u),e,l,h,y;if(c){if(e=yi(u,f),e===w?(y=!0,l=e.error,e=null):h=!0,i===e){t(i,oi());return}}else e=f,h=!0;i._state!==o||(c&&h?v(i,e):y?t(i,l):n===a?r(i,e):n===s&&t(i,e))}function pi(n,i){try{i(function(t){v(n,t)},function(i){t(n,i)})}catch(r){t(n,r)}}function u(n,i){var u=this;u._instanceConstructor=n;u.promise=new n(l);u._validateInput(i)?(u._input=i,u.length=i.length,u._remaining=i.length,u._init(),u.length===0?r(u.promise,u._result):(u.length=u.length||0,u._enumerate(),u._remaining===0&&r(u.promise,u._result))):t(u.promise,u._validationError())}function wi(n){return new ht(this,n).promise}function bi(n){function e(n){v(i,n)}function s(n){t(i,n)}var u=this,i=new u(l),f,r;if(!nt(n))return t(i,new TypeError("You must pass an array to race.")),i;for(f=n.length,r=0;i._state===o&&r<f;r++)p(u.resolve(n[r]),undefined,e,s);return i}function ki(n){var i=this,t;return n&&typeof n=="object"&&n.constructor===i?n:(t=new i(l),v(t,n),t)}function di(n){var r=this,i=new r(l);return t(i,n),i}function gi(){throw new TypeError("You must pass a resolver function as the first argument to the promise constructor");}function nr(){throw new TypeError("Failed to construct 'Promise': Please use the 'new' operator, this object constructor cannot be called as a function.");}function i(n){this._id=yt++;this._state=undefined;this._result=undefined;this._subscribers=[];l!==n&&(b(n)||gi(),this instanceof i||nr(),pi(this,n))}var g,e,et,w,ht,ct,lt,at,vt,yt,pt;g=Array.isArray?Array.isArray:function(n){return Object.prototype.toString.call(n)==="[object Array]"};var nt=g,h=0,tr={}.toString,tt,k,f=function(n,t){e[h]=n;e[h+1]=t;h+=2;h===2&&(k?k(c):et())};var it=typeof window!="undefined"?window:undefined,rt=it||{},ut=rt.MutationObserver||rt.WebKitMutationObserver,gt=typeof process!="undefined"&&{}.toString.call(process)==="[object process]",ni=typeof Uint8ClampedArray!="undefined"&&typeof importScripts!="undefined"&&typeof MessageChannel!="undefined";e=new Array(1e3);et=gt?ti():ut?ri():ni?ui():it===undefined&&typeof require=="function"?fi():ft();var o=void 0,a=1,s=2,y=new ot;w=new ot;u.prototype._validateInput=function(n){return nt(n)};u.prototype._validationError=function(){return new n.Error("Array Methods must be provided an Array")};u.prototype._init=function(){this._result=new Array(this.length)};ht=u;u.prototype._enumerate=function(){for(var n=this,i=n.length,r=n.promise,u=n._input,t=0;r._state===o&&t<i;t++)n._eachEntry(u[t],t)};u.prototype._eachEntry=function(n,t){var i=this,r=i._instanceConstructor;bt(n)?n.constructor===r&&n._state!==o?(n._onerror=null,i._settledAt(n._state,t,n._result)):i._willSettleAt(r.resolve(n),t):(i._remaining--,i._result[t]=n)};u.prototype._settledAt=function(n,i,u){var f=this,e=f.promise;e._state===o&&(f._remaining--,n===s?t(e,u):f._result[i]=u);f._remaining===0&&r(e,f._result)};u.prototype._willSettleAt=function(n,t){var i=this;p(n,undefined,function(n){i._settledAt(a,t,n)},function(n){i._settledAt(s,t,n)})};ct=wi;lt=bi;at=ki;vt=di;yt=0;pt=i;i.all=ct;i.race=lt;i.resolve=at;i.reject=vt;i._setScheduler=kt;i._setAsap=dt;i._asap=f;i.prototype={constructor:i,then:function(n,t){var u=this,i=u._state,r,e,o;return i===a&&!n||i===s&&!t?this:(r=new this.constructor(l),e=u._result,i?(o=arguments[i-1],f(function(){st(i,r,o,e)})):p(u,r,n,t),r)},"catch":function(n){return this.then(null,n)}};n.Promise=pt}).call(this)}t.Init=i})(t||(t={}))}(OfficeExtension||(OfficeExtension={})),function(n){(function(n){n[n.Default=0]="Default";n[n.Read=1]="Read"})(n.OperationType||(n.OperationType={}));var t=n.OperationType}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(n){this._autoCleanupList={};this.m_context=n}return t.prototype.add=function(n){var t=this;Array.isArray(n)?n.forEach(function(n){return t._addCommon(n,!0)}):this._addCommon(n,!0)},t.prototype._autoAdd=function(n){this._addCommon(n,!1);this._autoCleanupList[n._objectPath.objectPathInfo.Id]=n},t.prototype._addCommon=function(t,i){var r=t[n.Constants.referenceId];n.Utility.isNullOrEmptyString(r)&&t._KeepReference&&(t._KeepReference(),n.ActionFactory.createInstantiateAction(this.m_context,t),i&&this.m_context._autoCleanup&&delete this._autoCleanupList[t._objectPath.objectPathInfo.Id])},t.prototype.remove=function(n){var t=this;Array.isArray(n)?n.forEach(function(n){return t._removeCommon(n)}):this._removeCommon(n)},t.prototype._removeCommon=function(t){var r=t[n.Constants.referenceId],i;n.Utility.isNullOrEmptyString(r)||(i=this.m_context._rootObject,i._RemoveReference&&i._RemoveReference(r))},t.prototype._retrieveAndClearAutoCleanupList=function(){var n=this._autoCleanupList;return this._autoCleanupList={},n},t}();n.TrackedObjects=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function n(){}return n.invalidObjectPath="InvalidObjectPath",n.propertyNotLoaded="PropertyNotLoaded",n.invalidRequestContext="InvalidRequestContext",n.invalidArgument="InvalidArgument",n.runMustReturnPromise="RunMustReturnPromise",n}();n.ResourceStrings=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function n(){}return n.buildRequestMessageSafeArray=function(n,t,i,r,u,f){var e=[],o;if(u)for(o in u)e.push(o),e.push(u[o]);var s=0,h="",c="",l="";return[n,i,r,e,f,s,t,h,c,l]},n.getResponseBody=function(t){return n.getResponseBodyFromSafeArray(t.value.data)},n.getResponseHeaders=function(t){return n.getResponseHeadersFromSafeArray(t.value.data)},n.getResponseBodyFromSafeArray=function(n){var t=n[2],i;return typeof t=="string"?t:(i=t,i.join(""))},n.getResponseHeadersFromSafeArray=function(n){var i=n[1],r,t;if(!i)return null;for(r={},t=0;t<i.length-1;t+=2)r[i[t]]=i[t+1];return r},n.getResponseStatusCode=function(t){return n.getResponseStatusCodeFromSafeArray(t.value.data)},n.getResponseStatusCodeFromSafeArray=function(n){return n[0]},n}();n.RichApiMessageUtility=t}(OfficeExtension||(OfficeExtension={})),function(n){var t=function(){function t(){}return t.checkArgumentNull=function(n,t){},t.isNullOrUndefined=function(n){return n===null?!0:typeof n=="undefined"?!0:!1},t.isUndefined=function(n){return typeof n=="undefined"?!0:!1},t.isNullOrEmptyString=function(n){return n===null?!0:typeof n=="undefined"?!0:n.length==0?!0:!1},t.trim=function(n){return n.replace(new RegExp("^\\s+|\\s+$","g"),"")},t.caseInsensitiveCompareString=function(n,i){return t.isNullOrUndefined(n)?t.isNullOrUndefined(i):t.isNullOrUndefined(i)?!1:n.toUpperCase()==i.toUpperCase()},t.isReadonlyRestRequest=function(n){return t.caseInsensitiveCompareString(n,"GET")},t.setMethodArguments=function(i,r,u){var f,e;if(t.isNullOrUndefined(u))return null;var s=[],o=[],h=!1;for(f=0;f<u.length;f++)u[f]instanceof n.ClientObject?(e=u[f],t.validateContext(i,e),u[f]=e._objectPath.objectPathInfo.Id,o.push(e._objectPath.objectPathInfo.Id),s.push(e._objectPath),h=!0):o.push(0);return(r.Arguments=u,h)?(r.ReferencedObjectPathIds=o,s):null},t.fixObjectPathIfNecessary=function(n,t){n&&n._objectPath&&t&&n._objectPath.updateUsingObjectData(t)},t.validateObjectPath=function(i){for(var r=i._objectPath,u;r;)r.isValid||(u=t.getObjectPathExpression(r),t.throwError(n.ResourceStrings.invalidObjectPath,u)),r=r.parentObjectPath},t.validateReferencedObjectPaths=function(i){var u,r,f;if(i)for(u=0;u<i.length;u++)for(r=i[u];r;)r.isValid||(f=t.getObjectPathExpression(r),t.throwError(n.ResourceStrings.invalidObjectPath,f)),r=r.parentObjectPath},t.validateContext=function(i,r){r&&r.context!==i&&t.throwError(n.ResourceStrings.invalidRequestContext)},t.log=function(n){t._logEnabled&&window.console&&window.console.log&&window.console.log(n)},t.load=function(n,t){n.context.load(n,t)},t.throwError=function(i,r){throw new n._Internal.RuntimeError(i,t._getResourceString(i,r),[],{});},t.createRuntimeError=function(t,i,r){return new n._Internal.RuntimeError(t,i,[],{errorLocation:r})},t._getResourceString=function(n,i){var r=n,f,u;return window.Strings&&window.Strings.OfficeOM&&(f="L_"+n,u=window.Strings.OfficeOM[f],u&&(r=u)),t.isNullOrUndefined(i)||(r=r.replace("{0}",i)),r},t.throwIfNotLoaded=function(i,r){t.isUndefined(r)&&i.charCodeAt(0)!=t.s_underscoreCharCode&&t.throwError(n.ResourceStrings.propertyNotLoaded,i)},t.getObjectPathExpression=function(n){for(var i="";n;){switch(n.objectPathInfo.ObjectPathType){case 1:i=i;break;case 2:i="new()"+(i.length>0?".":"")+i;break;case 3:i=t.normalizeName(n.objectPathInfo.Name)+"()"+(i.length>0?".":"")+i;break;case 4:i=t.normalizeName(n.objectPathInfo.Name)+(i.length>0?".":"")+i;break;case 5:i="getItem()"+(i.length>0?".":"")+i;break;case 6:i="_reference()"+(i.length>0?".":"")+i;break}n=n.parentObjectPath}return i},t._createPromiseFromResult=function(t){return n._EnsurePromise(),new n.Promise(function(n,i){n(t)})},t._addActionResultHandler=function(n,t,i){n.context._pendingRequest.addActionResultHandler(t,i)},t._handleNavigationPropertyResults=function(n,i,r){for(var u=0;u<r.length-1;u+=2)t.isUndefined(i[r[u+1]])||n[r[u]]._handleResult(i[r[u+1]])},t.normalizeName=function(n){return n.substr(0,1).toLowerCase()+n.substr(1)},t._logEnabled=!1,t.s_underscoreCharCode="_".charCodeAt(0),t}();n.Utility=t}(OfficeExtension||(OfficeExtension={}));__extends=this.__extends||function(n,t){function r(){this.constructor=n}for(var i in t)t.hasOwnProperty(i)&&(n[i]=t[i]);r.prototype=t.prototype;n.prototype=new r},function(n){var f=OfficeExtension.ObjectPathFactory.createPropertyObjectPath,s=OfficeExtension.ObjectPathFactory.createMethodObjectPath,l=OfficeExtension.ObjectPathFactory.createIndexerObjectPath,kr=OfficeExtension.ObjectPathFactory.createNewObjectObjectPath,a=OfficeExtension.ObjectPathFactory.createChildItemObjectPathUsingIndexer,v=OfficeExtension.ObjectPathFactory.createChildItemObjectPathUsingGetItemAt,c=OfficeExtension.ActionFactory.createMethodAction,r=OfficeExtension.ActionFactory.createSetPropertyAction,u=OfficeExtension.Utility.isNullOrUndefined,i=OfficeExtension.Utility.isUndefined,t=OfficeExtension.Utility.throwIfNotLoaded,e=OfficeExtension.Utility.load,o=OfficeExtension.Utility.fixObjectPathIfNecessary,y=OfficeExtension.Utility._addActionResultHandler,h=OfficeExtension.Utility._handleNavigationPropertyResults,bi=function(n){function r(){n.apply(this,arguments)}return __extends(r,n),Object.defineProperty(r.prototype,"calculationMode",{get:function(){return t("calculationMode",this.m_calculationMode),this.m_calculationMode},enumerable:!0,configurable:!0}),r.prototype.calculate=function(n){c(this.context,this,"Calculate",0,[n])},r.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.CalculationMode)||(this.m_calculationMode=t.CalculationMode)}},r.prototype.load=function(n){return e(this,n),this},r}(OfficeExtension.ClientObject),w,b,k,p,d,g,nt,tt,it,rt,ut,ft,et,ot,st,ht,ct,lt,at,vt,yt,pt,wt,bt,kt,dt,gt,ni,ti,ii,ri,ui,fi,ei,oi,si,hi,ci,li,ai,vi,yi,pi,wi,ki,di,gi,nr,tr,ir,rr,ur,fr,er,or,sr,hr,cr,lr,ar,vr,yr,pr,wr,br;n.Application=bi;w=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"application",{get:function(){return this.m_application||(this.m_application=new n.Application(this.context,f(this.context,this,"Application",!1,!1))),this.m_application},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"bindings",{get:function(){return this.m_bindings||(this.m_bindings=new n.BindingCollection(this.context,f(this.context,this,"Bindings",!0,!1))),this.m_bindings},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"names",{get:function(){return this.m_names||(this.m_names=new n.NamedItemCollection(this.context,f(this.context,this,"Names",!0,!1))),this.m_names},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"tables",{get:function(){return this.m_tables||(this.m_tables=new n.TableCollection(this.context,f(this.context,this,"Tables",!0,!1))),this.m_tables},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"worksheets",{get:function(){return this.m_worksheets||(this.m_worksheets=new n.WorksheetCollection(this.context,f(this.context,this,"Worksheets",!0,!1))),this.m_worksheets},enumerable:!0,configurable:!0}),i.prototype.getSelectedRange=function(){return new n.Range(this.context,s(this.context,this,"GetSelectedRange",1,[],!1,!0))},i.prototype._GetObjectByReferenceId=function(n){var i=c(this.context,this,"_GetObjectByReferenceId",1,[n]),t=new OfficeExtension.ClientResult;return y(this,i,t),t},i.prototype._GetObjectTypeNameByReferenceId=function(n){var i=c(this.context,this,"_GetObjectTypeNameByReferenceId",1,[n]),t=new OfficeExtension.ClientResult;return y(this,i,t),t},i.prototype._GetReferenceCount=function(){var t=c(this.context,this,"_GetReferenceCount",1,[]),n=new OfficeExtension.ClientResult;return y(this,t,n),n},i.prototype._RemoveAllReferences=function(){c(this.context,this,"_RemoveAllReferences",1,[])},i.prototype._RemoveReference=function(n){c(this.context,this,"_RemoveReference",1,[n])},i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["application","Application","bindings","Bindings","names","Names","tables","Tables","worksheets","Worksheets"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.Workbook=w;b=function(l){function a(){l.apply(this,arguments)}return __extends(a,l),Object.defineProperty(a.prototype,"charts",{get:function(){return this.m_charts||(this.m_charts=new n.ChartCollection(this.context,f(this.context,this,"Charts",!0,!1))),this.m_charts},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"tables",{get:function(){return this.m_tables||(this.m_tables=new n.TableCollection(this.context,f(this.context,this,"Tables",!0,!1))),this.m_tables},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"id",{get:function(){return t("id",this.m_id),this.m_id},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"position",{get:function(){return t("position",this.m_position),this.m_position},set:function(n){this.m_position=n;r(this.context,this,"Position",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"visibility",{get:function(){return t("visibility",this.m_visibility),this.m_visibility},enumerable:!0,configurable:!0}),a.prototype.activate=function(){c(this.context,this,"Activate",1,[])},a.prototype.delete=function(){c(this.context,this,"Delete",0,[])},a.prototype.getCell=function(t,i){return new n.Range(this.context,s(this.context,this,"GetCell",1,[t,i],!1,!0))},a.prototype.getRange=function(t){return new n.Range(this.context,s(this.context,this,"GetRange",1,[t],!1,!0))},a.prototype.getUsedRange=function(){return new n.Range(this.context,s(this.context,this,"GetUsedRange",1,[],!1,!0))},a.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Id)||(this.m_id=t.Id);i(t.Name)||(this.m_name=t.Name);i(t.Position)||(this.m_position=t.Position);i(t.Visibility)||(this.m_visibility=t.Visibility);h(this,t,["charts","Charts","tables","Tables"])}},a.prototype.load=function(n){return e(this,n),this},a}(OfficeExtension.ClientObject);n.Worksheet=b;k=function(i){function r(){i.apply(this,arguments)}return __extends(r,i),Object.defineProperty(r.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),r.prototype.add=function(t){return new n.Worksheet(this.context,s(this.context,this,"Add",0,[t],!1,!0))},r.prototype.getActiveWorksheet=function(){return new n.Worksheet(this.context,s(this.context,this,"GetActiveWorksheet",1,[],!1,!1))},r.prototype.getItem=function(t){return new n.Worksheet(this.context,l(this.context,this,[t]))},r.prototype._handleResult=function(t){var r,f,i,e;if(!u(t)&&(r=t,o(this,r),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],f=r[OfficeExtension.Constants.items],i=0;i<f.length;i++)e=new n.Worksheet(this.context,a(this.context,this,f[i])),e._handleResult(f[i]),this.m__items.push(e)},r.prototype.load=function(n){return e(this,n),this},r}(OfficeExtension.ClientObject);n.WorksheetCollection=k;p=function(l){function a(){l.apply(this,arguments)}return __extends(a,l),Object.defineProperty(a.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.RangeFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"worksheet",{get:function(){return this.m_worksheet||(this.m_worksheet=new n.Worksheet(this.context,f(this.context,this,"Worksheet",!1,!1))),this.m_worksheet},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"address",{get:function(){return t("address",this.m_address),this.m_address},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"addressLocal",{get:function(){return t("addressLocal",this.m_addressLocal),this.m_addressLocal},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"cellCount",{get:function(){return t("cellCount",this.m_cellCount),this.m_cellCount},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"columnCount",{get:function(){return t("columnCount",this.m_columnCount),this.m_columnCount},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"columnIndex",{get:function(){return t("columnIndex",this.m_columnIndex),this.m_columnIndex},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"formulas",{get:function(){return t("formulas",this.m_formulas),this.m_formulas},set:function(n){this.m_formulas=n;r(this.context,this,"Formulas",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"formulasLocal",{get:function(){return t("formulasLocal",this.m_formulasLocal),this.m_formulasLocal},set:function(n){this.m_formulasLocal=n;r(this.context,this,"FormulasLocal",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"numberFormat",{get:function(){return t("numberFormat",this.m_numberFormat),this.m_numberFormat},set:function(n){this.m_numberFormat=n;r(this.context,this,"NumberFormat",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"rowCount",{get:function(){return t("rowCount",this.m_rowCount),this.m_rowCount},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"rowIndex",{get:function(){return t("rowIndex",this.m_rowIndex),this.m_rowIndex},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"text",{get:function(){return t("text",this.m_text),this.m_text},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"valueTypes",{get:function(){return t("valueTypes",this.m_valueTypes),this.m_valueTypes},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"values",{get:function(){return t("values",this.m_values),this.m_values},set:function(n){this.m_values=n;r(this.context,this,"Values",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"_ReferenceId",{get:function(){return t("_ReferenceId",this.m__ReferenceId),this.m__ReferenceId},enumerable:!0,configurable:!0}),a.prototype.clear=function(n){c(this.context,this,"Clear",0,[n])},a.prototype.delete=function(n){c(this.context,this,"Delete",0,[n])},a.prototype.getBoundingRect=function(t){return new n.Range(this.context,s(this.context,this,"GetBoundingRect",1,[t],!1,!0))},a.prototype.getCell=function(t,i){return new n.Range(this.context,s(this.context,this,"GetCell",1,[t,i],!1,!0))},a.prototype.getColumn=function(t){return new n.Range(this.context,s(this.context,this,"GetColumn",1,[t],!1,!0))},a.prototype.getEntireColumn=function(){return new n.Range(this.context,s(this.context,this,"GetEntireColumn",1,[],!1,!0))},a.prototype.getEntireRow=function(){return new n.Range(this.context,s(this.context,this,"GetEntireRow",1,[],!1,!0))},a.prototype.getIntersection=function(t){return new n.Range(this.context,s(this.context,this,"GetIntersection",1,[t],!1,!0))},a.prototype.getLastCell=function(){return new n.Range(this.context,s(this.context,this,"GetLastCell",1,[],!1,!0))},a.prototype.getLastColumn=function(){return new n.Range(this.context,s(this.context,this,"GetLastColumn",1,[],!1,!0))},a.prototype.getLastRow=function(){return new n.Range(this.context,s(this.context,this,"GetLastRow",1,[],!1,!0))},a.prototype.getOffsetRange=function(t,i){return new n.Range(this.context,s(this.context,this,"GetOffsetRange",1,[t,i],!1,!0))},a.prototype.getRow=function(t){return new n.Range(this.context,s(this.context,this,"GetRow",1,[t],!1,!0))},a.prototype.getUsedRange=function(){return new n.Range(this.context,s(this.context,this,"GetUsedRange",1,[],!1,!0))},a.prototype.insert=function(t){return new n.Range(this.context,s(this.context,this,"Insert",0,[t],!1,!0))},a.prototype.select=function(){c(this.context,this,"Select",1,[])},a.prototype._KeepReference=function(){c(this.context,this,"_KeepReference",1,[])},a.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Address)||(this.m_address=t.Address);i(t.AddressLocal)||(this.m_addressLocal=t.AddressLocal);i(t.CellCount)||(this.m_cellCount=t.CellCount);i(t.ColumnCount)||(this.m_columnCount=t.ColumnCount);i(t.ColumnIndex)||(this.m_columnIndex=t.ColumnIndex);i(t.Formulas)||(this.m_formulas=t.Formulas);i(t.FormulasLocal)||(this.m_formulasLocal=t.FormulasLocal);i(t.NumberFormat)||(this.m_numberFormat=t.NumberFormat);i(t.RowCount)||(this.m_rowCount=t.RowCount);i(t.RowIndex)||(this.m_rowIndex=t.RowIndex);i(t.Text)||(this.m_text=t.Text);i(t.ValueTypes)||(this.m_valueTypes=t.ValueTypes);i(t.Values)||(this.m_values=t.Values);i(t._ReferenceId)||(this.m__ReferenceId=t._ReferenceId);h(this,t,["format","Format","worksheet","Worksheet"])}},a.prototype.load=function(n){return e(this,n),this},a.prototype._initReferenceId=function(n){this.m__ReferenceId=n},a}(OfficeExtension.ClientObject);n.Range=p;d=function(i){function r(){i.apply(this,arguments)}return __extends(r,i),Object.defineProperty(r.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),r.prototype.getItem=function(t){return new n.NamedItem(this.context,l(this.context,this,[t]))},r.prototype._handleResult=function(t){var r,f,i,e;if(!u(t)&&(r=t,o(this,r),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],f=r[OfficeExtension.Constants.items],i=0;i<f.length;i++)e=new n.NamedItem(this.context,a(this.context,this,f[i])),e._handleResult(f[i]),this.m__items.push(e)},r.prototype.load=function(n){return e(this,n),this},r}(OfficeExtension.ClientObject);n.NamedItemCollection=d;g=function(f){function h(){f.apply(this,arguments)}return __extends(h,f),Object.defineProperty(h.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"type",{get:function(){return t("type",this.m_type),this.m_type},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"value",{get:function(){return t("value",this.m_value),this.m_value},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"visible",{get:function(){return t("visible",this.m_visible),this.m_visible},set:function(n){this.m_visible=n;r(this.context,this,"Visible",n)},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"_Id",{get:function(){return t("_Id",this.m__Id),this.m__Id},enumerable:!0,configurable:!0}),h.prototype.getRange=function(){return new n.Range(this.context,s(this.context,this,"GetRange",1,[],!1,!0))},h.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Name)||(this.m_name=t.Name);i(t.Type)||(this.m_type=t.Type);i(t.Value)||(this.m_value=t.Value);i(t.Visible)||(this.m_visible=t.Visible);i(t._Id)||(this.m__Id=t._Id)}},h.prototype.load=function(n){return e(this,n),this},h}(OfficeExtension.ClientObject);n.NamedItem=g;nt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"id",{get:function(){return t("id",this.m_id),this.m_id},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"type",{get:function(){return t("type",this.m_type),this.m_type},enumerable:!0,configurable:!0}),f.prototype.getRange=function(){return new n.Range(this.context,s(this.context,this,"GetRange",1,[],!1,!1))},f.prototype.getTable=function(){return new n.Table(this.context,s(this.context,this,"GetTable",1,[],!1,!1))},f.prototype.getText=function(){var t=c(this.context,this,"GetText",1,[]),n=new OfficeExtension.ClientResult;return y(this,t,n),n},f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Id)||(this.m_id=t.Id);i(t.Type)||(this.m_type=t.Type)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.Binding=nt;tt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.getItem=function(t){return new n.Binding(this.context,l(this.context,this,[t]))},f.prototype.getItemAt=function(t){return new n.Binding(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.Binding(this.context,a(this.context,this,e[f])),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.BindingCollection=tt;it=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.add=function(t,i){return new n.Table(this.context,s(this.context,this,"Add",0,[t,i],!1,!0))},f.prototype.getItem=function(t){return new n.Table(this.context,l(this.context,this,[t]))},f.prototype.getItemAt=function(t){return new n.Table(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.Table(this.context,a(this.context,this,e[f])),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.TableCollection=it;rt=function(l){function a(){l.apply(this,arguments)}return __extends(a,l),Object.defineProperty(a.prototype,"columns",{get:function(){return this.m_columns||(this.m_columns=new n.TableColumnCollection(this.context,f(this.context,this,"Columns",!0,!1))),this.m_columns},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"rows",{get:function(){return this.m_rows||(this.m_rows=new n.TableRowCollection(this.context,f(this.context,this,"Rows",!0,!1))),this.m_rows},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"id",{get:function(){return t("id",this.m_id),this.m_id},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"showHeaders",{get:function(){return t("showHeaders",this.m_showHeaders),this.m_showHeaders},set:function(n){this.m_showHeaders=n;r(this.context,this,"ShowHeaders",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"showTotals",{get:function(){return t("showTotals",this.m_showTotals),this.m_showTotals},set:function(n){this.m_showTotals=n;r(this.context,this,"ShowTotals",n)},enumerable:!0,configurable:!0}),Object.defineProperty(a.prototype,"style",{get:function(){return t("style",this.m_style),this.m_style},set:function(n){this.m_style=n;r(this.context,this,"Style",n)},enumerable:!0,configurable:!0}),a.prototype.delete=function(){c(this.context,this,"Delete",0,[])},a.prototype.getDataBodyRange=function(){return new n.Range(this.context,s(this.context,this,"GetDataBodyRange",1,[],!1,!0))},a.prototype.getHeaderRowRange=function(){return new n.Range(this.context,s(this.context,this,"GetHeaderRowRange",1,[],!1,!0))},a.prototype.getRange=function(){return new n.Range(this.context,s(this.context,this,"GetRange",1,[],!1,!0))},a.prototype.getTotalRowRange=function(){return new n.Range(this.context,s(this.context,this,"GetTotalRowRange",1,[],!1,!0))},a.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Id)||(this.m_id=t.Id);i(t.Name)||(this.m_name=t.Name);i(t.ShowHeaders)||(this.m_showHeaders=t.ShowHeaders);i(t.ShowTotals)||(this.m_showTotals=t.ShowTotals);i(t.Style)||(this.m_style=t.Style);h(this,t,["columns","Columns","rows","Rows"])}},a.prototype.load=function(n){return e(this,n),this},a}(OfficeExtension.ClientObject);n.Table=rt;ut=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.add=function(t,i){return new n.TableColumn(this.context,s(this.context,this,"Add",0,[t,i],!1,!0))},f.prototype.getItem=function(t){return new n.TableColumn(this.context,l(this.context,this,[t]))},f.prototype.getItemAt=function(t){return new n.TableColumn(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.TableColumn(this.context,a(this.context,this,e[f])),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.TableColumnCollection=ut;ft=function(f){function h(){f.apply(this,arguments)}return __extends(h,f),Object.defineProperty(h.prototype,"id",{get:function(){return t("id",this.m_id),this.m_id},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"index",{get:function(){return t("index",this.m_index),this.m_index},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"values",{get:function(){return t("values",this.m_values),this.m_values},set:function(n){this.m_values=n;r(this.context,this,"Values",n)},enumerable:!0,configurable:!0}),h.prototype.delete=function(){c(this.context,this,"Delete",0,[])},h.prototype.getDataBodyRange=function(){return new n.Range(this.context,s(this.context,this,"GetDataBodyRange",1,[],!1,!0))},h.prototype.getHeaderRowRange=function(){return new n.Range(this.context,s(this.context,this,"GetHeaderRowRange",1,[],!1,!0))},h.prototype.getRange=function(){return new n.Range(this.context,s(this.context,this,"GetRange",1,[],!1,!0))},h.prototype.getTotalRowRange=function(){return new n.Range(this.context,s(this.context,this,"GetTotalRowRange",1,[],!1,!0))},h.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Id)||(this.m_id=t.Id);i(t.Index)||(this.m_index=t.Index);i(t.Name)||(this.m_name=t.Name);i(t.Values)||(this.m_values=t.Values)}},h.prototype.load=function(n){return e(this,n),this},h}(OfficeExtension.ClientObject);n.TableColumn=ft;et=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.add=function(t,i){return new n.TableRow(this.context,s(this.context,this,"Add",0,[t,i],!1,!0))},f.prototype.getItemAt=function(t){return new n.TableRow(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.TableRow(this.context,v(this.context,this,e[f],f)),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.TableRowCollection=et;ot=function(f){function h(){f.apply(this,arguments)}return __extends(h,f),Object.defineProperty(h.prototype,"index",{get:function(){return t("index",this.m_index),this.m_index},enumerable:!0,configurable:!0}),Object.defineProperty(h.prototype,"values",{get:function(){return t("values",this.m_values),this.m_values},set:function(n){this.m_values=n;r(this.context,this,"Values",n)},enumerable:!0,configurable:!0}),h.prototype.delete=function(){c(this.context,this,"Delete",0,[])},h.prototype.getRange=function(){return new n.Range(this.context,s(this.context,this,"GetRange",1,[],!1,!0))},h.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Index)||(this.m_index=t.Index);i(t.Values)||(this.m_values=t.Values)}},h.prototype.load=function(n){return e(this,n),this},h}(OfficeExtension.ClientObject);n.TableRow=ot;st=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"borders",{get:function(){return this.m_borders||(this.m_borders=new n.RangeBorderCollection(this.context,f(this.context,this,"Borders",!0,!1))),this.m_borders},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.RangeFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.RangeFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"horizontalAlignment",{get:function(){return t("horizontalAlignment",this.m_horizontalAlignment),this.m_horizontalAlignment},set:function(n){this.m_horizontalAlignment=n;r(this.context,this,"HorizontalAlignment",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"verticalAlignment",{get:function(){return t("verticalAlignment",this.m_verticalAlignment),this.m_verticalAlignment},set:function(n){this.m_verticalAlignment=n;r(this.context,this,"VerticalAlignment",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"wrapText",{get:function(){return t("wrapText",this.m_wrapText),this.m_wrapText},set:function(n){this.m_wrapText=n;r(this.context,this,"WrapText",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.HorizontalAlignment)||(this.m_horizontalAlignment=t.HorizontalAlignment);i(t.VerticalAlignment)||(this.m_verticalAlignment=t.VerticalAlignment);i(t.WrapText)||(this.m_wrapText=t.WrapText);h(this,t,["borders","Borders","fill","Fill","font","Font"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.RangeFormat=st;ht=function(n){function f(){n.apply(this,arguments)}return __extends(f,n),Object.defineProperty(f.prototype,"color",{get:function(){return t("color",this.m_color),this.m_color},set:function(n){this.m_color=n;r(this.context,this,"Color",n)},enumerable:!0,configurable:!0}),f.prototype.clear=function(){c(this.context,this,"Clear",0,[])},f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Color)||(this.m_color=t.Color)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.RangeFill=ht;ct=function(n){function f(){n.apply(this,arguments)}return __extends(f,n),Object.defineProperty(f.prototype,"color",{get:function(){return t("color",this.m_color),this.m_color},set:function(n){this.m_color=n;r(this.context,this,"Color",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"sideIndex",{get:function(){return t("sideIndex",this.m_sideIndex),this.m_sideIndex},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"style",{get:function(){return t("style",this.m_style),this.m_style},set:function(n){this.m_style=n;r(this.context,this,"Style",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"weight",{get:function(){return t("weight",this.m_weight),this.m_weight},set:function(n){this.m_weight=n;r(this.context,this,"Weight",n)},enumerable:!0,configurable:!0}),f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Color)||(this.m_color=t.Color);i(t.SideIndex)||(this.m_sideIndex=t.SideIndex);i(t.Style)||(this.m_style=t.Style);i(t.Weight)||(this.m_weight=t.Weight)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.RangeBorder=ct;lt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.getItem=function(t){return new n.RangeBorder(this.context,l(this.context,this,[t]))},f.prototype.getItemAt=function(t){return new n.RangeBorder(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.RangeBorder(this.context,v(this.context,this,e[f],f)),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.RangeBorderCollection=lt;at=function(n){function f(){n.apply(this,arguments)}return __extends(f,n),Object.defineProperty(f.prototype,"bold",{get:function(){return t("bold",this.m_bold),this.m_bold},set:function(n){this.m_bold=n;r(this.context,this,"Bold",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"color",{get:function(){return t("color",this.m_color),this.m_color},set:function(n){this.m_color=n;r(this.context,this,"Color",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"italic",{get:function(){return t("italic",this.m_italic),this.m_italic},set:function(n){this.m_italic=n;r(this.context,this,"Italic",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"size",{get:function(){return t("size",this.m_size),this.m_size},set:function(n){this.m_size=n;r(this.context,this,"Size",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"underline",{get:function(){return t("underline",this.m_underline),this.m_underline},set:function(n){this.m_underline=n;r(this.context,this,"Underline",n)},enumerable:!0,configurable:!0}),f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Bold)||(this.m_bold=t.Bold);i(t.Color)||(this.m_color=t.Color);i(t.Italic)||(this.m_italic=t.Italic);i(t.Name)||(this.m_name=t.Name);i(t.Size)||(this.m_size=t.Size);i(t.Underline)||(this.m_underline=t.Underline)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.RangeFont=at;vt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.add=function(t,i,r){if(!(i instanceof p))throw OfficeExtension.Utility.createRuntimeError(OfficeExtension.ResourceStrings.invalidArgument,"sourceData","Charts.Add");return new n.Chart(this.context,s(this.context,this,"Add",0,[t,i,r],!1,!0))},f.prototype.getItem=function(t){return new n.Chart(this.context,s(this.context,this,"GetItem",1,[t],!1,!1))},f.prototype.getItemAt=function(t){return new n.Chart(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._GetItem=function(t){return new n.Chart(this.context,l(this.context,this,[t]))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.Chart(this.context,a(this.context,this,e[f])),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.ChartCollection=vt;yt=function(s){function l(){s.apply(this,arguments)}return __extends(l,s),Object.defineProperty(l.prototype,"axes",{get:function(){return this.m_axes||(this.m_axes=new n.ChartAxes(this.context,f(this.context,this,"Axes",!1,!1))),this.m_axes},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"dataLabels",{get:function(){return this.m_dataLabels||(this.m_dataLabels=new n.ChartDataLabels(this.context,f(this.context,this,"DataLabels",!1,!1))),this.m_dataLabels},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartAreaFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"legend",{get:function(){return this.m_legend||(this.m_legend=new n.ChartLegend(this.context,f(this.context,this,"Legend",!1,!1))),this.m_legend},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"series",{get:function(){return this.m_series||(this.m_series=new n.ChartSeriesCollection(this.context,f(this.context,this,"Series",!0,!1))),this.m_series},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"title",{get:function(){return this.m_title||(this.m_title=new n.ChartTitle(this.context,f(this.context,this,"Title",!1,!1))),this.m_title},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"height",{get:function(){return t("height",this.m_height),this.m_height},set:function(n){this.m_height=n;r(this.context,this,"Height",n)},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"left",{get:function(){return t("left",this.m_left),this.m_left},set:function(n){this.m_left=n;r(this.context,this,"Left",n)},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"top",{get:function(){return t("top",this.m_top),this.m_top},set:function(n){this.m_top=n;r(this.context,this,"Top",n)},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"width",{get:function(){return t("width",this.m_width),this.m_width},set:function(n){this.m_width=n;r(this.context,this,"Width",n)},enumerable:!0,configurable:!0}),Object.defineProperty(l.prototype,"_Id",{get:function(){return t("_Id",this.m__Id),this.m__Id},enumerable:!0,configurable:!0}),l.prototype.delete=function(){c(this.context,this,"Delete",0,[])},l.prototype.setData=function(n,t){if(!(n instanceof p))throw OfficeExtension.Utility.createRuntimeError(OfficeExtension.ResourceStrings.invalidArgument,"sourceData","Chart.setData");c(this.context,this,"SetData",0,[n,t])},l.prototype.setPosition=function(n,t){c(this.context,this,"SetPosition",0,[n,t])},l.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Height)||(this.m_height=t.Height);i(t.Left)||(this.m_left=t.Left);i(t.Name)||(this.m_name=t.Name);i(t.Top)||(this.m_top=t.Top);i(t.Width)||(this.m_width=t.Width);i(t._Id)||(this.m__Id=t._Id);h(this,t,["axes","Axes","dataLabels","DataLabels","format","Format","legend","Legend","series","Series","title","Title"])}},l.prototype.load=function(n){return e(this,n),this},l}(OfficeExtension.ClientObject);n.Chart=yt;pt=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill","font","Font"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartAreaFormat=pt;wt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.getItemAt=function(t){return new n.ChartSeries(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.ChartSeries(this.context,v(this.context,this,e[f],f)),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.ChartSeriesCollection=wt;bt=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartSeriesFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"points",{get:function(){return this.m_points||(this.m_points=new n.ChartPointsCollection(this.context,f(this.context,this,"Points",!0,!1))),this.m_points},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Name)||(this.m_name=t.Name);h(this,t,["format","Format","points","Points"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartSeries=bt;kt=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"line",{get:function(){return this.m_line||(this.m_line=new n.ChartLineFormat(this.context,f(this.context,this,"Line",!1,!1))),this.m_line},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill","line","Line"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartSeriesFormat=kt;dt=function(r){function f(){r.apply(this,arguments)}return __extends(f,r),Object.defineProperty(f.prototype,"items",{get:function(){return t("items",this.m__items),this.m__items},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"count",{get:function(){return t("count",this.m_count),this.m_count},enumerable:!0,configurable:!0}),f.prototype.getItemAt=function(t){return new n.ChartPoint(this.context,s(this.context,this,"GetItemAt",1,[t],!1,!1))},f.prototype._handleResult=function(t){var r,e,f,s;if(!u(t)&&(r=t,o(this,r),i(r.Count)||(this.m_count=r.Count),!u(r[OfficeExtension.Constants.items])))for(this.m__items=[],e=r[OfficeExtension.Constants.items],f=0;f<e.length;f++)s=new n.ChartPoint(this.context,v(this.context,this,e[f],f)),s._handleResult(e[f]),this.m__items.push(s)},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.ChartPointsCollection=dt;gt=function(r){function s(){r.apply(this,arguments)}return __extends(s,r),Object.defineProperty(s.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartPointFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(s.prototype,"value",{get:function(){return t("value",this.m_value),this.m_value},enumerable:!0,configurable:!0}),s.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Value)||(this.m_value=t.Value);h(this,t,["format","Format"])}},s.prototype.load=function(n){return e(this,n),this},s}(OfficeExtension.ClientObject);n.ChartPoint=gt;ni=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartPointFormat=ni;ti=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"categoryAxis",{get:function(){return this.m_categoryAxis||(this.m_categoryAxis=new n.ChartAxis(this.context,f(this.context,this,"CategoryAxis",!1,!1))),this.m_categoryAxis},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"seriesAxis",{get:function(){return this.m_seriesAxis||(this.m_seriesAxis=new n.ChartAxis(this.context,f(this.context,this,"SeriesAxis",!1,!1))),this.m_seriesAxis},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"valueAxis",{get:function(){return this.m_valueAxis||(this.m_valueAxis=new n.ChartAxis(this.context,f(this.context,this,"ValueAxis",!1,!1))),this.m_valueAxis},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["categoryAxis","CategoryAxis","seriesAxis","SeriesAxis","valueAxis","ValueAxis"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartAxes=ti;ii=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartAxisFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"majorGridlines",{get:function(){return this.m_majorGridlines||(this.m_majorGridlines=new n.ChartGridlines(this.context,f(this.context,this,"MajorGridlines",!1,!1))),this.m_majorGridlines},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"minorGridlines",{get:function(){return this.m_minorGridlines||(this.m_minorGridlines=new n.ChartGridlines(this.context,f(this.context,this,"MinorGridlines",!1,!1))),this.m_minorGridlines},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"title",{get:function(){return this.m_title||(this.m_title=new n.ChartAxisTitle(this.context,f(this.context,this,"Title",!1,!1))),this.m_title},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"majorUnit",{get:function(){return t("majorUnit",this.m_majorUnit),this.m_majorUnit},set:function(n){this.m_majorUnit=n;r(this.context,this,"MajorUnit",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"maximum",{get:function(){return t("maximum",this.m_maximum),this.m_maximum},set:function(n){this.m_maximum=n;r(this.context,this,"Maximum",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"minimum",{get:function(){return t("minimum",this.m_minimum),this.m_minimum},set:function(n){this.m_minimum=n;r(this.context,this,"Minimum",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"minorUnit",{get:function(){return t("minorUnit",this.m_minorUnit),this.m_minorUnit},set:function(n){this.m_minorUnit=n;r(this.context,this,"MinorUnit",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.MajorUnit)||(this.m_majorUnit=t.MajorUnit);i(t.Maximum)||(this.m_maximum=t.Maximum);i(t.Minimum)||(this.m_minimum=t.Minimum);i(t.MinorUnit)||(this.m_minorUnit=t.MinorUnit);h(this,t,["format","Format","majorGridlines","MajorGridlines","minorGridlines","MinorGridlines","title","Title"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartAxis=ii;ri=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"line",{get:function(){return this.m_line||(this.m_line=new n.ChartLineFormat(this.context,f(this.context,this,"Line",!1,!1))),this.m_line},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["font","Font","line","Line"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartAxisFormat=ri;ui=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartAxisTitleFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"text",{get:function(){return t("text",this.m_text),this.m_text},set:function(n){this.m_text=n;r(this.context,this,"Text",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"visible",{get:function(){return t("visible",this.m_visible),this.m_visible},set:function(n){this.m_visible=n;r(this.context,this,"Visible",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Text)||(this.m_text=t.Text);i(t.Visible)||(this.m_visible=t.Visible);h(this,t,["format","Format"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartAxisTitle=ui;fi=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["font","Font"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartAxisTitleFormat=fi;ei=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartDataLabelFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"position",{get:function(){return t("position",this.m_position),this.m_position},set:function(n){this.m_position=n;r(this.context,this,"Position",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"separator",{get:function(){return t("separator",this.m_separator),this.m_separator},set:function(n){this.m_separator=n;r(this.context,this,"Separator",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showBubbleSize",{get:function(){return t("showBubbleSize",this.m_showBubbleSize),this.m_showBubbleSize},set:function(n){this.m_showBubbleSize=n;r(this.context,this,"ShowBubbleSize",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showCategoryName",{get:function(){return t("showCategoryName",this.m_showCategoryName),this.m_showCategoryName},set:function(n){this.m_showCategoryName=n;r(this.context,this,"ShowCategoryName",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showLegendKey",{get:function(){return t("showLegendKey",this.m_showLegendKey),this.m_showLegendKey},set:function(n){this.m_showLegendKey=n;r(this.context,this,"ShowLegendKey",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showPercentage",{get:function(){return t("showPercentage",this.m_showPercentage),this.m_showPercentage},set:function(n){this.m_showPercentage=n;r(this.context,this,"ShowPercentage",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showSeriesName",{get:function(){return t("showSeriesName",this.m_showSeriesName),this.m_showSeriesName},set:function(n){this.m_showSeriesName=n;r(this.context,this,"ShowSeriesName",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"showValue",{get:function(){return t("showValue",this.m_showValue),this.m_showValue},set:function(n){this.m_showValue=n;r(this.context,this,"ShowValue",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Position)||(this.m_position=t.Position);i(t.Separator)||(this.m_separator=t.Separator);i(t.ShowBubbleSize)||(this.m_showBubbleSize=t.ShowBubbleSize);i(t.ShowCategoryName)||(this.m_showCategoryName=t.ShowCategoryName);i(t.ShowLegendKey)||(this.m_showLegendKey=t.ShowLegendKey);i(t.ShowPercentage)||(this.m_showPercentage=t.ShowPercentage);i(t.ShowSeriesName)||(this.m_showSeriesName=t.ShowSeriesName);i(t.ShowValue)||(this.m_showValue=t.ShowValue);h(this,t,["format","Format"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartDataLabels=ei;oi=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill","font","Font"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartDataLabelFormat=oi;si=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartGridlinesFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"visible",{get:function(){return t("visible",this.m_visible),this.m_visible},set:function(n){this.m_visible=n;r(this.context,this,"Visible",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Visible)||(this.m_visible=t.Visible);h(this,t,["format","Format"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartGridlines=si;hi=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"line",{get:function(){return this.m_line||(this.m_line=new n.ChartLineFormat(this.context,f(this.context,this,"Line",!1,!1))),this.m_line},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["line","Line"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartGridlinesFormat=hi;ci=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartLegendFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"overlay",{get:function(){return t("overlay",this.m_overlay),this.m_overlay},set:function(n){this.m_overlay=n;r(this.context,this,"Overlay",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"position",{get:function(){return t("position",this.m_position),this.m_position},set:function(n){this.m_position=n;r(this.context,this,"Position",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"visible",{get:function(){return t("visible",this.m_visible),this.m_visible},set:function(n){this.m_visible=n;r(this.context,this,"Visible",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Overlay)||(this.m_overlay=t.Overlay);i(t.Position)||(this.m_position=t.Position);i(t.Visible)||(this.m_visible=t.Visible);h(this,t,["format","Format"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartLegend=ci;li=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill","font","Font"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartLegendFormat=li;ai=function(s){function c(){s.apply(this,arguments)}return __extends(c,s),Object.defineProperty(c.prototype,"format",{get:function(){return this.m_format||(this.m_format=new n.ChartTitleFormat(this.context,f(this.context,this,"Format",!1,!1))),this.m_format},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"overlay",{get:function(){return t("overlay",this.m_overlay),this.m_overlay},set:function(n){this.m_overlay=n;r(this.context,this,"Overlay",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"text",{get:function(){return t("text",this.m_text),this.m_text},set:function(n){this.m_text=n;r(this.context,this,"Text",n)},enumerable:!0,configurable:!0}),Object.defineProperty(c.prototype,"visible",{get:function(){return t("visible",this.m_visible),this.m_visible},set:function(n){this.m_visible=n;r(this.context,this,"Visible",n)},enumerable:!0,configurable:!0}),c.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Overlay)||(this.m_overlay=t.Overlay);i(t.Text)||(this.m_text=t.Text);i(t.Visible)||(this.m_visible=t.Visible);h(this,t,["format","Format"])}},c.prototype.load=function(n){return e(this,n),this},c}(OfficeExtension.ClientObject);n.ChartTitle=ai;vi=function(t){function i(){t.apply(this,arguments)}return __extends(i,t),Object.defineProperty(i.prototype,"fill",{get:function(){return this.m_fill||(this.m_fill=new n.ChartFill(this.context,f(this.context,this,"Fill",!1,!1))),this.m_fill},enumerable:!0,configurable:!0}),Object.defineProperty(i.prototype,"font",{get:function(){return this.m_font||(this.m_font=new n.ChartFont(this.context,f(this.context,this,"Font",!1,!1))),this.m_font},enumerable:!0,configurable:!0}),i.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);h(this,t,["fill","Fill","font","Font"])}},i.prototype.load=function(n){return e(this,n),this},i}(OfficeExtension.ClientObject);n.ChartTitleFormat=vi;yi=function(n){function t(){n.apply(this,arguments)}return __extends(t,n),t.prototype.clear=function(){c(this.context,this,"Clear",0,[])},t.prototype.setSolidColor=function(n){c(this.context,this,"SetSolidColor",0,[n])},t.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t)}},t.prototype.load=function(n){return e(this,n),this},t}(OfficeExtension.ClientObject);n.ChartFill=yi;pi=function(n){function f(){n.apply(this,arguments)}return __extends(f,n),Object.defineProperty(f.prototype,"color",{get:function(){return t("color",this.m_color),this.m_color},set:function(n){this.m_color=n;r(this.context,this,"Color",n)},enumerable:!0,configurable:!0}),f.prototype.clear=function(){c(this.context,this,"Clear",0,[])},f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Color)||(this.m_color=t.Color)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.ChartLineFormat=pi;wi=function(n){function f(){n.apply(this,arguments)}return __extends(f,n),Object.defineProperty(f.prototype,"bold",{get:function(){return t("bold",this.m_bold),this.m_bold},set:function(n){this.m_bold=n;r(this.context,this,"Bold",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"color",{get:function(){return t("color",this.m_color),this.m_color},set:function(n){this.m_color=n;r(this.context,this,"Color",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"italic",{get:function(){return t("italic",this.m_italic),this.m_italic},set:function(n){this.m_italic=n;r(this.context,this,"Italic",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"name",{get:function(){return t("name",this.m_name),this.m_name},set:function(n){this.m_name=n;r(this.context,this,"Name",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"size",{get:function(){return t("size",this.m_size),this.m_size},set:function(n){this.m_size=n;r(this.context,this,"Size",n)},enumerable:!0,configurable:!0}),Object.defineProperty(f.prototype,"underline",{get:function(){return t("underline",this.m_underline),this.m_underline},set:function(n){this.m_underline=n;r(this.context,this,"Underline",n)},enumerable:!0,configurable:!0}),f.prototype._handleResult=function(n){if(!u(n)){var t=n;o(this,t);i(t.Bold)||(this.m_bold=t.Bold);i(t.Color)||(this.m_color=t.Color);i(t.Italic)||(this.m_italic=t.Italic);i(t.Name)||(this.m_name=t.Name);i(t.Size)||(this.m_size=t.Size);i(t.Underline)||(this.m_underline=t.Underline)}},f.prototype.load=function(n){return e(this,n),this},f}(OfficeExtension.ClientObject);n.ChartFont=wi,function(n){n.range="Range";n.table="Table";n.text="Text"}(ki=n.BindingType||(n.BindingType={})),function(n){n.edgeTop="EdgeTop";n.edgeBottom="EdgeBottom";n.edgeLeft="EdgeLeft";n.edgeRight="EdgeRight";n.insideVertical="InsideVertical";n.insideHorizontal="InsideHorizontal";n.diagonalDown="DiagonalDown";n.diagonalUp="DiagonalUp"}(di=n.BorderIndex||(n.BorderIndex={})),function(n){n.none="None";n.continuous="Continuous";n.dash="Dash";n.dashDot="DashDot";n.dashDotDot="DashDotDot";n.dot="Dot";n.double="Double";n.slantDashDot="SlantDashDot"}(gi=n.BorderLineStyle||(n.BorderLineStyle={})),function(n){n.hairline="Hairline";n.thin="Thin";n.medium="Medium";n.thick="Thick"}(nr=n.BorderWeight||(n.BorderWeight={})),function(n){n.automatic="Automatic";n.automaticExceptTables="AutomaticExceptTables";n.manual="Manual"}(tr=n.CalculationMode||(n.CalculationMode={})),function(n){n.recalculate="Recalculate";n.full="Full";n.fullRebuild="FullRebuild"}(ir=n.CalculationType||(n.CalculationType={})),function(n){n.all="All";n.formats="Formats";n.contents="Contents"}(rr=n.ClearApplyTo||(n.ClearApplyTo={})),function(n){n.invalid="Invalid";n.none="None";n.center="Center";n.insideEnd="InsideEnd";n.insideBase="InsideBase";n.outsideEnd="OutsideEnd";n.left="Left";n.right="Right";n.top="Top";n.bottom="Bottom";n.bestFit="BestFit";n.callout="Callout"}(ur=n.ChartDataLabelPosition||(n.ChartDataLabelPosition={})),function(n){n.invalid="Invalid";n.top="Top";n.bottom="Bottom";n.left="Left";n.right="Right";n.corner="Corner";n.custom="Custom"}(fr=n.ChartLegendPosition||(n.ChartLegendPosition={})),function(n){n.auto="Auto";n.columns="Columns";n.rows="Rows"}(er=n.ChartSeriesBy||(n.ChartSeriesBy={})),function(n){n.invalid="Invalid";n.columnClustered="ColumnClustered";n.columnStacked="ColumnStacked";n.columnStacked100="ColumnStacked100";n._3DColumnClustered="3DColumnClustered";n._3DColumnStacked="3DColumnStacked";n._3DColumnStacked100="3DColumnStacked100";n.barClustered="BarClustered";n.barStacked="BarStacked";n.barStacked100="BarStacked100";n._3DBarClustered="3DBarClustered";n._3DBarStacked="3DBarStacked";n._3DBarStacked100="3DBarStacked100";n.lineStacked="LineStacked";n.lineStacked100="LineStacked100";n.lineMarkers="LineMarkers";n.lineMarkersStacked="LineMarkersStacked";n.lineMarkersStacked100="LineMarkersStacked100";n.pieOfPie="PieOfPie";n.pieExploded="PieExploded";n._3DPieExploded="3DPieExploded";n.barOfPie="BarOfPie";n.xyscatterSmooth="XYScatterSmooth";n.xyscatterSmoothNoMarkers="XYScatterSmoothNoMarkers";n.xyscatterLines="XYScatterLines";n.xyscatterLinesNoMarkers="XYScatterLinesNoMarkers";n.areaStacked="AreaStacked";n.areaStacked100="AreaStacked100";n._3DAreaStacked="3DAreaStacked";n._3DAreaStacked100="3DAreaStacked100";n.doughnutExploded="DoughnutExploded";n.radarMarkers="RadarMarkers";n.radarFilled="RadarFilled";n.surface="Surface";n.surfaceWireframe="SurfaceWireframe";n.surfaceTopView="SurfaceTopView";n.surfaceTopViewWireframe="SurfaceTopViewWireframe";n.bubble="Bubble";n.bubble3DEffect="Bubble3DEffect";n.stockHLC="StockHLC";n.stockOHLC="StockOHLC";n.stockVHLC="StockVHLC";n.stockVOHLC="StockVOHLC";n.cylinderColClustered="CylinderColClustered";n.cylinderColStacked="CylinderColStacked";n.cylinderColStacked100="CylinderColStacked100";n.cylinderBarClustered="CylinderBarClustered";n.cylinderBarStacked="CylinderBarStacked";n.cylinderBarStacked100="CylinderBarStacked100";n.cylinderCol="CylinderCol";n.coneColClustered="ConeColClustered";n.coneColStacked="ConeColStacked";n.coneColStacked100="ConeColStacked100";n.coneBarClustered="ConeBarClustered";n.coneBarStacked="ConeBarStacked";n.coneBarStacked100="ConeBarStacked100";n.coneCol="ConeCol";n.pyramidColClustered="PyramidColClustered";n.pyramidColStacked="PyramidColStacked";n.pyramidColStacked100="PyramidColStacked100";n.pyramidBarClustered="PyramidBarClustered";n.pyramidBarStacked="PyramidBarStacked";n.pyramidBarStacked100="PyramidBarStacked100";n.pyramidCol="PyramidCol";n._3DColumn="3DColumn";n.line="Line";n._3DLine="3DLine";n._3DPie="3DPie";n.pie="Pie";n.xyscatter="XYScatter";n._3DArea="3DArea";n.area="Area";n.doughnut="Doughnut";n.radar="Radar"}(or=n.ChartType||(n.ChartType={})),function(n){n.none="None";n.single="Single"}(sr=n.ChartUnderlineStyle||(n.ChartUnderlineStyle={})),function(n){n.up="Up";n.left="Left"}(hr=n.DeleteShiftDirection||(n.DeleteShiftDirection={})),function(n){n.general="General";n.left="Left";n.center="Center";n.right="Right";n.fill="Fill";n.justify="Justify";n.centerAcrossSelection="CenterAcrossSelection";n.distributed="Distributed"}(cr=n.HorizontalAlignment||(n.HorizontalAlignment={})),function(n){n.down="Down";n.right="Right"}(lr=n.InsertShiftDirection||(n.InsertShiftDirection={})),function(n){n.string="String";n.integer="Integer";n.double="Double";n.boolean="Boolean";n.range="Range"}(ar=n.NamedItemType||(n.NamedItemType={})),function(n){n.none="None";n.single="Single";n.double="Double";n.singleAccountant="SingleAccountant";n.doubleAccountant="DoubleAccountant"}(vr=n.RangeUnderlineStyle||(n.RangeUnderlineStyle={})),function(n){n.visible="Visible";n.hidden="Hidden";n.veryHidden="VeryHidden"}(yr=n.SheetVisibility||(n.SheetVisibility={})),function(n){n.unknown="Unknown";n.empty="Empty";n.string="String";n.integer="Integer";n.double="Double";n.boolean="Boolean";n.error="Error"}(pr=n.RangeValueType||(n.RangeValueType={})),function(n){n.top="Top";n.center="Center";n.bottom="Bottom";n.justify="Justify";n.distributed="Distributed"}(wr=n.VerticalAlignment||(n.VerticalAlignment={})),function(n){n.accessDenied="AccessDenied";n.generalException="GeneralException";n.insertDeleteConflict="InsertDeleteConflict";n.invalidArgument="InvalidArgument";n.invalidBinding="InvalidBinding";n.invalidOperation="InvalidOperation";n.invalidReference="InvalidReference";n.invalidSelection="InvalidSelection";n.itemAlreadyExists="ItemAlreadyExists";n.itemNotFound="ItemNotFound";n.notImplemented="NotImplemented";n.unsupportedOperation="UnsupportedOperation"}(br=n.ErrorCodes||(n.ErrorCodes={}))}(Excel||(Excel={})),function(n){function i(t){return OfficeExtension.ClientRequestContext._run(function(){return new n.RequestContext},t)}var t=function(t){function i(i){t.call(this,i);this.m_workbook=new n.Workbook(this,OfficeExtension.ObjectPathFactory.createGlobalObjectObjectPath(this));this._rootObject=this.m_workbook}return __extends(i,t),Object.defineProperty(i.prototype,"workbook",{get:function(){return this.m_workbook},enumerable:!0,configurable:!0}),i}(OfficeExtension.ClientRequestContext);n.RequestContext=t;n.run=i}(Excel||(Excel={}))